<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */

namespace api\controllers;

use common\models\AlternateDate;
use yii\filters\VerbFilter;
use common\classes\Error;
use Yii;

/**
 * Alternate Date controller
 */
class AlternateController extends BaseController
{
    public function actionList()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'GetAlternateList', [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionUpdate($date, $weekday)
    {
        if (!is_numeric($weekday) || $weekday < 0 || $weekday > 6)
            return $this->response(Error::STATUS_ALTERNATE_WEEKDAY_ERROR);

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'UpdateAlternate',
            ['SpecialDate' => $date, 'weekday' => intval($weekday),
                'access_token' => $this->accessToken,
                'host'=>$this->clientIP,
                ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDelete($date)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'DeleteAlternate',
            ['SpecialDate' => $date,
                'access_token' => $this->accessToken,
                'host'=>$this->clientIP,
                ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionNew($date, $weekday)
    {
        if (!is_numeric($weekday) || $weekday < 0 || $weekday > 6)
            return $this->response(Error::STATUS_ALTERNATE_WEEKDAY_ERROR);

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'AddAlternate',
            ['SpecialDate' => $date, 'weekday' => intval($weekday),
                'access_token' => $this->accessToken,
                'host'=>$this->clientIP,
                ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}
