<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2017/12/21 11:43
 */

namespace api\controllers;

use common\models\BaseInfo;
use Yii;
use yii\db\ActiveQuery;
use yii\rest\Controller;
use yii\filters\Cors;
use common\classes\Error;
use common\classes\Common;
use common\models\SystemLog;
use yii\filters\auth\QueryParamAuth;

/**
 * Site controller
 */
class BaseController extends Controller
{
    // token expire seconds
    const LOGIN_EXPIRE = 3600;
    // old token expire seconds after refresh
    CONST REFRESH_EXPIRE = 3600;
    public $clientIP='';
    public $postRequest;
    public $data;
    public $isTeacher = false;
    public $serverPri = 0;
    public $mq = null;
    public $accessToken = '';
    public $teacherPri = [
        'device' => ['status', 'brd-mute', 'control'],
        'device-ext' => ['get'],
        'device-class' => ['list', 'get-all-device', 'pri-device'],
        'device->ext' => ['get'],
        'scene' => ['list', 'view', 'icon-list', 'ctrl-init-list'],
        'notification' => ['new', 'list'],
        'system' => ['iot-list'],
        'user' => ['view']
    ];
    public $priMap = [
        'mode' => 'mode',
        'template' => 'template',
        'session' => 'brd',
        'media' => 'brd',
        'system' => 'system',
        'support' => 'system',
        'monitor' => 'system',
        'sub-network' => 'system',
        'network' => 'system',
        'alternate' => 'system',
        'datetime' => 'system',
        'device' => 'device',
        'device-class' => 'device',
        'projector' => 'device',
        'log' => 'log',
        'statistic' => 'log',
        'message' => 'log',
        'user' => 'ic',
        'event' => 'system',
        'ipcall' => 'ipcall',
        'timetable' => 'device',
        'timetable-sync' => 'system',
    ];
    public $priExempt = [
        'user' => [
            'update-mgr', 'view-mgr'
        ]
    ];

    public $logTitle = [
        'username' => null,
        'title1' => null,
        'title2' => null,
        'parameter_detail' => null
    ];
    public $logDetail = [];
    public $logActions = [
        'site' => ['login', 'oauth-login', 'unionid-login', 'logout'],
        'alternate' => ['update', 'new', 'delete'],
        'datetime' => ['update'],
        'device' => ['set-config', 'control', 'delete', 'copy', 'cloud-control', 'reset', 'trouble-clean', 'set-second-projector'],
        'log' => ['clean'],
        'media' => ['update', 'new', 'delete', 'upload', 'move', 'check-file'],
        'message' => ['clean'],
        'mode' => ['new', 'update', 'delete', 'duplicate'],
        'network' => ['update'],
        'projector' => ['new', 'update', 'delete'],
        'session' => ['realtime-update', 'realtime-delete', 'delete', 'update', 'new', 'schedule-new', 'schedule-update', 'schedule-delete', 'playtime-delete', 'playtime-update',
            'ctrl', 'realtime-ctrl', 'bw80x-play-ctrl', 'device-update', 'set-config'],
        'statistic' => ['clean'],
        'sub-network' => ['new', 'update', 'delete'],
        'support' => ['update'],
        'system' => ['new-ca-certificate', 'push-certificate', 'iot-update', 'iot-mode-update', 'reboot', 'shutdown', 'chann-name-set', 'check-file-symbol-set', 'set-app-solution'],
        'template' => ['active', 'new', 'update', 'delete', 'duplicate'],
        'user' => ['update-mgr', 'auth', 'update', 'delete', 'new', 'unionid-bind', 'unionid-unbind', 'wx-unbind', 'cloud-unionid-bind', 'import-ic-info'],
        'monitor' => ['set-dev-offline-monitor-config', 'set-dev-statistics-config', 'set-db-error-config'],
        'timetable-config' => ['set-config', 'section-delete', 'section-new'],
        'event' => ['new', 'delete', 'update'],
        'server' => ['ic_sync', 'timetable_sync'],
        'notification' => ['new', 'handle'],
        'timetable-v1' => ['new', 'update', 'delete', 'replace'],
        'scene' => ['new', 'update', 'delete']
    ];
    public $devicePriActions = [];
    public $sessionPriActions = [
        'session' => ['realtime-list', 'realtime-ctrl', 'realtime-media-list', 'realtime-view', 'realtime-update',
            'realtime-delete', 'bw80x-play-ctrl'],
        'media' => true
    ];
    public $resultCode = Error::STATUS_BLANK;

    public function behaviors()
    {
        $behaviors = parent::behaviors(); // TODO: Change the autogenerated stub
        $behaviors['authenticator'] = [
            'class' => QueryParamAuth::className(),
            'optional' => []
        ];
        $behaviors['corsFilter'] = [
            'class' => Cors::className(),
            'cors' => [
                // restrict access to
                'Origin' => ['*'],
                // restrict access to
                'Access-Control-Request-Method' => ['*'],
                // Allow only POST and PUT methods
                'Access-Control-Request-Headers' => ['*'],
                // Allow only headers 'X-Wsse'
                'Access-Control-Allow-Credentials' => true,
                // Allow OPTIONS caching
                'Access-Control-Max-Age' => 3600,
                // Allow the X-Pagination-Current-Page header to be exposed to the browser.
                'Access-Control-Expose-Headers' => ['X-Pagination-Current-Page'],
            ],
        ];
        return $behaviors;
    }

    public function beforeAction($action)
    {
        $this->clientIP = $this->getClientIp();
        //sign data verify
        date_default_timezone_set(Yii::$app->base->timezone);
        $token = Yii::$app->request->get('access_token');
        if (!empty($token)) {
            $this->accessToken = $token;
        }
        $lang = Yii::$app->request->get('lang', 'zh-CN');
        if (!in_array($lang, Common::LANGUAGE_SUPPORT))
            $lang = 'zh-CN';
        Yii::$app->language = $lang;
        return true;
    }

    public function response($status, $data = null)
    {
        $this->resultCode = $status;
        $ret = [
            'status' => $status,
            'message' => Yii::t('app/error', $status)
        ];
        if (!is_null($data))
            $ret['data'] = $data;
        return $ret;
    }

    public function responseByGoRet($status, $message = null, $data = null)
    {
        $this->resultCode = $status;
        $ret = [
            'status' => $status,
            'message' => Yii::t('app/error', $status)
        ];

        if ($message != null) {
            $ret['message'] = $message;
        }

        if (!is_null($data))
            $ret['data'] = $data;
        return $ret;
    }

    public function actionError()
    {
        $error = Yii::$app->errorHandler->exception;
        return $this->response(Error::STATUS_FAILED, $error);
    }

    public function afterAction($action, $result)
    {
        $ret = parent::afterAction($action, $result); // TODO: Change the autogenerated stub
        header('Cache-Control: no-cache');
        header( 'Access-Control-Allow-Origin: https://'.yii::$app->base->cloudServer);
        return $ret;
    }

    public function getClientIp()
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP']))
            return $_SERVER['HTTP_CLIENT_IP'];
        elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
            return $_SERVER['HTTP_X_FORWARDED_FOR'];
        else
            return $_SERVER['REMOTE_ADDR'];
    }

    public function logChange($model, $key, $value)
    {
        $model->$key = $value;
        $model->save();
    }
}

