<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/23 14:18
 */
namespace api\controllers;
use common\classes\Error;
use common\models\PlaySessions;
use Yii;
use yii\helpers\ArrayHelper;

/**
 * BW80x controller
 */
class Bw80xController extends BaseController
{
    public $uid;
    public $data;
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => ['login','list-real-time-bw80x','list-session-devices','load-play-session', 'edit-session-for-app', 'forbid', 'srshook']
            ]
        ]);
    }

    public function beforeAction($action)
    {
        $this->uid = Yii::$app->getRequest()->post('UserID');
        $session = Yii::$app->getRequest()->post('Session');
        $this->data = Yii::$app->getRequest()->post('DataParam');
        if(!empty($this->data)) {
            $this->data = str_replace("'", '"', $this->data);
            $this->data = json_decode($this->data);
        }
        if($this->action->id != 'login' && $this->action->id != 'srshook'){
            if(empty($session) || strlen($session) < 32) {
                Yii::$app->response->data = json_encode(['ErrCode' => -1, 'ErrMsg' => '参数错误', null]);
                return false;
            }
        }
        return parent::beforeAction($action); // TODO: Change the autogenerated stub
    }

    public function actionLogin(){
        if(is_null($this->uid) || empty($this->data) || empty($this->data->Username) || empty($this->data->Password))
            return ['ErrCode' => 101, 'ErrMsg' => '登录失败', 'Data' => ''];
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player',  'Bw80xLogin', [
            'Uid'=>intval($this->uid),
            'UserName'=>$this->data->Username,
            'Password'=>$this->data->Password
        ]);
        if (!$ret){
            return ['ErrCode' => -1, 'ErrMsg' => '', null];
        }

        return ['ErrCode' =>$ret->errCode, 'ErrMsg' => '登录成功', 'Data' => $ret->data];
    }

    public function actionListRealTimeBw80x(){
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player',  'ListRealTimeBw80x', null);
        if (!$ret){
            return ['ErrCode' => -1, 'ErrMsg' => '', null];
        }
        if($ret->errCode != 0){
            return ['ErrCode' => -1, 'ErrMsg' => '', null];
        }
        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $ret->data];
    }

    public function actionListSessionDevices(){
        if(empty($this->data) || !isset($this->data->SessionID))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player', 'ListDevicesOfBw80x',
            ['sessionID' => intval($this->data->SessionID)]);
        if (!$ret){
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        }
        if($ret->errCode != 0){
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        }

        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $ret->data];
    }

    public function actionLoadPlaySession(){
        if(empty($this->data) || !isset($this->data->SessionID))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player', 'LoadPlaySessionOfBw80x',
            ['sessionID' => intval($this->data->SessionID)]);
        if (!$ret){
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        }
        if($ret->errCode != 0){
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        }

        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $ret->data];
    }

    public function actionEditSessionForApp(){
        if (empty($this->data) || empty($this->data->BaseInfo) || empty( $this->data->BaseInfo->SessionID))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];

        if (property_exists($this->data, 'SessionDevice')) {

            $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player', 'EditSessionForApp',
                ['sessionID' => intval($this->data->BaseInfo->SessionID), 'devices' => $this->data->SessionDevice]);
            if (!$ret) {
                return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
            }
            if ($ret->errCode != 0) {
                return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
            }
        }

        return ['ErrCode' => 0, 'ErrMsg' => '操作成功', null];
    }

    public static $srsAction = array('on_publish', 'on_unpublish', 'on_connect', 'on_close');
    public static $srsApp = array('bw803', 'camera', 'live');
    public function actionSrshook(){
        $body = Yii::$app->request->post();
        $action = $body['action'];
        $app = $body['app'];
        if(!in_array($app, self::$srsApp)){
            echo '1';
            return;
        }

        if($app == 'bw803'){
            echo '0';
            return;
        }

        if(!in_array($action, self::$srsAction)){
            echo '0';
            return;
        }

        echo '0';
        return;
    }

    public function actionForbid(){
        return ['ErrCode' => 0, 'ErrMsg' => '', null];
    }
}
