<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/2 15:34
 */
namespace api\controllers;
use common\classes\Common;
use common\classes\Error;
use common\models\VDevice;
use common\models\VDeviceLog;
use common\models\VDeviceStatistic;
use Yii;

/**
 * Site controller
 */
class LogController extends BaseController
{
    private $startDate, $endDate;
    private $dateRange = [];
    public function beforeAction($action)
    {
        $data = Yii::$app->request->post('data');
        if(!$data || !key_exists('startDate', $data) || strptime($data['startDate'], '%Y-%m-%d') == false) {
            $start = Yii::$app->request->get('startDate');
            if(!$start || strptime($start, '%Y-%m-%d') == false)
                $this->startDate = date("Y-m-d", (time()  - 86400 * 30));
            else
                $this->startDate = $start;

        }
        else
            $this->startDate = $data['startDate'];

        if(!$data || !key_exists('endDate', $data) || strptime($data['endDate'], '%Y-%m-%d') == false) {
            $end = Yii::$app->request->get('endDate');
            if (!$end || strptime($end, '%Y-%m-%d') == false)
                $this->endDate = date("Y-m-d");
            else
                $this->endDate = $end;
        }
        else
            $this->endDate = $data['endDate'];
        $dateTmp = range(strtotime($this->startDate), strtotime($this->endDate), 24*60*60);
        if(!empty($dateTmp)) {
            foreach ($dateTmp as $date) {
                $this->dateRange[] = date("Y-m-d", $date);
            }
        }
        return parent::beforeAction($action); // TODO: Change the autogenerated stub
    }

    public function actionList($end){
        if(!is_numeric($end))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data = Yii::$app->getRequest()->post('data');
        $data['end'] = $end;
        $data['startDate'] = $this->startDate;
        $data['endDate']= $this->endDate;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("log","List",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDownload(){
        $type = Yii::$app->getRequest()->get('type', 'device');
        $event = Yii::$app->getRequest()->get('event');
        $device = Yii::$app->getRequest()->get('device');
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;
        $data["type"] = $type;
        $data["event"] = $event;
        $data["device"] = $device;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("log","Download",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        $filepath = $ret->data->filepath;
        $filename = $ret->data->filename;
        Yii::$app->response->sendFile($filepath, $filename,  ["mimeType" => "text/csv"]) && unlink($filepath);
    }

//
//    public function actionIpcallErr(){
//        $data = Yii::$app->getRequest()->post('data');
//        if(is_null($data)|| !is_numeric($data['limit'])) {
//            $data['limit'] = 200;
//        }
//        if(!is_numeric($data['page'])) {
//            $data['page'] = 0;
//        }
//        if(!is_numeric($data['end']))
//            $data['end'] = 0xffffffff;
//
//        $query = VDeviceLog::find()
//            ->Where(['<=', 'DevLogID', $data['end']])
//            ->andWhere(['Event' => 6])
//            ->andWhere(['EndTime' => null]);
//        if(!empty($data['device']) && is_array($data['device']))
//            $query->andWhere(['DeviceID' => $data['device']]);
//
//        return $this->list($query, $data);
//    }


    /**
     * 开机时长统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionPowerOnInterval(){
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("statistics","PowerOnTimeLength",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * 离线次数统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionOfflineStatistic(){
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("statistics","OfflineTimesCount",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * 开机次数统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionPowerOnStatistic(){
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("statistics","PowerOnTimesCount",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * 能耗统计
     * @return array
     * @throws \yii\db\Exception
     */
    public function actionEnergy(){
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("statistics","EnergyOverview",$data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}