<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */
namespace api\controllers;
use common\classes\Common;
use common\components\Media;
use common\models\MediaFiles;
use common\models\Users;
use common\models\VMediaFiles;
use common\models\VSessionFiles;
use common\models\BaseInfo;
use Yii;
use common\models\UploadFile;
use common\classes\Error;
use yii\db\Exception;
use yii\db\mssql\PDO;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;
use common\components\SendCmdToCmpn;

/**
 * Session controller
 */
class MediaController extends BaseController
{
//    public $checkAction = ['check-file'];
    public $hasFileCheckPri = false;
    public $needFileCheckPri = false;
    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action);
        if (!$ret) {
            return $ret;
        }
        $model = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_UPLOAD_FILES_CONFIG]);
        if (!empty($model) && $model->InfoValue === 1)
            $this->needFileCheckPri = true;
        if(!$this->needFileCheckPri)
            $this->hasFileCheckPri = true;
//        if(in_array($action->id, $this->checkAction) && !$this->hasFileCheckPri) {
//            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
//            return false;
//        }
        return $ret;
    }

    public function getUserBaseInfo() {
        $data = array();
        $data['access_token'] = $this->accessToken;

        $req = SendCmdToCmpn::SendCommonCmdNoID('user', 'BaseInfo', $data);
        if (empty($req) || $req->errCode != Error::STATUS_SUCCESS || empty($req->data)) {
            return array();
        }

        return $req->data;
    }

    public function actionFolderList(){
        $req = SendCmdToCmpn::sendCommonCmdNoID('media','FolderList',[
            'access_token' => $this->accessToken,
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionFolderNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder['FileName'] =  $data['FileName'];
        $folder['access_token'] = $this->accessToken;

        if(isset($data['Parent'])) {
            $folder['Parent'] = $data['Parent'];
        }
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','FolderNew',$folder);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionFolderUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || empty($data['FileName']) || !preg_match('/^[0-9][0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder['FileName'] =  $data['FileName'];
        $folder['access_token'] = $this->accessToken;
        $folder['ID'] = $id - 0;
        if(!isset($data['Parent']))
            $folder['Parent'] = $data['Parent'];

        $req = SendCmdToCmpn::SendCommonCmdNoID('media','FolderUpdate',$folder);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionLite($type, $folder){
        $data = Yii::$app->request->post('data');
        $reqData['Type'] = $type;
        $reqData['Folder'] = $folder == "true";
        $reqData['Sysfolder'] = $data['key']['SysFolder'];
        $reqData['Parent'] = $data['key']['Parent'];
        $reqData['access_token'] = $this->accessToken;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','Lite',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionList($type, $folder){
        $data = Yii::$app->request->post('data');
        $reqData['Type'] = $type;
        $reqData['Folder'] = $folder == "true";
        $reqData['Limit'] = $data['limit'];
        $reqData['Page'] = $data['page'];
        $reqData['Parent'] = $data['key']['Parent'];
        if (isset($data['search'])){
            $reqData['Search'] =$data['search'];
        }
        $reqData['access_token'] = $this->accessToken;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','List',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    /**
     * get file detail info
     * @param $id file id
     * @return array
     */
    public function actionView($id){
        $file = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($file))
            return $this->response(Error::STATUS_MEDIA_FILE_NOT_EXISTS);
        $sessions = VSessionFiles::find()
            ->select(['SessionName'])
            ->where(['FileID' => $file->MediaID])
            ->all();
        return $this->response(Error::STATUS_SUCCESS, [
            'file' => $file,
            'sessions' => $sessions
        ]);
    }

    /**
     * @param $id
     * @return array
     */
    public function actionUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !preg_match('/^[0-9][0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        $reqData['ID'] = $id - 0;
        if(!empty($data['FileName']))
            $reqData['FileName'] = $data['FileName'];
            $this->logTitle['title1'] = $data['FileName'];
         if (isset($data['FilePath'])) {
             $reqData['FilePath'] = $data['FilePath'];
         }
        if(isset($data['Gain'])) {
            if($data['Gain'] > 10)
                $gain = 10;
            else if($data['Gain'] < -10)
                $gain = -10;
            else
                $gain = $data['Gain'];
            $reqData['Gain'] = $gain;
        }
        if(isset($data['Parent'])) {
            $reqData['Parent'] = $data['Parent'];
        }
        $reqData['host'] = $this->clientIP;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','Update',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        return $this->response($req->errCode);
    }

    public function actionNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName'], $data['Gain'], $data['FilePath']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        $reqData['Parent'] = $data['Parent'];
        $reqData['FileName'] = $data['FileName'];
        $reqData['Gain'] = $data['Gain'];
        $reqData['FilePath'] = $data['FilePath'];
        $reqData['host'] = $this->clientIP;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','New',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionDelete($id){
        if(!preg_match('/^[0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $ids = explode(',', $id);
        if(empty($ids))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $mediaid = [];
        foreach ($ids as $id) {
            if(is_numeric($id))
                $mediaid[] = $id;
        }
        if(empty($mediaid))
            return $this->response(Error::STATUS_SUCCESS, $mediaid);
        $ids = implode(',', $mediaid);
        //驳回并删除文件日志
        $data = Yii::$app->getRequest()->post('data');
        $reqData['access_token'] = $this->accessToken;
        if(array_key_exists('reason', $data) && !empty($data['reason']) && $data['reason'] == 'reject') {
            $reqData['reason'] = 'reject';
        }
        $reqData['MediaID'] = $ids;
        $reqData['host'] = $this->clientIP;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','Delete',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionUpload($parent)
    {
        $curUser = $this->getUserBaseInfo();
        if (empty($curUser)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $isSuper = ($curUser->privilege & 0x40000000) > 0 ? true : false;

        if(is_numeric($parent)) {
            $parentFolder = MediaFiles::findOne(['MediaID' => $parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
            if(empty($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);

            if(!$isSuper){
                $tryCnt = 20;
                while ($parentFolder->Owner != $curUser->User_ID) {
                    $parentFolder = MediaFiles::findOne(['MediaID' => $parentFolder->Parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
                    $tryCnt--;
                    if(empty($parentFolder) || is_null($parentFolder->Parent) || $tryCnt <= 0)
                        return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                }
            }
        }
        else if($parent == 'null' && !$isSuper)
            return $this->response(Error::STATUS_FORBID);
        else
            $parent = null;
        $model = new UploadFile();
        $model->multiple = UploadedFile::getInstancesByName('multi-file');
        if(empty($model->multiple)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $files = $model->uploadMultiple($parent);
        $names = [];
        $mids = [];
        $ret = Error::STATUS_FAILED;
        if(!empty($files)) {
            foreach ($files as $key => $value) {
                if($value['error'] != Error::STATUS_SUCCESS) {
                    $ret = $value['error'];
                    continue;
                }
                $names[] = $key;
                $mids[] = $value['media']['MediaID'];
            }
        }
        if(!count($mids)) {
            return $this->response($ret);
        }

        $title1 = implode(",", $names);
        $username = empty($curUser->User_Desc) ? $curUser->User_Name : $curUser->User_Desc;

        $data = [];
        $data['access_token'] = $this->accessToken;
        $ret = SendCmdToCmpn::SendCommonCmdNoID('server','CheckFileSymbolGet',$data);
        $checkFileEnabled = $ret->data->enable;
        if ($checkFileEnabled&&!$isSuper&&($curUser->privilege&0x80)==0){
            if (mb_strlen($title1) > 10)
                Yii::$app->udp->mediaCheckMessage(date('Y-m-d h:i:s'), mb_substr($title1, 0, 8) . "...", $username);
            else
                Yii::$app->udp->mediaCheckMessage(date('Y-m-d h:i:s'), $title1, $username);
        }else {
            if (mb_strlen($title1) > 10)
                Yii::$app->udp->mediaUploadMessage(date('Y-m-d h:i:s'), mb_substr($title1, 0, 8) . "...", $username);
            else
                Yii::$app->udp->mediaUploadMessage(date('Y-m-d h:i:s'), $title1, $username);
        }
        $mid = implode(',', $mids);
        Yii::$app->db->createCommand('update MediaFiles set Owner = '. $curUser->User_ID . ' where MediaID in (' . $mid .');')
            ->execute();
        $reqData['mid'] = $mid;
        $reqData['filenames'] = $names;
        $reqData['host'] = $this->clientIP;
        $reqData['access_token'] = $this->accessToken;
        $ret=SendCmdToCmpn::SendCommonCmdNoID('media','Upload',$reqData);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, ['files' => $files]);
    }

    public function actionMove(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['MediaID']) || !preg_match('/^[0-9][0-9,]*$/', $data['MediaID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        if(isset($data['Parent'])) {
            $parent = MediaFiles::findOne(['MediaID' => $data['Parent']]);
            if(empty($parent))
                return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
            if($parent->VedioCodec != 0 || $parent->AudioCodec != 0)
                return $this->response(Error::STATUS_FILE_TYPE_ERROR);
            $reqData['Parent'] = $data['Parent'];
        } else {
            $data['Parent'] = 'null';
            $reqData['Parent'] = null;
        }

        $files = MediaFiles::find()->select(['FileName'])->where('MediaID in (' . $data['MediaID'] .')')
            ->all();
        $names = [];
        foreach ($files as $file) {
            $names[] = $file->FileName;
        }

      $reqData['MediaID'] = $data['MediaID'];
      $reqData['host'] = $this->clientIP;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','Move',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        return $this->response($req->errCode);

    }

    public function actionCheckFileList(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !isset($data['onlynum']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        $reqData['Limit'] = -1;
        $reqData['Page'] = -1;
        if (isset($data['search'])){
            $reqData['Search'] =$data['search'];
        }
        if (isset($data['limit'])) {
            $reqData['Limit'] = $data['limit'];
        }
        if (isset($data['page'])) {
            $reqData['Page'] = $data['page'];
        }
        if (isset($data['onlynum'])) {
            $reqData['Onlynum'] = $data['onlynum'];
        }
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','CheckFileList',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);

    }

    public function actionCheckFile(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !preg_match('/^[0-9][0-9,]*$/', $data['MediaID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        $mids = explode(',', $data['MediaID']);
        $names = [];
        foreach ($mids as $mid){
            $file = MediaFiles::findOne(['MediaID' => $mid]);
            if(empty($file))
                continue;
            $names[] = $file->FileName;
        }
        $reqData['MediaID'] = $data['MediaID'];
        $reqData['host']=$this->clientIP;
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','CheckFile',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }

        return $this->response($req->errCode);
    }

    public function actionGetSharedFiles () {
        $data = Yii::$app->getRequest()->post('data');
        $reqData['access_token'] = $this->accessToken;
        $reqData['Lite'] = isset($data['lite']);
        $reqData['FileType'] = '';
        if (isset($data['type'])) {
            $reqData['FileType'] = $data['type'];
        }
        $data['Limit'] = -1;
        $data['Page'] = -1;
        if (isset($data['limit'])) {
            $data['Limit'] = $data['limit'];
        }
        if (isset($data['page'])) {
            $data['Limit'] = $data['page'];
        }
        $req = SendCmdToCmpn::SendCommonCmdNoID('media','GetSharedFiles',$reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }
}