<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/04/27 16:37
 */

namespace api\controllers;

use common\models\BroadcastMode;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BrdModeMember;

/**
 * Task Mode controller
 */
class ModeController extends SessionBaseController
{
    /**
     * @return array
     */
    public function actionView($id)
    {
        //获取某一个模式的详细信息
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'ViewMode', $id, [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        if (is_null($ret->data))
            return $this->response(Error::STATUS_MODE_NOT_EXIST);

        $retPusher = Yii::$app->getDb()
            ->createCommand("select DevID as id, Dev_Name as name from DeviceList where DevID like '68%';")
            ->queryAll();

        return $this->response(Error::STATUS_SUCCESS,
            ['mode' => $ret->data->mode, 'member' => $ret->data->member, 'pusher' => $retPusher]);
    }

    /**
     * @return array
     */
    public function actionNew()
    {
        $data = Yii::$app->request->post('data');
        if (is_null($data) || !isset($data['mode']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        //日志
        $this->logTitle['title1'] = $data['mode']['ModeName'];
        if (array_key_exists('Enabled', $data['mode'])) {
            $this->logDetail['ele/enable'] = $data['mode']['Enabled'] ? "enable" : "disable";
        }
        if (array_key_exists('Active', $data['mode'])) {
            $this->logDetail['ele/active'] = $data['mode']['Active'] ? "active" : "inactive";
        }
        if (array_key_exists('Priority', $data['mode'])) {
            $this->logDetail['ele/priority'] = $data['mode']['Priority'];
        }
        if (array_key_exists('TimeStyle', $data['mode'])) {
            if ($data['mode']['TimeStyle'] != 0) {
                $start = preg_split('/ /', $data['mode']['StartTime'])[1];
                $stop = preg_split('/ /', $data['mode']['StopTime'])[1];
                for ($i = 0; $i < 7; $i++) {
                    if (($data['mode']['TimeStyle'] & (1 << $i)) > 0) {
                        $this->logDetail['ele/weekday' . $i] = [
                            'start' => $start,
                            'stop' => $stop
                        ];
                    }
                }
            } else {
                $this->logDetail['ele/specialdate'] = [
                    'start' => $data['mode']['StartTime'],
                    'stop' => $data['mode']['StopTime']
                ];
            }
            $this->logDetail['ele/priority'] = $data['mode']['Priority'];
        }
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        //添加模式
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'AddMode', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        $retPusher = Yii::$app->getDb()
            ->createCommand("select DevID as id, Dev_Name as name from DeviceList where DevID like '68%';")
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS,
            ['mode' => $ret->data->mode, 'member' => $ret->data->member, 'pusher' => $retPusher]);
    }

    /**
     * @param $id mode id
     * @return array
     */
    public function actionUpdate($id)
    {
        $data = Yii::$app->request->post('data');
        if (is_null($data) || !isset($data['mode']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        //更新模式
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'UpdateMode', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        $retPusher = Yii::$app->getDb()
            ->createCommand("select DevID as id, Dev_Name as name from DeviceList where DevID like '68%';")
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS,
            ['mode' => $ret->data->mode, 'member' => $ret->data->member, 'pusher' => $retPusher]);
    }

    /**
     * @param $id mode id
     * @return array
     */
    public function actionDelete($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'DeleteMode', $id, [
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * @param $id BroadcastMode id
     * @param $enable
     */
    public function actionDuplicate($id, $enable)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session' ,'DuplicateMode', ['oldID'=>intval($id),
            'newEnable'=>$enable,
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response($ret->errCode, $ret->data);
    }

    /**
     * get session list for special mode
     */
    public function actionSessionList($id)
    {
        if (!is_numeric($id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data = Yii::$app->getRequest()->post('data');
        $param = !empty($data) ? $data : null;
        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmd('schedule', 'GetTimedListOfMode', $id,
                ['data' => $param, 'access_token' => $this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}
