<?php

namespace api\controllers;

use common\classes\Common;
use common\models\BaseInfo;
use common\models\Users;
use Yii;
use common\classes\Error;
use yii\helpers\ArrayHelper;
use common\components\SendCmdToCmpn;

/**
 * System Monitor controller
 */
class MonitorController extends BaseController
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(
            parent::behaviors(), [
            'authenticator' => [
                'optional' => []
            ]
        ]);
    }

    public function actionGetDevOfflineMonitorConfig()
    {
        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'GetDevOfflineMonitorConfig', [
            'access_token' => $this->accessToken
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionSetDevOfflineMonitorConfig()
    {
        $data = Yii::$app->request->post('data');
        if (is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'SetDevOfflineMonitorConfig', [
            'list' => $data,
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionGetDevStatisticsConfig()
    {
        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'GetDevStatisticsConfig', [
            'access_token' => $this->accessToken
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionSetDevStatisticsConfig()
    {
        $data = Yii::$app->request->post('data');
        if (is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $filter_data = [];
        foreach ($data as $item) {
            if (array_key_exists('time', $item) && array_key_exists('user', $item)) {
                $filter_data[] = $item;
            }
        }
        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'SetDevStatisticsConfig', [
            'list' => $filter_data,
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionGetDbErrorConfig()
    {
        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'GetDbErrorConfig', [
            'access_token' => $this->accessToken
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }

    public function actionSetDbErrorConfig()
    {
        $data = Yii::$app->request->post('data');
        if (empty($data) || !isset($data['backupUser'], $data['syncUser']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $req = SendCmdToCmpn::sendCommonCmdNoID('monitor', 'SetDbErrorConfig', [
            'syncUser' => $data['syncUser'],
            'backupUser' => $data['backupUser'],
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $req->data);
    }
}
