<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2019/6/05 11:31
 */
namespace api\controllers;
use common\classes\Error;
use common\classes\Ponvif;
use common\models\DeviceList;
use Yii;
use yii\base\ErrorException;
use yii\base\Exception;

/**
 * Site controller
 * @property Ponvif $onvif_cli
 * @property DeviceList $device
 */
class PtzController extends BaseController
{
    private $onvif_cli = null;
    private $onvif_profile = 'Profile_1';
    private $device = null;

    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action); // TODO: Change the autogenerated stub
        if($ret) {
            $this->data = Yii::$app->request->post('data');
            if(!$this->data || empty($this->data['id']) || !is_string($this->data['id'])) {
                Yii::$app->response->data = $this->response(Error::STATUS_PARAM_ERROR);
                return false;
            }
            $this->device = DeviceList::findOne(['DevID' => $this->data['id']]);
            if(!$this->device) {
                Yii::$app->response->data = $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
                return false;
            }
            if(empty($this->data['type']) || $this->data['type'] == 'onvif') {
                if (isset($this->data['urlName']) && !empty($this->data['urlName'])) {
                    $url = $this->device->getAttribute($this->data['urlName']);
                } else {
                    if (!empty($this->data['channel'])) {
                        if ($this->data['channel'] == 1) {
                            $url = empty($this->device->RtspUrl)?$this->device->RtspSub:$this->device->RtspUrl;
                        }else if ($this->data['channel'] == 2) {
                            $url = empty($this->device->RtspUrl1)?$this->device->RtspSub1:$this->device->RtspUrl1;
                        }
                    }else {
                        if(empty($this->data['cam']) || $this->data['cam'] == 'main')
                            $url = empty($this->device->RtspUrl) ? $this->device->RtspUrl1 : $this->device->RtspUrl;
                        else
                            $url = empty($this->device->RtspSub) ? $this->device->RtspSub1 : $this->device->RtspSub;
                    }
                }

                if(!$url) {
                    Yii::$app->response->data = $this->response(Error::STATUS_DEVICE_RTSP_EMPTY);
                    return false;
                }
                preg_match('/rtsp:\/\/(\w+):(\w+)@([\w.]+)/i', $url, $matches);
                if(count($matches) != 4) {
                    Yii::$app->response->data = $this->response(Error::STATUS_DEVICE_RTSP_IDENTITY_ERROR);
                    return false;
                }
                try {
                    $this->onvif_cli = new Ponvif();
                    $this->onvif_cli->setUsername($matches[1]);
                    $this->onvif_cli->setPassword($matches[2]);
                    $this->onvif_cli->setIPAddress($matches[3]);
                    if(!$this->onvif_cli->initialize()) {
                        Yii::$app->response->data = $this->response(Error::STATUS_DEVICE_ONVIF_INITIAL_FAILED, $matches);
                        return false;
                    }
                }catch (ErrorException $e) {
                    Yii::$app->response->data = $this->response(Error::STATUS_CAM_ONVIF_INIT_FAILED,$matches);
                    return false;
                }
                try {
                    $profiles = $this->onvif_cli->media_GetProfiles();
                    foreach ($profiles as $pf) {
                        if (!empty($pf['@attributes']) || empty($pf['@attributes']['token']))
                            continue;
                        $this->onvif_profile = $pf['@attributes']['token'];
                        break;
                    }
                }catch (Exception $e) {
                }
            }
        }
        return $ret;
    }

    public function actionContinueMove() {
        if(!$this->onvif_cli)
            return $this->response(Error::STATUS_DEVICE_ONVIF_INITIAL_FAILED);
        if(empty($this->data['x']) || !is_numeric($this->data['x']))
            $x = 0;
        else
            $x = $this->data['x'];
        if(empty($this->data['y']) || !is_numeric($this->data['y']))
            $y = 0;
        else
            $y = $this->data['y'];
        try {
            $this->onvif_cli->ptz_ContinuousMove($this->onvif_profile, $x, $y);
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e) {
            return $this->response(Error::STATUS_FAILED);
        }
    }

    public function actionContinueMoveZoom(){
        if(!$this->onvif_cli)
            return $this->response(Error::STATUS_DEVICE_ONVIF_INITIAL_FAILED);
        if(empty($this->data['z']) || !is_numeric($this->data['z']))
            $z = 0;
        else
            $z = $this->data['z'];
        try {
            $this->onvif_cli->ptz_ContinuousMoveZoom($this->onvif_profile, $z);
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e) {
            return $this->response(Error::STATUS_FAILED);
        }
    }
}

