<?php
/**
 * User: marh(marh@idste.cn)
 * Date: 2019/12/04 10:53
 */

namespace api\controllers;

use common\classes\Common;
use common\models\BaseInfo;
use common\models\PwrDisplaySet;
use common\models\Scene;
use common\models\UploadFile;
use Yii;
use common\classes\Error;
use yii\web\UploadedFile;

/**
 * Scene controller
 */
class SceneController extends BaseController
{
    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action);
        return $ret; // TODO: Change the autogenerated stub
    }

    public function actionList()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('server', 'SceneList', [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionNew()
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('server', 'NewScene', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionUpdate()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (!isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('server', 'UpdateScene', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDelete($id)
    {
        if (!$id)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('server', 'DeleteScene', $id, ['access_token' => $this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * 上传图标
     * @return string data => 图标文件名
     */
    public function actionUploadIcon()
    {
        $model = new uploadFile();
        $model->imageFile = UploadedFile::getInstanceByName('image-file');
        if (empty($model->imageFile)) {
            return $this->response(Error::STATUS_FILE_UPLOAD_FAILED, ['error' => $model->errors]);
        }
        return $model->uploadImage('scene');
    }


    public function actionIconList()
    {
        $sceicon = [];
        $scedir = Common::CUSTOM_STORAGE_DIR . '/image';
        $fdir = dir($scedir);
        while ($f = $fdir->read()) {
            if (!is_dir("$scedir/$f") && ($f != ".") && ($f != "..") && preg_match('/^scene_(.*?)\.png$/', $f))
                $sceicon[] = $f;
        }
        $fdir->close();
        return $this->response(Error::STATUS_SUCCESS, $sceicon);
    }

    public function actionDeleteIcon()
    {
        $data = Yii::$app->getRequest()->post('data');
        try {
            $icon = $data['icon'];
        } catch (\Exception $ex) {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }
        system("rm " . Common::CUSTOM_STORAGE_DIR . "/image/scene_$icon.png", $ret);
        // 置空其他所有相同图标的场景图标
        try {
            Yii::$app->db->createCommand("update Scene set icon = '' where icon = :icon;", [':icon' => $icon])->execute();
        } catch (\Exception $ex) {
        }

        if (!$ret)
            return $this->response(Error::STATUS_SUCCESS, $ret);
        return $this->response(Error::STATUS_FAILED, $ret);
    }

    public function actionCtrlInitList()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('server', 'SceneCtrlListInit', [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } elseif ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}