<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */

namespace api\controllers;

use common\models\AlternateDate;
use common\models\BroadcastMode;
use common\models\DeviceList;
use common\models\Players;
use common\models\PlaySessionTime;
use common\models\SessionDevice;
use common\models\TaskTemplate;
use common\models\VDevice;
use common\models\VPlayerKeys;
use common\models\VPlayingSessions;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VSessionInfo;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BaseInfo;
use common\models\VSessionFiles;
use common\classes\Common;
use common\models\PlaySessions;
use yii\helpers\ArrayHelper;
use yii\helpers\Json;

/**
 * Session controller
 */
class SessionController extends SessionBaseController
{
    public $menualAction = ['manual-list', 'manual-list-lite'];
    public $realtimeAction = ['realtime-list', 'realtime-ctrl', 'realtime-media-list', 'realtime-view', 'realtime-update', 'realtime-delete', 'player-ctrl'];
    public $optionalAction = ['player-status'];

    public function behaviors()
    {
        return ArrayHelper::merge(
            parent::behaviors(), [
            'authenticator' => [
                'optional' => $this->optionalAction
            ]
        ]);
    }

    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action); // TODO: Change the autogenerated stub
        return $ret;
    }

    /**
     * get timed tasks
     */
    public function actionTimedList()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data)) {
            // 当data为空的时候，传递的是一个数组，会导致go处理异常
            $data["invalid"] = 0;
        }
        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmdNoID('schedule', 'GetTimedList',
                ['data' => $data, 'access_token' => $this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * get manual tasks
     */
    public function actionManualList()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data)) {
            // 当data为空的时候，php传递的是array，会导致go处理异常
            $data["invalid"] = 0;
        }

        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmdNoID('session', 'List',
                ['data' => $data,'access_token'=>$this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * get manual tasks
     */
    public function actionManualListLite()
    {
        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmdNoID('session', 'SimpleList',
                [null,'access_token'=>$this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * get realtime task
     */
    public function actionRealtimeList()
    {
        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmdNoID('player', 'RealtimeList', [
                'access_token'=>$this->accessToken
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * ctrl realtime state
     */
    public function actionRealtimeCtrl($playerid, $state)
    {
        $this->logTitle['title1'] = $playerid;
        $this->logTitle['title2'] = $state;

        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmd('player', 'RealtimeCtrl', $playerid, ['state' => $state,
                'access_token'=>$this->accessToken,
                'host'=>$this->clientIP,
                ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * get real time player details
     * @param $playerid
     * @return array
     */
    public function actionRealtimeView($playerid)
    {

        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmd('player', 'ViewRealtime', $playerid, ['access_token'=>$this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        $result = [
            'player' => $ret->data->player,
            'devices' => $ret->data->devices,
            'playtime' => $ret->data->playtime,
            'files' => $ret->data->files,
            'keymap' => $ret->data->keymap
        ];

        $result['pusher']['all'] = Yii::$app->getDb()
            ->createCommand("select DevID as id, Dev_Name as name from DeviceList where DevID like '68%';")
            ->queryAll();
        $result['pusher']['actions'] = SessionDevice::find()->where(['SessionID' => $ret->data->player->SessionID])->andWhere(['like', 'DeviceID', '68%', false])->one()['DeviceID'];

        return $this->response(Error::STATUS_SUCCESS, $result);
    }

    /**
     * get realtime task
     */
    public function actionRealtimeUpdate($playerid)
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || !isset($data['player']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $this->logTitle['title1'] = $playerid;
        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmd('player', 'UpdateRealtime', $playerid, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * delete realtime player
     * @param $id
     * @return array
     */
    public function actionRealtimeDelete($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('player', 'Delete', $id, [
            'access_token'=>$this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * get session detail
     */
    public function actionView($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'View', $id, [
            'access_token'=>$this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        $pusher = array();
        $pusher['all'] = Yii::$app->getDb()
            ->createCommand("select DevID as id, Dev_Name as name from DeviceList where DevID like '68%';")
            ->queryAll();
        $pusher['actions'] = SessionDevice::find()->where(['SessionID' => $id])->andWhere(['like', 'DeviceID', '68%', false])->one()['DeviceID'];
        $data = array(
            'pusher' => $pusher,
            'session' => $ret->data->session,
            'devices' => $ret->data->devices,
            'files' => $ret->data->files,
            'playtime' => $ret->data->playtime,
        );
        return $this->response(Error::STATUS_SUCCESS, $data);
    }

    /**
     * delete session
     */
    public function actionDelete($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'Delete', $id, [
            'access_token'=>$this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * create new playtime
     * @param $id SESSIONID
     * @return array
     */
    public function actionScheduleNew($id)
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || !isset($data['TempFlag']) || !isset($data['PlayTime']) || !isset($data['WeekDay']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'AddSchedule', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * delete schedule id from PlaySessionTime
     * @param $id SESSIONID
     * @param $playtime playtime hh:mm:ss
     * @param $weekday weekday 0-6
     * @return array
     */
    public function actionScheduleDelete($id, $scheduleid)
    {
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $data['ScheduleID'] = intval($scheduleid);
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'DeleteSchedule', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * @param $id
     * @param $scheduleid
     * @return array
     */
    public function actionScheduleUpdate($id, $scheduleid)
    {
        $data = Yii::$app->request->post('data');
        if (is_null($data) || !isset($data['WeekDay']) || empty($data['PlayTime']) || !isset($data['TempFlag']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $data['ScheduleID'] = intval($scheduleid);
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'UpdateSchedule', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionPlaytimeDelete($id, $playtime)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'DeleteTimedSession', $id, ['oldTime' => $playtime,
            'access_token'=>$this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionPlaytimeUpdate($id, $old_playtime, $new_playtime)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('schedule', 'UpdatePlayTime', $id, ['oldTime' => $old_playtime,
            'newTime' => $new_playtime,
            'access_token'=>$this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * update session
     */
    public function actionUpdate($id)
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || is_null($data['session']) ||
            (isset($data['session']['RfDevCtrl']) && (strlen($data['session']['RfDevCtrl']) != 32 || !ctype_xdigit($data['session']['RfDevCtrl']))) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])) ||
            (!empty($data['pusher']['action']) && !preg_match('/^[0-9]+[0-9]$/', $data['pusher']['action'])))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'Update', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * create new session
     */
    public function actionNew()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || is_null($data['session']) ||
            (!empty($data['session']['RfDevCtrl']) && (strlen($data['session']['RfDevCtrl']) != 32 || !ctype_xdigit($data['session']['RfDevCtrl']))) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session', 'New', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * @param $id session id
     * @param $state control state: can be 'start'/'stop'/'pause'
     * @param $schedule schedule id,for manual task ,set zero
     */
    public function actionCtrl($id, $state, $schedule)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'Ctrl', $id, ['schedule' => $schedule, "state" => $state, 'access_token'=>$this->accessToken,'host'=>$this->clientIP,]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->responseByGoRet($ret->errCode, $ret->errMsg);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * return template and model list
     */
    public function actionMenuList()
    {
        $tret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session', 'GetTemplateList', ['access_token'=>$this->accessToken]);
        if (!$tret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($tret->errCode != 0) {
            return $this->response($tret->errCode);
        }

        //获取模式列表
        $mret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'GetModeList', ['access_token'=>$this->accessToken]);
        if (!$mret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($mret->errCode != 0) {
            return $this->response($mret->errCode);
        }

        $wret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('schedule', 'GetWeekday', ['access_token'=>$this->accessToken]);
        if (!$wret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($wret->errCode != 0) {
            return $this->response($wret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, [
            'templates' => [
                'list' => $tret->data->list,
                'active' => $tret->data->active,
            ],
            'modes' => $mret->data,
            'weekday' => $wret->data,
        ]);
    }

    public function actionBw80xPlayCtrl()
    {
        $data = Yii::$app->request->post('data');
        if (empty($data) || empty($data['id']) || !preg_match('/^[0-9][0-9]*[0-9]$/', $data['id'])) {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }

        $data['access_token'] = $this->accessToken;
        $data['host']=$this->clientIP;
        //801分区控制
        if (array_key_exists("brdGrpCtrl", $data)) {
            $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('player', 'Bw801GrpCtrl', $data['id'], $data);
            if (!$ret) {
                return $this->response(Error::STATUS_FAILED);
            } else if ($ret->errCode != 0) {
                return $this->response($ret->errCode);
            }
        } else {
            //其他控制
            $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('player', 'PlayerCmdCtrl', $data['id'], $data);
            if (!$ret) {
                return $this->response(Error::STATUS_FAILED);
            } else if ($ret->errCode == -1) {
                return $this->response($ret->errCode, $ret->data);
            } else if ($ret->errCode != 0) {
                return $this->response($ret->errCode);
            }
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionPlayerStatus()
    {
        $data = Yii::$app->request->post('data');
        if ((!empty($data) && !empty($data['playerid']) && !preg_match('/^[0-9][0-9,]*[0-9]$/', $data['playerid'])))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player', 'RealtimeStatus', [
            'access_token'=>$this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }


    public function actionDeviceUpdate()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !isset($data['sessionId']) ||
            (!array_key_exists('devices', $data) && !array_key_exists('pusher', $data)) ||
            (!empty($data['devices']) && !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])) ||
            (!empty($data['pusher']['action']) && !preg_match('/^[0-9]+[0-9]$/', $data['pusher']['action'])))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('player', 'UpdatePlayerDevice', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionGetConfig()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session', 'GetSessionConfig', [
            'access_token'=>$this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionSetConfig()
    {
        $data = Yii::$app->request->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session', 'SetSessionConfig', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        } else if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->actionGetConfig();
    }
}
