<?php
namespace api\controllers;

use common\classes\Common;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\CallWaiter;
use common\models\System;
use common\models\Users;
use common\models\Users as User;
use common\models\VDevice;
use Workerman\Protocols\Http;
use Yii;
use yii\db\Exception;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use common\classes\Error;
use common\models\Cache;
use yii\helpers\ArrayHelper;
use common\components\SendCmdToCmpn;

/**
 * Site controller
 */
class SiteController extends BaseController
{
    const QRCODE_EXIPRE = 600;
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => ['login', 'logout', 'stat', 'login-qrcode','login-qrcode-check','flag', 'oauth', 'oauth-login', 'refresh', 'pub-info', 'dingding-login', 'dingding-info','idste-care-get-devip',
                ]
            ]
        ]);
    }
    public function actionDingdingInfoUpdate () {
        $data = Yii::$app->request->post('data');
        if(!array_key_exists('CorpId', $data) || !array_key_exists('AppKey', $data) || !array_key_exists('AppSecret', $data)) {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }
        if(empty($data['CorpId'])) {
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_CORPID, null, null);
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_APPKEY, null, null);
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_APPSECRET, null, null);
        }
        else {
            if(empty($data['AppKey']) || empty($data['AppSecret']))
                return $this->response(Error::STATUS_PARAM_ERROR);
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_CORPID, $data['CorpId'], null);
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_APPKEY, $data['AppKey'], null);
            Yii::$app->base->baseSet(Common::BASEINFO_DINGDING_APPSECRET, $data['AppSecret'], null);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }
    public function actionDingdingInfo () {
        Yii::$app->base->baseGet(Common::BASEINFO_DINGDING_CORPID, $des, $val);
        if (!$des)
            return $this->response(Error::STATUS_DINGDING_NOT_INIT);
        $data['CorpId'] = $des;
        return $this->response(Error::STATUS_SUCCESS, $data);
    }

    private function getDingdingToken() {
       $token = Yii::$app->cache->get("dingding.access_token");
       if ($token)
           return $token;
        Yii::$app->base->baseGet(Common::BASEINFO_DINGDING_APPKEY, $key, $val);
        Yii::$app->base->baseGet(Common::BASEINFO_DINGDING_APPSECRET, $secret, $val1);
        $h = new HttpRequest('https://oapi.dingtalk.com/gettoken', [
            'appkey' => $key,
            'appsecret' => $secret
        ]);
        $res = $h->get();
        if ($res && $res->errcode == 0) {
            $token = $res->access_token;
            Yii::$app->cache->set('dingding.access_token', $token, 5400);
            return $token;
        }
        return $this->response(Error::STATUS_DINGDING_ACCESS_TOKEN_GET_FAIL);
    }

    private function getDingdingUserinfo($code) {
        $userid = Yii::$app->cache->get('dingding.userid' . $code);
        $token = $this->getDingdingToken();
        if(!$userid) {
            $h = new HttpRequest('https://oapi.dingtalk.com/user/getuserinfo', [
                'access_token' => $token,
                'code' => $code
            ]);
            $res = $h->get();
            if ($res->errcode == 60020) {
                throw new \yii\base\Exception(Error::STATUS_SERVER_IP_NOT_IN_WHITE_LIST);
            }
            if (!$res  || ($res->errcode != 0 && $res->errcode != 60020)) {
                throw new \yii\base\Exception(Error::STATUS_DINGDING_USERID_GET_FAIL);
            }
            $userid = $res->userid;
            Yii::$app->cache->set('dingding.userid' . $code, $userid, 300);
        }
        $userinfo = Yii::$app->cache->get('dingding.userinfo' . $userid);
        if($userinfo)
            return $userinfo;

        $h = new HttpRequest('https://oapi.dingtalk.com/user/get', [
            'access_token' => $token,
            'userid' => $userid
        ]);
        $res = $h->get();
        if ($res && $res->errcode == 0) {
            if(property_exists($res, 'mobile') && !empty($res->mobile)) {
                Yii::$app->cache->set('dingding.userinfo' . $userid, $res, 7200);
                return $userinfo;
            }
            else {
                throw new \yii\base\Exception(Error::STATUS_DINGDING_USER_MOBILE_GET_FAIL);
            }
        }
        throw new \yii\base\Exception(Error::STATUS_DINGDING_USERINFO_GET_FAIL);
    }

    public function actionDingdingLogin () {
        $data = Yii::$app->request->post('data');
        $code = $data['code'];
        if (!array_key_exists('code', $data) || empty($code)) {
            return $this->response(Error::STATUS_PARAM_ERROR);
        }
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','DingDingLogin',[
            'code'=>$data['code'],
        ]);
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response($ret->errCode,$ret->data);
    }

    /*
     * login handle
     */
    public static function login($login_data){
        $res = $login_data->data->user;
        $token = $login_data->data->token;
        $expire = $login_data->data->expire;

        $loginInfo = [
            'token' => $token,
            'expire' => $expire,
            'datetime' => $login_data->data->datetime,
            'user' => $res,
        ];
        return $loginInfo;
    }
    /**
     * Login action.
     *
     * @return string
     */
    public function actionLogin()
    {
        $data = Yii::$app->request->post('data');
        if(empty($data['source']) || !is_string($data['source']))
            $title = "log/WEB";
        else
            $title = 'log/' . strtoupper($data['source']);
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Login',[
            'username'=>$data['username'],
            'password'=>$data['password'],
            'host'=>$this->clientIP,
            'source'=>$title,
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            return $this->response($err_code);
        }
        return $this->response(Error::STATUS_SUCCESS, self::login($ret));
    }

    /**
     * oauth for other platform
     * @param $token Description in baseinfo with InfomationID=0x158
     * @param $usersn
     * @return array
     */
    public function actionOauth($token, $usersn) {
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Oauth',[
            'token' => $token,
            'usersn' => $usersn,
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            if (isset($ret->data)) {
                return $this->response($err_code,$ret->data);
            }
            return $this->response($err_code);
        }
        $data = $ret->data;
        $credit = $data->credit;
        return $this->response(Error::STATUS_SUCCESS, ['credit' =>$credit, 'expire' => $data->credit_expire]);
    }

    public function actionOauthLogin($credit) {
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','OauthLogin',[
            'credit' => $credit,
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            if (isset($ret->data)) {
                return $this->response($err_code,$ret->data);
            }
            return $this->response($err_code);
        }
        return $this->response($err_code,$ret->data);
    }

    /**
     * Logout action.
     *
     * @return string
     */
    public function actionLogout() {
        $token = Yii::$app->getRequest()->get('access_token');
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Logout',[
            'access_token'=>$token,
            'host'=>$this->clientIP,
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            return $this->response($err_code);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * create new access_token with 30 minutes duration
     * old access_token only valid in 5 minutes
     *
     * @return string
     */

    public function actionRefresh()
    {
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Refresh',[
            'access_token' => $this->accessToken
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            return $this->response($err_code);
        }
        $data = $ret->data;
        $loginInfo = [
            'token' => $data->token,
            'expire' => self::LOGIN_EXPIRE,
            'datetime' => $data->datetime,
            'user' => $data->user,
        ];
        return $this->response(Error::STATUS_SUCCESS, $loginInfo);
    }

    /**
     * get server stat
     */
    public function actionStat(){
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Stat',[
            'access_token' => $this->accessToken,
        ]);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            return $this->response($err_code);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * login with unionid and usrename
     */
    public function actionUnionidLogin(){
        $data = Yii::$app->getRequest()->post("data");
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','UnionIDLogin',$data);
        if ($ret==null){
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
//        $data = Yii::$app->getRequest()->post("data");
//        if(empty($data['unionid']) && empty($data['phone']))
//            return $this->response(Error::STATUS_USER_UNIONID_NOT_BOUND);
//        if (empty($data['username']) && !empty($data['id'])) {
//            $this->logTitle['username'] = $data['phone'];
//            $this->logTitle['title1'] = $data['phone'];
//            $user = User::findOne(['User_ID' => $data['id']]);
//        } else {
//            $this->logTitle['username'] = $data['username'];
//            $this->logTitle['title1'] = $data['username'];
//            $user = User::findByUsername($data['username']);
//        }
//        if(is_null($user))
//            return $this->response(Error::STATUS_USER_NOT_EXISTS);
//        if(!empty($data['unionid'])) {
//            $hash = hash('sha256', Common::UNIONID_MD5_PRESTR . $data['unionid']);
//            if($hash == $user->UnionId) {
//                return $this->response(Error::STATUS_SUCCESS, self::login($user, $this->action->id));
//            }
//        }
//        if(!empty($data['phone']) && !empty($user->Phone)) {
//            $hash = hash('sha256', Common::UNIONID_MD5_PRESTR . Yii::$app->base->uuid . $user->Phone);
//            if($hash == $data['phone']) {
//                return $this->response(Error::STATUS_SUCCESS, self::login($user, $this->action->id));
//            }
//        }
//        return $this->response(Error::STATUS_LOGIN_FAILED, $data);
    }

    /**
     * get server flag
     */
    public function actionFlag()
    {
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','Flag',null);
        if (empty($ret)) {
            return $this->response(Error::STATUS_FAILED);
        }
        $err_code = $ret->errCode;
        if ($err_code != Error::STATUS_SUCCESS) {
            return $this->response($err_code);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionPubInfo() {
        return $this->response(Error::STATUS_SUCCESS, [
            'info' => [
                'title' => '服务器新版本发布!',
                'subTitle' => '「D06.00.09 2018-09-04」',
                'titleIcon' => 'el-icon-info',
                'content' => "版本说明\n\n1.修复Bug\n2.新增使用人数统计曲线\n3.新增投影仪通道切换功能",
                'img' => 'https://timgsa.baidu.com/timg?image&quality=80&size=b9999_10000&sec=1536230296924&di=9b501fc2d5e6e0df7e8dbbadb137aacd&imgtype=0&src=http%3A%2F%2Fatt.bbs.duowan.com%2Fforum%2F201801%2F28%2F2050073n37e33p3empekb9.jpg',
                'more' => [
                    'url' => 'https://www.baidu.com',
                    'label' => '查看更多'
                ]
            ]
        ]);
    }
    /**
     * @return array
     * return 401 if token is invalid.
     */
    public function actionLoginValid(){
        $req = SendCmdToCmpn::sendCommonCmdNoID('site','LoginValid',[
            'access_token' => $this->accessToken,
        ]);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        return $this->response($req->errCode);
    }

    /*iDsteCare get device ip*/
    public function actionIdsteCareGetDevip($name){
        $ret = SendCmdToCmpn::sendCommonCmdNoID('site','IdsteCareGetDevIP',[
            'name' => $name,
        ]);
        if (!$ret) {
            return null;
        }
        return $ret->ip;
    }
}
