<?php

namespace api\controllers;

use common\classes\Common;
use common\classes\MysqlAccess;
use common\models\BaseInfo;
use common\models\DeviceList;
use common\models\VEvent;
use common\models\PwrDisplaySet;
use common\models\System;
use common\models\SystemLog;
use common\models\TaskTemplate;
use common\models\VDeviceLog;
use common\models\UploadFile;
use common\models\Users;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\Cache;
use yii\helpers\ArrayHelper;
use yii\web\UploadedFile;

/**
 * Site controller
 */
class SystemController extends BaseController
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(
            parent::behaviors(), [
            'authenticator' => [
                'optional' => ['iot-list', 'info', 'check-file-symbol-get']
            ]
        ]);
    }

    public function actionPushCertificate()
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "PushCertificate", $data);

        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->responseByGoRet($ret->errCode, $ret->errMsg);
        }
        return $this->responseByGoRet($ret->errCode, $ret->errMsg, $ret->data);
    }

    public function actionIotList()
    {
        $data = [
            'access_token' => $this->accessToken
        ];
        $iots = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "GetRfList", $data);
        if (!$iots) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($iots->errCode != Error::STATUS_SUCCESS) {
            return $this->response($iots->errCode);
        }
        $ret = [];
        $ret['iots'] = $iots->data;
        $mode = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "GetAcMode", $data);
        if (!$mode) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($mode->errCode != Error::STATUS_SUCCESS) {
            return $this->response($mode->errCode);
        }
        $ret['mode'] = $mode->data;
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionIotUpdate()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || !is_array($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $success = [];
        foreach ($data as $iot) {
            if (!isset($iot['PwrID']) || !is_numeric($iot['PwrID']) || $iot['PwrID'] < 0)
                continue;
            if (is_null($iot['Type']) && $iot['PwrID'] != 0 && $iot['PwrID'] != 1) {
                $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "DeleteRfInfo", ["PwrID" => $iot['PwrID'],
                    'access_token' => $this->accessToken,
                    'host' => $this->clientIP,
                    ]);
            } else {
                $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "UpdateRfInfo", array_merge($iot,
                    ['access_token' => $this->accessToken,
                    'host' => $this->clientIP,
                    ]));
            }

            if (!$ret) {
                return $this->response(Error::STATUS_FAILED);
            }
            if ($ret->errCode != Error::STATUS_SUCCESS) {
                return $this->response($ret->errCode);
            }
            $success[] = $ret->data;
        }
        return $this->response(Error::STATUS_SUCCESS, $success);
    }

    public function actionIotModeUpdate()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || !is_array($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $success = [];
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "UpdateAcMode", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        $success[] = [1, 2, 3, 4];

        return $this->response(Error::STATUS_SUCCESS, $success);
    }

    /**
     * system reboot
     * @return array
     */
    public function actionReboot()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "Reboot", [
            'access_token' => $this->accessToken,
            'host' => $this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $ret;
    }

    /**
     * system shutdown
     * @return array
     */
    public function actionShutdown()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "Shutdown", [
            'access_token' => $this->accessToken,
            'host' => $this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $ret;
    }

    /**
     * idste-service restart
     * @return array
     */
    public function actionServiceRestart()
    {
        Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "ServiceRestart", [
            'access_token' => $this->accessToken,
            'host' => $this->clientIP,
        ]);

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionInfo()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "Info", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDbFileInfo()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "DbFileInfo", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDbFileDownload()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "DbFileDownload", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionIconUpload()
    {
        $location = Yii::$app->request->get('location');
        if (empty($location))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = new uploadFile();
        $model->imageFile = UploadedFile::getInstanceByName('image-file');
        if (empty($model->imageFile)) {
            return $this->response(Error::STATUS_FAILED, ['error' => $model->errors]);
        }
        return $model->uploadImage($location);
    }

    public function actionIconReset()
    {
        $data = Yii::$app->request->post('data');
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "IconReset", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionCheckFileSymbolSet()
    {
        $data = Yii::$app->request->post('data');
        if (empty($data) || !key_exists('enable', $data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "CheckFileSymbolSet", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionCheckFileSymbolGet()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "CheckFileSymbolGet", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionChannNameSet()
    {
        $data = Yii::$app->request->post('data');
        if (empty($data) || empty($data['projector']) || empty($data['preview']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "GlobalChannelNameSet", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionChannNameGet()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("device", "GlobalChannelNameGet", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * 获取系统设置-设备管理相关通用配置
     * @return array 配置项
     */
    public function actionDeviceConfigGet()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "DeviceConfigGet", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    /**
     * 设置系统设置-设备管理相关通用配置
     * @return array
     */
    public function actionDeviceConfigSet()
    {
        $data = Yii::$app->request->post('data');
        if (empty($data) || !is_array($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        //配置扫码自动开机功能
        if (array_key_exists('scan-and-poweron', $data)) {
            $data['access_token'] = $this->accessToken;
            $data['host'] = $this->clientIP;
            $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "DeviceConfigSet", $data);
            if (!$ret) {
                return $this->response(Error::STATUS_FAILED);
            }
            if ($ret->errCode != Error::STATUS_SUCCESS) {
                return $this->response($ret->errCode);
            }
            return $this->response(Error::STATUS_SUCCESS, $ret->data);
        }
        return $this->response(Error::STATUS_PARAM_ERROR);
    }

    public function actionSCU()
    {
        if (!Yii::$app->user->identity->cloudLogin)
            return;
        $data = Yii::$app->request->post('data');
        if (empty($data) || empty($data['crt']) || empty($data['key']))
            return;
        return Yii::$app->udp->system($this->id, $this->action->id, $data);
    }

    public function actionGetDingtalkConfig()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "GetDingTalkConfig", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionSetDingtalkConfig()
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "SetDingTalkConfig", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionSetAppSolution()
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "SetAppSolution", $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionGetAppSolution()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID("server", "GetAppSolution", [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }
        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}
