<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/04/27 15:11
 */

namespace api\controllers;

use common\models\PlaySessions;
use common\models\TaskTemplate;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BaseInfo;
use common\classes\Common;

/**
 * Task Template controller
 */
class TemplateController extends SessionBaseController
{
    public function actionView()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !key_exists('idTaskTemplate', $data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $template = TaskTemplate::findOne(['idTaskTemplate' => $data['idTaskTemplate']]);
        if (empty($template))
            return $this->response(Error::STATUS_ACTIVE_TEMPLATE_NOT_SET);
        return $this->response(Error::STATUS_SUCCESS, $template);
    }

    /**
     * @param $id string template's id
     * @return array
     */
    public function actionActive($id, $active)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'ActiveTemplate', $id, ['active' => intval($active),
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, ['active' => $ret->data]);
    }

    /**
     * @return array
     */
    public function actionNew()
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('session', 'NewTemplate', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, ['active' => $ret->data]);
    }

    /**
     * @param $id template id
     * @return array
     */
    public function actionUpdate($id)
    {
        $data = Yii::$app->getRequest()->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'UpdateTemplate', $id, $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, ['active' => $ret->data]);
    }

    /**
     * @param $id template id
     * @return array
     */
    public function actionDelete($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'DeleteTemplate', $id, [
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }


    /**
     * @param $id TaskTemplate id
     */
    public function actionDuplicate($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('session', 'DuplicateTemplate', $id, [
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        return $this->response($ret->errCode, $ret->data);
    }

    /**
     * get session list for special template
     */
    public function actionSessionList($id)
    {
        if (!is_numeric($id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data = Yii::$app->getRequest()->post('data');

        $ret = Yii::$app->sendcmd2cmpn
            ->sendCommonCmd('session', 'GetTaskOfTemplate', $id,
                ['data' => $data, 'access_token' => $this->accessToken]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != Error::STATUS_SUCCESS) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }
}
