<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/08/16 09:23
 */

namespace api\controllers;

use common\classes\Common;
use common\components\SendCmdToCmpn;
use common\models\BaseInfo;
use common\models\SectionTable;
use common\models\TimetableV1;
use Yii;
use common\classes\Error;

/**
 * Timetable controller
 */
class TimetableConfigController extends BaseController
{
    public function actionSectionList()
    {
        $data = Yii::$app->getRequest()->post('data');
        $reqData['access_token'] = $this->accessToken;
        $reqData = array_merge($reqData, $data);
        $req = SendCmdToCmpn::SendCommonCmdNoID('timetableConfig', 'SectionList', $reqData);
        if (!$req) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response($req->errCode, $req->data);

    }

    public function actionSectionDelete()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !array_key_exists('name', $data) || empty($data['zone']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $reqData['access_token'] = $this->accessToken;
        $reqData['host'] = $this->clientIP;
        $reqData = array_merge($reqData, $data);
        $req = SendCmdToCmpn::SendCommonCmdNoID('timetableConfig', 'SectionDelete', $reqData);
        if (empty($req)) {
            return $this->response(Error::STATUS_FAILED);
        }
        if (!array_key_exists('data', $req)) {
            return $this->response($req->errCode);
        }
        return $this->response($req->errCode, $req->data);
    }

    public function actionSectionNew()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || !array_key_exists('name', $data) || empty($data['zone']) || empty($data['start']) || empty($data['end']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if (SectionTable::findOne(['name' => $data['name'], 'zone' => $data['zone']]))
            return $this->response(Error::STATUS_SECTION_EXIST);
        $reqData['access_token'] = $this->accessToken;
        $reqData['host'] = $this->clientIP;
        $reqData = array_merge($reqData, $data);
        $req = SendCmdToCmpn::SendCommonCmdNoID('timetableConfig', 'SectionNew', $reqData);
        if (!$req) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response($req->errCode);
    }

    public function actionGetConfig()
    {
        $reqData['access_token'] = $this->accessToken;
        $req = SendCmdToCmpn::SendCommonCmdNoID('timetableConfig', 'GetConfig', $reqData);
        if (!$req) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response($req->errCode, $req->data);
    }

    public function actionSetConfig()
    {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data) || !array_key_exists('enable', $data) || !is_numeric($data['enable']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $reqData['access_token'] = $this->accessToken;
        $reqData['host'] = $this->clientIP;
        $reqData = array_merge($reqData, $data);
        $req = SendCmdToCmpn::SendCommonCmdNoID('timetableConfig', 'SetConfig', $reqData);
        if (!$req) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($req->errCode != Error::STATUS_SUCCESS) {
            return $this->response($req->errCode);
        }
        return $this->response($req->errCode, $req->data);
    }
}
