<?php

namespace api\controllers;

use common\classes\Common;
use common\classes\HttpRequest;
use common\components\SendCmdToCmpn;
use common\models\BaseInfo;
use common\models\System;
use common\models\Users;
use Yii;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use common\classes\Error;
use common\models\Cache;
use common\models\UserDevices;
use yii\helpers\ArrayHelper;
use common\models\Users as User;
use yii\web\ForbiddenHttpException;
use yii\web\UploadedFile;

/**
 * User controller
 */
class UserController extends BaseController
{
    public $postRequest = ['list', 'update', 'new', 'delete', 'auth', 'update-mgr', 'unionid-bind', 'unionid-unbind'];

    /**
     * get detail infomation for current login user
     */
    public function actionViewMgr()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'ViewLoginUser', [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionUpdateMgr()
    {
        $this->data = Yii:: $app->getRequest()->post('data');
        $this->data = array_merge($this->data, ['access_token' => $this->accessToken]);
        if (!array_key_exists('User_Desc', $this->data) || !array_key_exists('IcCard', $this->data)
            || !array_key_exists('Phone', $this->data) || !array_key_exists('User_Sn', $this->data)
            || !isset($this->data['Pass_Word']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $this->data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'UpdateLoginUser', $this->data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * bind new unionid
     */
    public function actionUnionidBind()
    {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    /**
     * unbind unionid
     */
    public function actionUnionidUnbind()
    {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    public function actionWxUnbind()
    {
        return $this->response(Error::STATUS_API_DEPRECATED);
    }

    public function actionDpmList()
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'DpmList', [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionList()
    {
        $this->data = Yii:: $app->getRequest()->post('data');
        $this->data = array_merge($this->data, ['access_token' => $this->accessToken]);

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'List', $this->data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionAuth()
    {
        $this->data = Yii:: $app->getRequest()->post('data');
        $this->data = array_merge($this->data, ['access_token' => $this->accessToken]);

        $this->data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'AuthDevice', $this->data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionView($id)
    {
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('user', 'View', $id, [
            'access_token' => $this->accessToken
        ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionDelete()
    {
        $this->data = Yii:: $app->getRequest()->post('data');
        $this->data = array_merge($this->data, ['access_token' => $this->accessToken]);

        if (!isset($this->data['User_ID']) || !preg_match('/^[0-9][0-9,]*$/', $this->data['User_ID']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'Delete', ["users" => $this->data['User_ID'],
            'access_token' => $this->accessToken,
            'host'=>$this->clientIP,
            ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionUpdate($id)
    {
        $this->data = Yii:: $app->getRequest()->post('data');

        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);

        $this->data['access_token'] = $this->accessToken;
        $this->data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmd('user', 'Update', $id, $this->data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionNew()
    {
        $this->data = Yii:: $app->getRequest()->post('data');
        $this->data = array_merge($this->data, ['access_token' => $this->accessToken]);

        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);
        $token = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'New',
            ['user' => $this->data['user'],
                'access_token' => $token,
                'host'=>$this->clientIP,
                ]);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    public function actionFind()
    {
        $data = Yii::$app->request->post('data');
        $data['access_token'] = $this->accessToken;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'CloudFindUser', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0) {
            return $this->response($ret->errCode);
        }

        return $this->response(Error::STATUS_SUCCESS, $ret->data);
    }

    //该接口在6.5以上版本没有用，后续确定没有用时，可删除
    public function actionCloudUnionidBind()
    {
        $data = Yii::$app->request->post('data');
        $data['access_token'] = $this->accessToken;
        $data['host'] = $this->clientIP;
        $ret = Yii::$app->sendcmd2cmpn->sendCommonCmdNoID('user', 'UnionIDBind', $data);
        if (!$ret) {
            return $this->response(Error::STATUS_FAILED);
        }
        if ($ret->errCode != 0 && $ret->data) {
            return $this->response($ret->errCode, $ret->data);
        }

        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionImportIcInfo()
    {
        $file = UploadedFile::getInstanceByName('csv-file');
        if ($file) {
            if ($file->type !== 'application/vnd.ms-excel' && $file->type !== 'application/octet-stream')
                return $this->response(Error::STATUS_FILE_TYPE_ERROR);
            $time = time();
            $file->saveAs('/tmp/' . $time . $file->name);
            $filePath = '/tmp/' . $time . $file->name;
            $handle = fopen($filePath, 'r');
            $errFlag = [];

            $dbt = Yii::$app->db->beginTransaction();
            $sql = 'INSERT INTO Users(User_Desc, IcCard, User_Sn, Phone, Dpm_Sn, Dpm_Name, User_Type) VALUES
                        (:User_Desc, :IcCard, :User_Sn, :Phone, :Dpm_Sn, :Dpm_Name, 2)
                        ON DUPLICATE KEY 
                        UPDATE User_Desc=VALUES(User_Desc), IcCard=VALUES(IcCard), User_Sn=VALUES(User_Sn), 
                        Phone=VALUES(Phone), Dpm_Sn=VALUES(Dpm_Sn), Dpm_Name=VALUES(Dpm_Name), User_Type=VALUES(User_Type);';
            $insert = $dbt->db->createCommand($sql);
            $insert->bindParam('User_Desc', $User_Desc);
            $insert->bindParam('IcCard', $IcCard);
            $insert->bindParam('User_Sn', $User_Sn);
            $insert->bindParam('Phone', $Phone);
            $insert->bindParam('Dpm_Sn', $Dpm_Sn);
            $insert->bindParam('Dpm_Name', $Dpm_Name);

            while (($fileOp = fgetcsv($handle, 99999, ',')) !== false) {
                if (count($fileOp) < 3) {
                    continue;
                }
                if (empty($fileOp[0]) || empty($fileOp[1]) || empty($fileOp[2])) {
                    continue;
                }
                $encoding = mb_detect_encoding($fileOp[0], 'UTF-8', true);
                if ($encoding !== 'UTF-8') {
                    return $this->response(Error::STATUS_FILE_ENCODING_ERROR);
                }
                $User_Desc = $fileOp[0];
                $User_Sn = $fileOp[1];
                $IcCard = $fileOp[2];
                $Phone = !empty($fileOp[3]) ? $fileOp[3] : null;
                $Dpm_Sn = !empty($fileOp[4]) ? $fileOp[4] : null;
                $Dpm_Name = !empty($fileOp[5]) ? $fileOp[5] : null;
                try {
                    $insert->execute();
                } catch (\Exception $e) {
                    $errFlag[] = $User_Desc;
                }
            }
            try {
                $dbt->commit();
            } catch (\Exception $e) {
                $errFlag[] = $User_Desc;
            }

            if (empty($errFlag)) {
                $ret=Yii::$app->sendcmd2cmpn->SendCommonCmdNoID('user','ImportIcInfo',
                    ['access_token'=>$this->accessToken,
                    'fileName'=>$file->name,
                        'host'=>$this->clientIP]);
                if (empty($ret)) {
                    return $this->response(Error::STATUS_FAILED);
                }
                if ($ret->errCode != Error::STATUS_SUCCESS) {
                    return $this->response($ret->errCode);
                }
                return $this->response(Error::STATUS_SUCCESS);
            }

            return $this->response(Error::STATUS_USER_IMPORT_FAILED);
        }

        return $this->response(Error::STATUS_FAILED);
    }
}
