<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/4/3 11:58
 */

namespace common\components;

use common\classes\Common;
use common\models\AlternateDate;
use common\models\BaseInfo;
use Yii;

/**
 * called by workerman deamon progress only
 *
 * @property string $uuid
 * @property string $cloudServer
 * @property string $accessToken
 * @property string $secretToken
 * @property string $support
 * @property string $timezone
 * @property string $version
 * @property string $activeTemplateId
 * @property string $weekday
 * @property string $date
 * @property string $baseGet
 * @property string $baseSet
 * @property string $timetableState
 */
class Base extends yii\base\Component
{
    public function getCloudServer(){
        self::baseGet(Common::BASEINFO_CLOUD_DOMAIN, $des, $val);
        if (!empty($des)) {
            $ret = $des;
        }
        else
            $ret = 'www.idste.net';
        return $ret;
    }

    public function getAccessToken(){
        self::baseGet(Common::BASEINFO_CLOUD_ACCESS_TOKEN, $des, $val);
        return $des;
    }

    public function getSecretToken(){
        self::baseGet(Common::BASEINFO_CLOUD_SECRET_TOKEN, $des, $val);
        return $des;
    }

    public function getUuid() {
        self::baseGet(Common::BASEINFO_SCHOOL_UUID, $des, $val);
        if (!empty($des) && strlen($des) >= 32) {
            $ret = $des;
        }
        else
            $ret = null;
        return $ret;
    }

    public function getSupport(){
        self::baseGet(Common::BASEINFO_SUPPORT_STATE, $des, $val);
        if(isset($des))
            return $des;
        BaseInfo::deleteAll(['InfomationID' => Common::BASEINFO_SUPPORT_STATE]);
        return null;
    }

    public function getActiveTemplateId() {
        self::baseGet(Common::BASE_ENABLED_TEMPLATE_ID, $des, $val);
        if(!is_null($val))
            return $val;
        return -1;
    }

    public function setActiveTemplateId($value){
        return self::baseSet(Common::BASE_ENABLED_TEMPLATE_ID, null, $value);
    }

    public function getTimezone(){
        $zone = shell_exec('cat /etc/timezone');
        $zone = empty($zone) ? null : trim($zone);
        if(empty($zone) || !file_exists(Common::ZONEINFO_PATH . $zone)) {
            $zone = 'Asia/Chongqing';
            $this->setTimezone($zone);
        }
        return $zone;
    }

    public function setTimezone($value) {
        if(!file_exists(Common::ZONEINFO_PATH . $value))
            return false;
        system('timedatectl set-timezone ' . $value, $ret);
        return ($ret == 0);
    }

    public function getVersion() {
        self::baseGet(Common::BASEINFO_SERVER_VERSION, $des, $val);
        if(!empty($des))
            return $des;
        return "unknow";
    }

    public function getWeekday(){
        $alterData = AlternateDate::findOne(['SpecialDate' => date('Y-n-j')]);
        if(!empty($alterData))
            $weekday = $alterData->AlteredScheduleDay;
        else{
            $weekday = `date +%u` - 1;
        }
        return $weekday;
    }

    public function getTimetableState(){
        self::baseGet(Common::BASEINFO_ENABLE_TIMETABLE, $des, $val);
        return $val;
    }

    public function setTimetableState($state){
        if($state == 1)
            $on = 1;
        else
            $on = 0;
        $this->baseSet(Common::BASEINFO_ENABLE_TIMETABLE, "EnableTimetable", $on);
    }

    public function getAppSolution(){
        self::baseGet(Common::BASEINFO_APP_SOLUTION, $des, $val);
        return $des;
    }

    public function setAppSolution($value){
        $this->baseSet(Common::BASEINFO_APP_SOLUTION, $value, null);
    }

    public function getDate(){
        return trim(`date +'%Y-%m-%d %T'`);
    }

    public function baseSet($id, $description, $value) {
        $model = BaseInfo::findOne(['InfomationID' => $id]);
        if(empty($model)) {
            $model = new BaseInfo();
            $model->InfomationID = $id;
        }
        $model->Description = $description;
        $model->InfoValue = $value;

        if($model->save()) {
            $cache = ['Description' => $description, 'InfoValue' => $value + 0];
            Yii::$app->cache->set("baseInfo'$id'", $cache, 60);
            return true;
        }
        return false;
    }

    public function baseGet($id, &$description, &$value) {
        $model_cache = Yii::$app->cache->get("baseInfo'$id'");
        if(empty($model_cache)) {
           $model_base = BaseInfo::findOne(['InfomationID' => $id]);
            if(empty($model_base)) {
                $description = NULL;
                $value = NULL;
            }else {
                $description = $model_base->Description;
                $value = $model_base->InfoValue;
                $cache = ['Description' => $model_base->Description, 'InfoValue' => $model_base->InfoValue];
                Yii::$app->cache->set("baseInfo'$id'", $cache, 60);
            }

        }
        else {
            $description = $model_cache['Description'];
            $value = $model_cache['InfoValue'];
        }
    }

    public function __get($name)
    {
        if(in_array($name, get_class_vars(self::className())))
            return $this->$name;
        $method = 'get' . ucfirst($name);
        if(!method_exists($this, $method))
            throwException('method or parameter not exists');
        return $this->$method();
    }


    public function __set($name, $value)
    {
        if(in_array($name, get_class_vars(self::className())))
            return $this->$name;
        $method = 'set' . ucfirst($name);
        if(!method_exists($this, $method))
            throwException('method or parameter not exists');
        return $this->$method($value);
    }
}
