<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/4/3 11:58
 */

namespace common\components;

use common\classes\Error;
use Yii;

class Network extends yii\base\Component
{
    private $cards;
    const ROUTE_FILE_PATH = '/etc/network/if-up.d/idste-route';
    public function __construct(array $config = [])
    {
        parent::__construct($config);
        $this->cards = [];
        if ($handle = opendir('/sys/class/net')) {
            while (false != ($entry = readdir($handle))) {
                if ($entry == 'lo' || $entry == '.' || $entry == '..')
                    continue;
                $this->cards[] = $entry;
            }
        }
    }

    /**
     * get network cards infomation(config and state)
     * @return array
     */
    public function ipView()
    {
        $state = [];
        $config = [];
        foreach ($this->cards as $card) {
            $temp = [
                'mac' => '',
                'address' => '',
                'netmask' => ''
            ];
            $fp = fopen('/sys/class/net/' . $card . '/address', 'r');
            $temp['mac'] = fread($fp, 17);
            fclose($fp);
            do {
                $ip = exec("ifconfig " . $card . " | sed -n '/inet addr:/p' | awk -F'[: ]+' '{print $4}'");
                if(ip2long($ip) === false)
                    break;
                $temp['address'] = $ip;
                $netmask = exec("ifconfig " . $card . " | sed -n '/Mask:/p' | awk -F'[: ]+' '{print $8}'");
                if(ip2long($netmask) === false)
                    break;
                $temp['netmask'] = $netmask;
            } while (false);
            $state[$card] = $temp;
        }
        $output = [];
        exec('cat /etc/network/interfaces', $output);
        $lines = [];
        foreach ($output as $line) {
            $line = strtolower(trim($line));
            if (!empty($line) && !preg_match('/^#/', $line))
                $lines[] = $line;
        }
        foreach ($this->cards as $card) {
            for ($i = 0; $i < count($lines); $i++) {
                if (preg_match("/auto[ ]+$card/", $lines[$i])) {
                    $i++;
                    $temp = [];
                    for ($j = $i; $j < count($lines); $j++) {
                        $eles = preg_split('/[ ]+/', $lines[$j]);
                        if (count($eles) < 2)
                            break;
                        if ($eles[0] == 'iface' && $eles[1] == $card) {
                            if (count($eles) !== 4)
                                break;
                            if ($eles[3] == 'static')
                                $temp['type'] = 'static';
                            else if ($eles[3] == 'dhcp') {
                                $temp['type'] = 'dhcp';
                                break;
                            } else
                                break;
                        } else if (!in_array($eles[0], ['address', 'netmask']))
                            break;
                        if (ip2long($eles[1]) !== false)
                            $temp[$eles[0]] = $eles[1];
                        else
                            continue;
                    }
                    $config[$card] = $temp;
                    break;
                }
            }
        }
        return ['config' => $config, 'state' => $state];
    }

    /**
     * get dns config and state
     * @return array
     */
    private function dnsView()
    {
        $cmd = 'cat /etc/resolvconf/resolv.conf.d/base | grep "nameserver"';
        $ips = [];
        exec($cmd, $output);
        if (!empty($output) && count($output) > 0) {
            foreach ($output as $item) {
                $item = trim($item);
                if (preg_match('/^nameserver/', $item)) {
                    $temp = preg_split('/[ ]+/', $item);
                    if (count($temp) >= 2)
                        $ips = array_merge($ips, array_slice($temp, 1));
                }
            }
        }
        $config = [];
        foreach ($ips as $ip) {
            if (false != ip2long($ip))
                $config[] = $ip;
        }

        if(empty($config)) {
            $cmd = 'cat /etc/network/interfaces | grep "dns-nameservers"';
            $ips = [];
            exec($cmd, $output);
            if (!empty($output) && count($output) > 0) {
                foreach ($output as $item) {
                    $item = trim($item);
                    if (preg_match('/^dns-nameservers/', $item)) {
                        $temp = preg_split('/[ ]+/', $item);
                        if (count($temp) >= 2)
                            $ips = array_merge($ips, array_slice($temp, 1));
                    }
                }
            }
            foreach ($ips as $ip) {
                if (false != ip2long($ip))
                    $config[] = $ip;
            }
        }

        $ips = [];
        $output = null;
        exec('cat /run/resolvconf/resolv.conf', $output);
        if (!empty($output) && count($output) > 0) {
            foreach ($output as $item) {
                $item = trim($item);
                if (preg_match('/^nameserver/', $item)) {
                    $temp = explode(' ', $item);
                    if (count($temp) >= 2)
                        $ips = array_merge($ips, array_slice($temp, 1));
                }
            }
        }
        $state = [];
        foreach ($ips as $ip) {
            if (false != ip2long($ip))
                $state[] = $ip;
        }

        return ['config' => $config, 'state' => $state];
    }

    /**
     * get route config and state
     * @return array
     */
    private function routeView()
    {
        $cmd = 'cat ' . self::ROUTE_FILE_PATH . ' | grep "^route add"';
        exec($cmd, $output);
        $config = [];
        $default = null;
        if (!empty ($output) && is_array($output)) {
            foreach ($output as $item) {
                $item = strtolower(trim($item));
                $arr = preg_split('/[ ]+/', $item);
                $tmp = [
                    'net' => '',
                    'netmask' => '',
                    'type' => 'gw',
                    'gateway' => ''
                ];

                for ($i = 0; $i < count($arr); $i++) {
                    if ($arr[$i] == '-net') {
                        $tmp['net'] = $arr [$i + 1];
                        $i++;
                    } else if ($arr[$i] == 'netmask') {
                        $tmp['netmask'] = $arr [$i + 1];
                        $i++;
                    }  else if ($arr[$i] == 'host') {
                        $tmp['host'] = $arr [$i + 1];
                        $i++;
                    } else if ($arr[$i] == 'gw') {
                        $tmp['type'] = 'gw';
                        $tmp['gateway'] = $arr [$i + 1];
                        $i+=2;
                    } else if ($arr[$i] == 'dev') {
                        $tmp['type'] = 'dev';
                        $tmp['gateway'] = $arr [$i + 1];
                        $i+=2;
                    } else if ($arr[$i] == 'default') {
                        if ($arr[$i + 1] == 'gw') {
                            $default = ['type' => 'gw', 'gateway' => $arr[$i + 2]];
                        } else if ($arr[$i + 1] == 'dev') {
                            $default = ['type' => 'dev', 'gateway' => $arr[$i + 2]];
                        }
                    }
                }
                if (!empty ($tmp ['net']))
                    $config[] = $tmp;
            }
        }

        if(empty($default)) {
            $ret = trim(shell_exec("route -n | grep '^0.0.0.0' | awk '{print $2}'"));
            if(!empty($ret) && ip2long($ret) != false) {
                $default = ['type' => 'gw', 'gateway' => $ret];
            }
        }

        $output = null;
        exec('route -n | tail -n +2', $output);
        $routes = [];
        if (!empty ($output) && is_array($output) && count($output) > 1) {
            $keys = preg_split('/[ ]+/', $output[0]);
            for ($i = 1; $i < count($output); $i++) {
                $arr = preg_split('/[ ]+/', $output[$i]);
                if (count($keys) == count($arr)) {
                    $routes[] = array_combine($keys, $arr);
                }
            }
        }
        if(!file_exists(self::ROUTE_FILE_PATH))
            $this->routeUpdate([], $default);
        return ['config' => $config, 'default' => $default, 'state' => $routes];
    }

    /**
     * get network card/dns nameservers/route details
     * @return string
     */
    private function view()
    {
        $ret['ip'] = $this->ipView();
        $ret['dns'] = $this->dnsView();
        $ret['route'] = $this->routeView();
        return Error::serializeResponse(Error::STATUS_SUCCESS, $ret);
    }

    /**
     * @param $file
     * @param $config
     */
    private function ipUpdate($config)
    {
        exec('sed -n \'/^[ ]*dns-nameserver/p\' /etc/network/interfaces', $dns);
        $file = fopen('/etc/network/interfaces', 'w');
        if(file_exists('/etc/network/interfaces.d'))
            fwrite($file, "source /etc/network/interfaces.d/*\n\n");
        fwrite($file, "auto lo\niface lo inet loopback\n\n");
        foreach ($this->cards as $card) {
            if (empty($config) || empty($config[$card])) {
//                fwrite($file, "iface $card inet dhcp\n\n");
                continue;
            }
            fwrite($file, "auto $card\n");
            $conf = $config[$card];
            if (isset($conf['type']) && $conf['type'] == 'static' &&
                isset($conf['address']) && ip2long($conf['address']) !== false &&
                isset($conf['netmask']) && ip2long($conf['netmask']) !== false
            ) {
                fwrite($file, "iface $card inet static\n");
                fwrite($file, "    address " . $conf['address'] . "\n");
                fwrite($file, "    netmask " . $conf['netmask'] . "\n\n");
                Yii::$app->system->netcardReset($card, $conf['address'], $conf['netmask']);
            }
            else
                fwrite($file, "iface $card inet dhcp\n\n");
        }
        fwrite($file, "\n");
        if(!empty($dns)) {
            foreach ($dns as $value)
                fwrite($file, trim($value) . "\n");
        }
        fclose($file);
    }

    /**
     * @param $file
     * @param $config
     */
    private function dnsUpdate($config)
    {
        if (!is_array($config))
            return;
        shell_exec("sed -i /^\s*dns-nameserver/d /etc/network/interfaces");
        $file = fopen('/etc/resolvconf/resolv.conf.d/base', 'w');
        $dns = [];
        foreach ($config as $item) {
            if (ip2long($item))
                $dns[] = $item;
        }
        $ctx = [];
        foreach ($dns as $ip) {
            $ctx[] = 'nameserver ' . $ip;
        }
        fwrite($file, implode("\n", $ctx));
        fclose($file);
    }

    /**
     * @param $file
     * @param $config
     * @param $default
     */
    private function routeUpdate($config, $default)
    {
        if (!is_array($config))
            return;
        $file = fopen(self::ROUTE_FILE_PATH , 'w');
        fwrite($file, "#!/bin/sh -e\n\n");
        foreach ($config as $item) {
            if(isset($item['type']) && !empty($item['gateway'])) {
                if (isset($item['net']) && ip2long($item['net']) !== false &&
                    isset($item['netmask']) && ip2long($item['netmask']) !== false) {
                    $net = ip2long($item['net']);
                    $netmask = ip2long($item['netmask']);
                    //invalid netmask
                    if ((($netmask | ($netmask << 1)) & 0xffffffff) != $netmask)
                        continue;
                    $net = $net & $netmask;
                    $net = long2ip($net);
                    if ($item['type'] == 'gw' && ip2long($item['gateway']) !== false)
                        fwrite($file, 'route add -net ' . $net . ' netmask ' . $item['netmask'] . ' gw ' . $item['gateway'] . " || :\n");
                    else if ($item['type'] == 'dev' && in_array($item['gateway'], $this->cards))
                        fwrite($file, 'route add -net ' . $net . ' netmask ' . $item['netmask'] . ' dev ' . $item['gateway'] . " || :\n");
                }
                else if(isset($item['host']) && ip2long($item['host']) !== false) {
                    if ($item['type'] == 'gw' && ip2long($item['gateway']) !== false)
                        fwrite($file, 'route add -host ' . $item['host'] . ' gw ' . $item['gateway'] . " || :\n");
                    else if ($item['type'] == 'dev' && in_array($item['gateway'], $this->cards))
                        fwrite($file, 'route add -host ' . $item['host'] . ' dev ' . $item['gateway'] . " || :\n");
                }
            }
        }
        do{
            if(empty($default) || empty($default['gateway']))
                break;
            if ($default['type'] == 'gw' && ip2long($default['gateway']))
                fwrite($file, 'route add default gw ' . $default['gateway'] . " || :\n");
            else if ($default['type'] == 'dev' && in_array($item['gateway'], $this->cards))
                fwrite($file, 'route add default dev ' . $default['gateway'] . " || :\n");
        }while(false);
        fclose($file);
        chmod(self::ROUTE_FILE_PATH, 0755);
    }

    /**
     * @param $param
     * @return string
     */
    private function update($param)
    {
        if (empty($param))
            $param = [];
        if (!empty($param['route']) && is_array($param['route']['config'])) {
            if (!empty($param['route']['default']))
                $this->routeUpdate($param['route']['config'], $param['route']['default']);
            else
                $this->routeUpdate($param['route']['config'], null);
        }

        if (!empty($param['ip'])) {
            if (!empty($param['ip']['config'])) {
                $this->ipUpdate($param['ip']['config']);
            } else {
                $this->ipUpdate(null);
            }
        }
        if (isset($param['dns'])) {
            if(!empty($param['dns']['config']))
                $this->dnsUpdate($param['dns']['config']);
            else
                $this->dnsUpdate(['114.114.114.114', '8.8.8.8']);
        }
        if(isset($param['reboot']) && $param['reboot'] === true) {
            Yii::$app->system->reboot();
            sleep(2);
        }
        else {
            exec('systemctl restart networking');
        }
        return $this->view();
    }

    public function run($action, $param)
    {
        if (posix_getuid() !== 0)
            return Error::serializeResponse(Error::STATUS_FORBID);
        switch ($action) {
            case "view":
                return $this->view();
            case "update":
                return $this->update($param);
            default:
                return Error::serializeResponse(Error::STATUS_FAILED);
        }
    }
}