<?php

namespace common\models;

use api\controllers\BaseController;
use common\classes\Common;
use Yii;

/**
 * This is the model class for table "DeviceList".
 *
 * @property string $DevID
 * @property string $Dev_Name
 * @property integer $Parent
 * @property integer $DisableParentSchedule
 * @property resource $Dev_Description
 * @property resource $Dev_Configuration
 * @property string $RtspUrl
 * @property string $RtspSub
 * @property string $RtspUrl1
 * @property string $RtspSub1
 * @property integer $ProjWatermark
 * @property integer $ProjUsedTime
 * @property string $Dev_ConfigurationV2
 * @property string $PcIp
 * @property integer $PcPort
 * @property string $PcMainPasswd
 * @property string $PcViewPasswd
 * @property integer $RfInvalid
 *
 * @property BrdModeMember[] $brdModeMembers
 * @property BroadcastMode[] $brdModes
 * @property DevCtrlSchedule[] $devCtrlSchedules
 * @property DevClasses $parent
 * @property DisableDevSession[] $disableDevSessions
 * @property PlaySessionTime[] $schedules
 * @property MsgTaskDev[] $msgTaskDevs
 * @property MsgTasks[] $msgTasks
 * @property SessionDevice[] $sessionDevices
 * @property Timetable[] $timetables
 * @property UserDevices[] $userDevices
 * @property Users[] $users
 */
class DeviceList extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'DeviceList';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['DevID'], 'required'],
            [['Parent', 'DisableParentSchedule', 'ProjWatermark', 'ProjUsedTime', 'PcPort', 'RfInvalid'], 'integer'],
            [['DevID'], 'string', 'max' => 50],
            [['Dev_Name', 'PcMainPasswd', 'PcViewPasswd'], 'string', 'max' => 32],
            [['Dev_Description'], 'string', 'max' => 96],
            [['Dev_Configuration'], 'string', 'max' => 2048],
            [['RtspUrl', 'RtspSub', 'RtspUrl1', 'RtspSub1'], 'string', 'max' => 512],
            [['Dev_ConfigurationV2'], 'string', 'max' => 16384],
            [['PcIp'], 'string', 'max' => 41],
            [['DevID'], 'unique'],
            [['Parent'], 'exist', 'skipOnError' => true, 'targetClass' => DevClasses::className(), 'targetAttribute' => ['Parent' => 'ClassID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'DevID' => 'Dev ID',
            'Dev_Name' => 'Dev  Name',
            'Parent' => 'Parent',
            'DisableParentSchedule' => 'Disable Parent Schedule',
            'Dev_Description' => 'Dev  Description',
            'Dev_Configuration' => 'Dev  Configuration',
            'RtspUrl' => 'Rtsp Url',
            'RtspSub' => 'Rtsp Sub',
            'RtspUrl1' => 'Rtsp Url1',
            'RtspSub1' => 'Rtsp Sub1',
            'ProjWatermark' => 'Proj Watermark',
            'ProjUsedTime' => 'Proj Used Time',
            'Dev_ConfigurationV2' => 'Dev  Configuration V2',
            'PcIp' => 'Pc Ip',
            'PcPort' => 'Pc Port',
            'PcMainPasswd' => 'Pc Main Passwd',
            'PcViewPasswd' => 'Pc View Passwd',
            'RfInvalid' => 'Rf Invalid',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrdModeMembers()
    {
        return $this->hasMany(BrdModeMember::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrdModes()
    {
        return $this->hasMany(BroadcastMode::className(), ['idBroadcastMode' => 'BrdModeID'])->viaTable('BrdModeMember', ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDevCtrlSchedules()
    {
        return $this->hasMany(DevCtrlSchedule::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getParent()
    {
        return $this->hasOne(DevClasses::className(), ['ClassID' => 'Parent']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDisableDevSessions()
    {
        return $this->hasMany(DisableDevSession::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSchedules()
    {
        return $this->hasMany(PlaySessionTime::className(), ['ScheduleID' => 'ScheduleID'])->viaTable('DisableDevSession', ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMsgTaskDevs()
    {
        return $this->hasMany(MsgTaskDev::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMsgTasks()
    {
        return $this->hasMany(MsgTasks::className(), ['MsgTaskID' => 'MsgTaskID'])->viaTable('MsgTaskDev', ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSessionDevices()
    {
        return $this->hasMany(SessionDevice::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTimetables()
    {
        return $this->hasMany(Timetable::className(), ['DevId' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserDevices()
    {
        return $this->hasMany(UserDevices::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsers()
    {
        return $this->hasMany(Users::className(), ['User_ID' => 'UserID'])->viaTable('UserDevices', ['DevID' => 'DevID']);
    }

    static function getDevTreeArray(){
        $list = Yii::$app->cache->get(Common::DEVICE_TREE_SALT);
        if(!empty($list))
            return $list;
        $classList = DevClasses::find()->select(['ClassID', 'Parent'])
            ->asArray()
            ->all();
        $classes = [];
        foreach ($classList as $item) {
            if(is_null($item['Parent']))
                $item['Parent'] = 0;
            $classes[$item['ClassID']] = $item['Parent'];
        }
        $devList = DeviceList::find()->select(['DevID', 'Parent'])
            ->asArray()
            ->all();
        $list = [];
        foreach ($devList as $item) {
            if(is_null($item['Parent']))
                $item['Parent'] = 0;
            if(empty($list[$item['Parent']]))
                $list[$item['Parent']] = [$item['DevID']];
            else
                $list[$item['Parent']][] = $item['DevID'];
        }

        foreach ($list as $parent => $ids) {
            while(1) {
                if ($parent == 0)
                    break;
                if (!array_key_exists($parent, $classes))
                    break;
                $parent = $classes[$parent];
                if (empty($list[$parent]))
                    $list[$parent] = $ids;
                else
                    $list[$parent] = array_merge($list[$parent], $ids);
            }
        }
        Yii::$app->cache->set(Common::DEVICE_TREE_SALT, $list, 3600);
        return $list;
    }
}
