<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "MediaFiles".
 *
 * @property integer $MediaID
 * @property string $FileName
 * @property integer $FileSize
 * @property integer $TotalTime
 * @property integer $FileFmt
 * @property integer $BitRate
 * @property integer $VedioCodec
 * @property integer $Height
 * @property integer $Width
 * @property integer $FrameRate
 * @property integer $AudioCodec
 * @property integer $Channels
 * @property integer $SampleRate
 * @property integer $Parent
 * @property integer $Gain
 * @property string $CreateTime
 * @property string $FilePath
 * @property integer $Owner
 * @property integer $Sysfolder
 * @property integer $Check
 *
 * @property MediaFiles $parent
 * @property MediaFiles[] $mediaFiles
 * @property Users $owner
 * @property PlayListFiles[] $playListFiles
 */
class MediaFiles extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'MediaFiles';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['FileSize', 'TotalTime', 'FileFmt', 'BitRate', 'VedioCodec', 'Height', 'Width', 'FrameRate', 'AudioCodec', 'Channels', 'SampleRate', 'Parent', 'Gain', 'Owner', 'Sysfolder', 'Check'], 'integer'],
            [['FileSize', 'TotalTime', 'FileFmt', 'BitRate', 'VedioCodec', 'Height', 'Width', 'FrameRate', 'AudioCodec', 'Channels', 'SampleRate'], 'default', 'value' => 0],
            [['Parent'], 'default', 'value' => null],
            [['CreateTime'], 'default', 'value' => date('Y-m-d H:i:s')],
            [['Sysfolder'], 'default', 'value' => null],
            [['CreateTime'], 'safe'],
            [['FileName'], 'string', 'max' => 64],
            [['FilePath'], 'string', 'max' => 256],
            [['Parent'], 'exist', 'skipOnError' => true, 'targetClass' => MediaFiles::className(), 'targetAttribute' => ['Parent' => 'MediaID']],
            [['Owner'], 'exist', 'skipOnError' => true, 'targetClass' => Users::className(), 'targetAttribute' => ['Owner' => 'User_ID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'MediaID' => 'Media ID',
            'FileName' => 'File Name',
            'FileSize' => 'File Size',
            'TotalTime' => 'Total Time',
            'FileFmt' => 'File Fmt',
            'BitRate' => 'Bit Rate',
            'VedioCodec' => 'Vedio Codec',
            'Height' => 'Height',
            'Width' => 'Width',
            'FrameRate' => 'Frame Rate',
            'AudioCodec' => 'Audio Codec',
            'Channels' => 'Channels',
            'SampleRate' => 'Sample Rate',
            'Parent' => 'Parent',
            'Gain' => 'Gain',
            'CreateTime' => 'Create Time',
            'FilePath' => 'File Path',
            'Owner' => 'Owner',
            'Sysfolder' => 'Sysfolder',
            'Check' => 'Check',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getParent()
    {
        return $this->hasOne(MediaFiles::className(), ['MediaID' => 'Parent']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMediaFiles()
    {
        return $this->hasMany(MediaFiles::className(), ['Parent' => 'MediaID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOwner()
    {
        return $this->hasOne(Users::className(), ['User_ID' => 'Owner']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPlayListFiles()
    {
        return $this->hasMany(PlayListFiles::className(), ['FileID' => 'MediaID']);
    }
}
