<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "PlaySessionTime".
 *
 * @property integer $ScheduleID
 * @property integer $SessionID
 * @property integer $WeekDay
 * @property string $PlayTime
 * @property integer $TempFlag
 *
 * @property DisableDevSession[] $disableDevSessions
 * @property DeviceList[] $devices
 * @property PlaySessions $session
 */
class PlaySessionTime extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'PlaySessionTime';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['SessionID', 'WeekDay', 'PlayTime'], 'required'],
            [['SessionID', 'WeekDay', 'TempFlag'], 'integer'],
            [['PlayTime'], 'safe'],
            [['SessionID', 'WeekDay', 'PlayTime'], 'unique', 'targetAttribute' => ['SessionID', 'WeekDay', 'PlayTime'], 'message' => 'The combination of Session ID, Week Day and Play Time has already been taken.'],
            [['SessionID'], 'exist', 'skipOnError' => true, 'targetClass' => PlaySessions::className(), 'targetAttribute' => ['SessionID' => 'SessionID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'ScheduleID' => 'Schedule ID',
            'SessionID' => 'Session ID',
            'WeekDay' => 'Week Day',
            'PlayTime' => 'Play Time',
            'TempFlag' => 'Temp Flag',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDisableDevSessions()
    {
        return $this->hasMany(DisableDevSession::className(), ['ScheduleID' => 'ScheduleID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDevices()
    {
        return $this->hasMany(DeviceList::className(), ['DevID' => 'DeviceID'])->viaTable('DisableDevSession', ['ScheduleID' => 'ScheduleID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSession()
    {
        return $this->hasOne(PlaySessions::className(), ['SessionID' => 'SessionID']);
    }
}
