<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "PlaySessions".
 *
 * @property integer $SessionID
 * @property string $SessionName
 * @property integer $SessionType
 * @property integer $Priority
 * @property integer $Status
 * @property integer $CircleLength
 * @property integer $ClassID
 * @property string $PlayerID
 * @property integer $Parent
 * @property integer $BrdMode
 * @property integer $MemberMode
 * @property integer $StreamType
 * @property integer $SysEnabled
 * @property string $RfDevCtrl
 * @property integer $Owner
 * @property integer $PlayMode
 * @property integer $AudioGain
 * @property integer $Template
 *
 * @property PlayListFiles[] $playListFiles
 * @property PlaySessionTime[] $playSessionTimes
 * @property Players $player
 * @property BroadcastMode $brdMode
 * @property PlaySessions $parent
 * @property PlaySessions[] $playSessions
 * @property Users $owner
 * @property TaskTemplate $template
 * @property SessionDevice[] $sessionDevices
 */
class PlaySessions extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'PlaySessions';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['SessionID', 'SessionName'], 'required'],
            [['SessionID', 'SessionType', 'Priority', 'Status', 'CircleLength', 'ClassID', 'Parent', 'BrdMode', 'MemberMode', 'StreamType', 'SysEnabled', 'Owner', 'PlayMode', 'AudioGain', 'Template'], 'integer'],
            [['SessionName', 'PlayerID'], 'string', 'max' => 50],
            [['RfDevCtrl'], 'string', 'max' => 16],
            [['SessionID'], 'unique'],
            [['PlayerID'], 'exist', 'skipOnError' => true, 'targetClass' => Players::className(), 'targetAttribute' => ['PlayerID' => 'PlayerID']],
            [['BrdMode'], 'exist', 'skipOnError' => true, 'targetClass' => BroadcastMode::className(), 'targetAttribute' => ['BrdMode' => 'idBroadcastMode']],
            [['Parent'], 'exist', 'skipOnError' => true, 'targetClass' => PlaySessions::className(), 'targetAttribute' => ['Parent' => 'SessionID']],
            [['Owner'], 'exist', 'skipOnError' => true, 'targetClass' => Users::className(), 'targetAttribute' => ['Owner' => 'User_ID']],
            [['Template'], 'exist', 'skipOnError' => true, 'targetClass' => TaskTemplate::className(), 'targetAttribute' => ['Template' => 'idTaskTemplate']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'SessionID' => 'Session ID',
            'SessionName' => 'Session Name',
            'SessionType' => 'Session Type',
            'Priority' => 'Priority',
            'Status' => 'Status',
            'CircleLength' => 'Circle Length',
            'ClassID' => 'Class ID',
            'PlayerID' => 'Player ID',
            'Parent' => 'Parent',
            'BrdMode' => 'Brd Mode',
            'MemberMode' => 'Member Mode',
            'StreamType' => 'Stream Type',
            'SysEnabled' => 'Sys Enabled',
            'RfDevCtrl' => 'Rf Dev Ctrl',
            'Owner' => 'Owner',
            'PlayMode' => 'Play Mode',
            'AudioGain' => 'Audio Gain',
            'Template' => 'Template',
        ];
    }

    public function fields()
    {
        $fileds = parent::fields(); // TODO: Change the autogenerated stub
        unset($fileds['ClassID'], $fileds['Parent'], $fileds['Status']);
        return $fileds;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPlayListFiles()
    {
        return $this->hasMany(PlayListFiles::className(), ['SessionID' => 'SessionID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPlaySessionTimes()
    {
        return $this->hasMany(PlaySessionTime::className(), ['SessionID' => 'SessionID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPlayer()
    {
        return $this->hasOne(Players::className(), ['PlayerID' => 'PlayerID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrdMode()
    {
        return $this->hasOne(BroadcastMode::className(), ['idBroadcastMode' => 'BrdMode']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getParent()
    {
        return $this->hasOne(PlaySessions::className(), ['SessionID' => 'Parent']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPlaySessions()
    {
        return $this->hasMany(PlaySessions::className(), ['Parent' => 'SessionID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOwner()
    {
        return $this->hasOne(Users::className(), ['User_ID' => 'Owner']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTemplate()
    {
        return $this->hasOne(TaskTemplate::className(), ['idTaskTemplate' => 'Template']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSessionDevices()
    {
        return $this->hasMany(SessionDevice::className(), ['SessionID' => 'SessionID']);
    }
}
