<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "SessionDevice".
 *
 * @property integer $SessionDevID
 * @property string $DeviceID
 * @property integer $SessionID
 * @property integer $KeyIndex
 * @property integer $DevDisabled
 *
 * @property DeviceList $device
 * @property PlaySessions $session
 */
class SessionDevice extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'SessionDevice';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['DeviceID', 'SessionID'], 'required'],
            [['SessionID', 'KeyIndex', 'DevDisabled'], 'integer'],
            [['DeviceID'], 'string', 'max' => 50],
            [['SessionID', 'DeviceID', 'KeyIndex'], 'unique', 'targetAttribute' => ['SessionID', 'DeviceID', 'KeyIndex'], 'message' => 'The combination of Device ID, Session ID and Key Index has already been taken.'],
            [['DeviceID'], 'exist', 'skipOnError' => true, 'targetClass' => DeviceList::className(), 'targetAttribute' => ['DeviceID' => 'DevID']],
            [['SessionID'], 'exist', 'skipOnError' => true, 'targetClass' => PlaySessions::className(), 'targetAttribute' => ['SessionID' => 'SessionID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'SessionDevID' => 'Session Dev ID',
            'DeviceID' => 'Device ID',
            'SessionID' => 'Session ID',
            'KeyIndex' => 'Key Index',
            'DevDisabled' => 'Dev Disabled',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDevice()
    {
        return $this->hasOne(DeviceList::className(), ['DevID' => 'DeviceID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSession()
    {
        return $this->hasOne(PlaySessions::className(), ['SessionID' => 'SessionID']);
    }
}
