<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "Timetable".
 *
 * @property integer $Id
 * @property string $DevId
 * @property integer $UserId
 * @property integer $WeekDay
 * @property string $Start
 * @property string $End
 * @property string $Type
 * @property integer $OneTime
 * @property integer $Sync
 *
 * @property DeviceList $dev
 * @property Users $user
 */
class Timetable extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'Timetable';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['DevId', 'UserId', 'WeekDay'], 'required'],
            [['UserId', 'WeekDay', 'OneTime', 'Sync'], 'integer'],
            [['Start', 'End'], 'safe'],
            [['Type'], 'string'],
            [['DevId'], 'string', 'max' => 50],
            [['DevId', 'UserId', 'WeekDay', 'Start', 'End', 'Type'], 'unique', 'targetAttribute' => ['DevId', 'UserId', 'WeekDay', 'Start', 'End', 'Type'], 'message' => 'The combination of Dev ID, User ID, Week Day, Start, End and Type has already been taken.'],
            [['DevId'], 'exist', 'skipOnError' => true, 'targetClass' => DeviceList::className(), 'targetAttribute' => ['DevId' => 'DevID']],
            [['UserId'], 'exist', 'skipOnError' => true, 'targetClass' => Users::className(), 'targetAttribute' => ['UserId' => 'User_ID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'Id' => 'ID',
            'DevId' => 'Dev ID',
            'UserId' => 'User ID',
            'WeekDay' => 'Week Day',
            'Start' => 'Start',
            'End' => 'End',
            'Type' => 'Type',
            'OneTime' => 'One Time',
            'Sync' => 'Sync',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDev()
    {
        return $this->hasOne(DeviceList::className(), ['DevID' => 'DevId']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(Users::className(), ['User_ID' => 'UserId']);
    }
}
