<?php
namespace common\models;

use common\classes\Common;
use common\classes\Error;
use yii\base\Model;
use yii\db\Exception;
use yii;


class UploadFile extends Model
{
    const FILE_EXTENSIONS = 'mp3,ts,mp4,3gp,mov,avi,mpg,amr,aac,ac3,amr,ape,flac,mmf,mp2,wav,ogg,wma,mkv';
    const FILE_MAX_COUNT = 10;

    /**
     * @var  multiple files instance
     */
    public $multiple;

    /**
     * @var imageFile
     */
    public $imageFile;

    public $firmwareFile;

    /**
     * @return array the validation rules.
     */
    public function rules()
    {
        return [
            [['multiple'], 'file', 'skipOnEmpty' => false, 'extensions' => self::FILE_EXTENSIONS, 'maxFiles' => self::FILE_MAX_COUNT, 'maxSize' => 4294967295, 'checkExtensionByMimeType' => false],
            [['imageFile'], 'file', 'skipOnEmpty' => false, 'extensions' => 'png, ico'],
            [['firmwareFile'], 'file', 'skipOnEmpty' => false]
        ];
    }

    //upload multiple files
    public function uploadMultiple($parentid)
    {
        if ($this->validate(['multiple']))
        {
            $files = [];
            foreach ($this->multiple as $file) {
                $filepath = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16) . ".tmp";
                $file->saveAs($filepath);
                $streams = Yii::$app->media->info($filepath);
                $media = null;
                do{
                    if(empty($streams)) {
                        $ret = Error::STATUS_MEDIA_GET_INFO_FAILED;
                        break;
                    }
                    $ret = Yii::$app->media->codecCheck($filepath, $streams);
                    if ($ret != Error::STATUS_SUCCESS)
                        break;
                    $media = Yii::$app->media->saveAs($file->name, $filepath, $streams, $parentid);
                    if(empty($media))
                        $ret = Error::STATUS_FAILED;
                }while(false);
                $files[$file->name] = [
                    'error' => $ret,
                    'media' => $media,
                    'message' => Yii::t('app/error', $ret)
                ];
                if(file_exists($filepath))
                    unlink($filepath);
            }
            return $files;
        } else {
            return [];
        }
    }
    //upload signle file
    public function upload($location)
    {
        if ($this->validate(['firmwareFile'])) {
            $file = $this->firmwareFile;
            $path = $location . '/' . $file->name;
            $file->saveAs($path);
            return true;
        } else {
            return false;
        }
    }
    //upload image files
    public function uploadImage($location) {
        if ($this->validate(['imageFile'])) {
            $file = $this->imageFile;

            $tempFileName = Yii::$app->security->generateRandomString(16);
            $srcFile = Common::MEDIA_TEMPORARA_DIR . $tempFileName;

            if ($location == 'scene') {
                if ($file->type !== 'image/png')
                return Yii::$app->utils->response(Error::STATUS_ICONFILE_TYPE_ERROR);
                $ret = $file->saveAs(Common::CUSTOM_STORAGE_DIR . '/image/scene_' . $tempFileName . '.png');
                if (!$ret)
                    return Yii::$app->utils->response(Error::STATUS_FILE_UPLOAD_FAILED);
                return Yii::$app->utils->response(Error::STATUS_FILE_UPLOAD_SUCCESS, $tempFileName);
            }

            $file->saveAs($srcFile);

            if ($file->type !== 'image/x-icon' && $file->type !== 'image/png')
                return Yii::$app->utils->response(Error::STATUS_FILE_TYPE_ERROR);
            if (!file_exists($srcFile))
                return Yii::$app->utils->response(Error::STATUS_FAILED);

            if (!file_exists(Common::IMAGE_STORAGE_DIR)) {
                mkdir(Common::IMAGE_STORAGE_DIR);
            }
            $mime = null;
            if ($file->type === 'image/x-icon')
                $mime = '.ico';
            else
                $mime = '.png';

            $filepath = null;
            switch ($location) {
                case 'header':
                    $filepath = Common::IMAGE_STORAGE_DIR . 'logo' . $mime;
                    break;
                case 'login':
                    $filepath = Common::IMAGE_STORAGE_DIR . 'idste_logo' . $mime;
                    break;
                case 'tab':
                    $filepath = Common::IMAGE_STORAGE_DIR . 'favicon.ico';
                    break;
            }
            if (is_null($filepath))
                return Yii::$app->utils->response(Error::STATUS_FAILED);

            system("mv $srcFile $filepath");
            $customPath = Common::CUSTOM_STORAGE_DIR;
            system("cp $filepath $customPath");
            return Yii::$app->utils->response(Error::STATUS_SUCCESS);
        } else
            return Yii::$app->utils->response(Error::STATUS_FAILED);
    }

}