<?php
namespace common\models;

use api\controllers\BaseController;
use common\classes\Common;
use common\classes\System;
use Faker\Provider\Base;
use Yii;
use yii\base\Exception;
use yii\base\NotSupportedException;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveRecord;
use yii\web\IdentityInterface;

/**
 * Users model
 * @property integer $User_ID
 * @property string $User_Name
 * @property string $User_Desc
 * @property string $Pass_Word
 * @property string $UsbKey
 * @property integer $Priority
 * @property integer $privilege
 * @property string $IcCard
 * @property string $Phone
 * @property integer $ServerSync
 * @property integer $User_Type
 * @property string $User_Sn
 * @property string $User_Expiry
 * @property integer $Mgr_Privilege
 * @property integer $Dev_Privilege
 * @property integer $Create_Type
 * @property integer $IC_Status
 * @property string $Dpm_Sn
 * @property string $Dpm_Name
 * @property integer $Del_Flag
 * @property string $UnionId
 *
 * @property MediaFiles[] $mediaFiles
 * @property PlaySessions[] $playSessions
 * @property Timetable[] $timetables
 * @property UserDevices[] $userDevices
 * @property DeviceList[] $devs
 * @property Players $usbKey
 */
class Users extends ActiveRecord implements IdentityInterface
{
    // const CLOUD_SERVER_API = ['message/push'];
    //login flag for cloud server
    public $cloudLogin;

    public function __construct(array $config = [])
    {
        parent::__construct($config);
        $this->cloudLogin = false;
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['Priority', 'privilege', 'ServerSync', 'User_Type', 'Mgr_Privilege', 'Dev_Privilege', 'Create_Type', 'IC_Status', 'Del_Flag'], 'integer'],
            [['User_Expiry'], 'safe'],
            [['User_Name', 'User_Desc'], 'string', 'max' => 255],
            [['Pass_Word'], 'string', 'max' => 256],
            [['UsbKey', 'IcCard', 'Phone'], 'string', 'max' => 50],
            [['User_Sn', 'Dpm_Sn'], 'string', 'max' => 20],
            [['Dpm_Name'], 'string', 'max' => 100],
            [['UnionId'], 'string', 'max' => 64],
            [['UsbKey'], 'unique'],
            [['User_Name'], 'unique'],
            [['IcCard'], 'unique'],
            [['User_Sn'], 'unique'],
            [['UnionId'], 'unique']
        ];
    }

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'Users';
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
        ];
    }

    /**
     * @inheritdoc
     */
    public function fields()
    {
        $fields = parent::fields();
        unset($fields['Pass_Word']);
        unset($fields['ServerSync']);
        unset($fields['Del_Flag']);
        return $fields;
    }

    /**
     * @inheritdoc
     */
    public static function findIdentity($id)
    {
        return static::findOne(['User_ID' => $id]);
    }

    /**
     * @inheritdoc
     */
    public static function findIdentityByAccessToken($token, $type = null)
    {
        if (strlen($token) < 32)
            return null;

        do {
            $cloudInfo = System::getCloudInfo();
            if (is_null($cloudInfo))
                break;
            if ($token !== $cloudInfo['secret_token'])
                return null;
            $cloudUser = Yii::$app->getCache()->get('secret_token_cred');
            if ($cloudUser === false) {
                $cloudUser = Users::find()->where(['User_ID' => '0'])->orWhere(['User_Name' => 'admin'])->one();
                $cloudUser->cloudLogin = true;
                Yii::$app->getCache()->set('secret_token_cred', $cloudUser);
            }
            return $cloudUser;
        }while(false);
        //set allow origin header for 401 error special
        header('Access-Control-Allow-Origin:*');
        return null;
    }

    /**
     * Finds user by username
     *
     * @param string $username
     * @return static|null
     */
    public static function findByUsername($username)
    {
        return static::findOne(['User_Name' => $username]);
    }

    /**
     * Finds user by password reset token
     *
     * @param string $token password reset token
     * @return static|null
     */
    public static function findByPasswordResetToken($token)
    {
        throw new NotSupportedException('"findByPasswordResetToken" is not implemented.');
    }

    /**
     * Finds out if password reset token is valid
     *
     * @param string $token password reset token
     * @return bool
     */
    public static function isPasswordResetTokenValid($token)
    {
        throw new NotSupportedException('"isPasswordResetTokenValid" is not implemented.');
    }

    /**
     * @inheritdoc
     */
    public function getId()
    {
        return $this->getPrimaryKey();
    }

    /**
     * @inheritdoc
     */
    public function getAuthKey()
    {
        return $this->auth_key;
    }

    /**
     * @inheritdoc
     */
    public function validateAuthKey($authKey)
    {
        return $this->getAuthKey() === $authKey;
    }

    public static function mysql_password($password)
    {
        $result = Yii::$app->db
            ->createCommand("select PASSWORD(:password);", [':password' => $password])
            ->queryOne();
        foreach ($result as $key => $value)
            return $value;
        return null;
    }

    /**
     * Validates password
     *
     * @param string $password password to validate
     * @return bool if password provided is valid for current user
     */
    public function validatePassword($password)
    {
        return $this->Pass_Word == $this->mysql_password($password);
    }

    /**
     * Generates password hash from password and sets it to the model
     *
     * @param string $password
     */
    public function setPassword($password)
    {
        $this->Pass_Word = $this->mysql_password($password);
    }

    /**
     * Generates "remember me" authentication key
     */
    public function generateAuthKey()
    {
        $this->auth_key = Yii::$app->security->generateRandomString();
    }
}
