<?php

namespace common\models;

use common\classes\Common;
use Yii;
use common\classes\MysqlAccess;

/**
 * This is the model class for table "v_device".
 *
 * @property string $id
 * @property string $name
 * @property integer $parent
 * @property string $description
 * @property integer $rf_invalid
 * @property string $ModeName
 * @property integer $ModePriority
 * @property string $rtsp
 * @property string $rtsp1
 * @property string $ProjAlert
 * @property string $mic0
 * @property string $mic1
 * @property string $mic2
 * @property string $mic3
 * @property string $micChargeStub
 */
class VDevice extends \yii\db\ActiveRecord
{
    public $devices;
    public $uid;
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'v_device';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['devices'], 'match', 'pattern' => '/^[0-9][0-9, ]*[0-9]$/i'],
            [['id'], 'required'],
            [['parent', 'rf_invalid', 'ModePriority', 'ProjAlert'], 'integer'],
            [['id',  'ModeName'], 'string', 'max' => 50],
            [['name'], 'string', 'max' => 32],
            [['description'], 'string', 'max' => 96],
            [['rtsp', 'rtsp1', 'pcCtrl'], 'string', 'max' => 3],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'parent' => 'Parent',
            'description' => 'Description',
            'rf_invalid' => 'Rf Invalid',
            'ModeName' => 'Mode Name',
            'ModePriority' => 'Mode Priority',
            'rtsp' => 'Rtsp',
            'rtsp1' => 'Rtsp1',
            'ProjAlert' => 'Proj Alert',
            'pcCtrl' => 'Pc Ctrl',
        ];
    }

    public function save($runValidation = true, $attributeNames = null, $isTeacher = false)
    {
        $access = new MysqlAccess($this->devices, $isTeacher);
        if (is_null($access))
            return false;
        try {
            $userinfo = Yii::$app->cache->get(Common::USER_INFO_SALT . Yii::$app->user->id);
            /*物联设备*/
            if (isset($this->iotStatus) && strlen($this->iotStatus) == 32 && ctype_xdigit($this->iotStatus)) {
                if (empty($userinfo) || (!$isTeacher && !in_array('rfctrl', $userinfo['pri'])))
                    return false;
                foreach ($access->models as $model) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('rfCtrl', $model->DeviceID, ['Status' => $this->iotStatus]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
            } else if (!$isTeacher && (empty($userinfo) || !in_array('centerctrl', $userinfo['pri'])))
                return false;
            /*设备控制*/
            foreach ($access->models as $model) {
                if (isset($this->power) && $isTeacher && $model->State_SysPower == 2) {
                    return false;
                }
                if (isset($this->power)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('devCtrl', $model->DeviceID, ['PowerOn' => $this->power]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                    Yii::$app->db->createCommand('update DeviceStatus set User_ID=' . $this->uid . ' where State_SysPower != 1 and DeviceID = ' . $model->DeviceID . ';')
                        ->execute();
                }
                if (isset($this->pc)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('devCtrl', $model->DeviceID, ['pcPowerOn' => $this->pc]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->projector)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('devCtrl', $model->DeviceID, ['projectorPowerOn' => $this->projector]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->projector2)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('devCtrl', $model->DeviceID, ['projector2PowerOn' => $this->projector2]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->audioMute)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('VolumeCtrl', $model->DeviceID, ['mainVolumeMute' => $this->audioMute]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->micMute)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('VolumeCtrl', $model->DeviceID, ['micVolumeMute' => $this->micMute]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->micVol) && is_numeric($this->micVol)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('VolumeCtrl', $model->DeviceID, ['micVolume' => $this->micVol]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->audioVol) && is_numeric($this->audioVol)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('VolumeCtrl', $model->DeviceID, ['mainVolume' => $this->audioVol]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->projectorSrc)) {
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('VideoChannelCtrl', $model->DeviceID, ['projectorChannelSource' => $this->projectorSrc]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
                if (isset($this->scene)) {
                    if (!Scene::findOne(['id' => $this->scene]))
                        return false;
                    $ret = Yii::$app->sendcmd2cmpn->sendCmd('SceneCtrl', $model->DeviceID, ['SceneNow' => $this->scene]);
                    if (!$ret || $ret->errCode != 0)
                        return false;
                }
            }
        } catch (\Exception $e){
            return false;
        }
        return true;
    }

}
