<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "v_event".
 *
 * @property integer $Id
 * @property string $Name
 * @property integer $StartTask
 * @property string $StartTaskName
 * @property integer $EndTask
 * @property string $EndTaskName
 * @property resource $Config
 * @property integer $Model
 * @property string $Manufacturer
 * @property string $ModelName
 * @property integer $Version
 */
class VEvent extends \yii\db\ActiveRecord
{
    public $ip;
    public $io;
    public $port;
    public $level;
    public $errors;
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'v_event';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['Id', 'StartTask', 'EndTask', 'Model', 'Version', 'ip', 'io', 'port','level'], 'integer'],
            [['Name', 'Config', 'Model'], 'required'],
            [['io', 'port','level'], 'integer'],
            [['ip'], 'string','max' => 16],
            [['ip'], 'match', 'pattern' => '/^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$/i'],
            [['Name', 'Manufacturer', 'ModelName'], 'string', 'max' => 31],
            [['StartTaskName', 'EndTaskName'], 'string', 'max' => 50],
            [['Config'], 'string', 'max' => 64],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'Id' => 'ID',
            'Name' => 'Name',
            'StartTask' => 'Start Task',
            'StartTaskName' => 'Start Task Name',
            'EndTask' => 'End Task',
            'EndTaskName' => 'End Task Name',
            'Config' => 'Config',
            'Model' => 'Model',
            'Manufacturer' => 'Manufacturer',
            'ModelName' => 'Model Name',
            'Version' => 'Version',
        ];
    }

    public function fields()
    {
        $ret = parent::fields(); // TODO: Change the autogenerated stub
        unset($ret['Config'], $ret['Version']);
        if(strlen($this->Config) != 64 || strncmp($this->Config, 'ED23C3A3', 8) !== 0) {
            $ret['ip'] = function ($model){return null;};
            $ret['port'] = function ($model){return null;};
            $ret['io'] = function ($model){return null;};
            $ret['level'] = function ($model){return null;};
        }
        else {
            $ret['ip'] = function ($model) {
                $ip = '';
                for($i = 8; $i < 40; $i+=2) {
                    $dec = hexdec(substr($model->Config, $i, 2));
                    if($dec == 0)
                        break;
                    $ip .= chr($dec);
                }
                return $ip;
            };
            $ret['port'] = function ($model) {
                $tmp = '';
                for($i = 40; $i < 44; $i+=2) {
                    $sub = substr($model->Config, $i, 2);
                    $tmp = $sub . $tmp;
                }
                return hexdec($tmp);
            };
            $ret['io'] = function ($model) {
                $tmp = '';
                for($i = 48; $i < 56; $i+=2) {
                    $sub = substr($model->Config, $i, 2);
                    $tmp = $sub . $tmp;
                }
                return hexdec($tmp);
            };
            $ret['level'] = function ($model) {
                $tmp = '';
                for($i = 56; $i < 64; $i+=2) {
                    $sub = substr($model->Config, $i, 2);
                    $tmp = $sub . $tmp;
                }
                return hexdec($tmp);
            };
        }
        return $ret;
    }

    public function save($runValidation = true, $attributeNames = null)
    {
        if(ip2long($this->ip) == false)
            return false;
        if($this->isNewRecord)
            $model = new Event();
        else if(!isset($this->Id))
            return false;
        else {
            $model = Event::findOne(['Id' => $this->Id]);
            if(empty($model))
                return false;
        }
        $model->Name = $this->Name;
        $model->StartTask = $this->StartTask;
        $model->EndTask = $this->EndTask;
        $model->Model = $this->Model;
        $model->Config = 'ED23C3A3';
        for($i = 0; $i < strlen($this->ip); $i++){
            $num = ord(substr($this->ip, $i, 1));
            $model->Config .= ($num < 16) ? '0' . dechex($num) : dechex($num);
        }
        for(;$i < 16;$i++) {
            $model->Config .= '00';
        }
        $this->level &= $this->io;
        $this->port &= 0x0000ffff;
        $io = '';
        $level = '';
        $port = '';
        for($i = 0; $i < 4; $i++) {
            $num = ($this->port >> ($i * 8)) & 0xff;
            $chr = ($num < 16) ? '0' . dechex($num) : dechex($num);
            $port .= $chr;
            $num = ($this->level >> ($i * 8)) & 0xff;
            $chr = ($num < 16) ? '0' . dechex($num) : dechex($num);
            $level .= $chr;
            $num = ($this->io >> ($i * 8)) & 0xff;
            $chr = ($num < 16) ? '0' . dechex($num) : dechex($num);
            $io .= $chr;
        }
        $model->Config = hex2bin($model->Config . $port . $io . $level);
        $model->Version = 0;
        $ret = $model->save();
        if($ret) {
            $this->Id = $model->Id;
        }
        return $ret;
    }
}
