<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/19 15:58
 */

namespace Workerman\iDste;
use common\classes\Common;
use Workerman\Worker;
use Yii;

use common\models\BaseInfo;
use common\classes\HttpRequest;
use common\classes\System;
use Workerman\Lib\Timer;
use yii\db\Exception;

require_once __DIR__ . '/../Autoloader.php';
require_once  __DIR__ . '/../../common/classes/System.php';
require_once  __DIR__ . '/../../common/classes/HttpRequest.php';
require_once  __DIR__ . '/../../common/classes/Error.php';


class MsgDeamon
{
    const WORKER_COUNT = 1;
    private $id;
    private $cache;
    private $waitSendMsg = [];
    public function __construct($deamon){
        $deamon->count = self::WORKER_COUNT;
        $deamon->onWorkerStart = [$this, 'workerStart'];
    }

    public function workerStart($worker){
        $this->dbReconnect();
        $this->id = $worker->id;
        $port = Common::MSG_WORKER_PORT_START + $worker->id;
        $udpWorker = new Worker('udp://127.0.0.1:' . $port);
        $udpWorker->onMessage = [$this, 'onUdpMessage'];
        $udpWorker->listen();

        $this->cache['mac'] = System::getMac();
        $this->cache['access_token'] = 'empty';
        //cloud server url
        echo "message worker start\n";
        $this->cache['server'] = 'https://' . Yii::$app->base->cloudServer;
        while(true) {
            //uuid
            $uuid = Yii::$app->base->uuid;
            if (empty($uuid)) {
                echo "unregisted server.\n";
                sleep(30);
                continue;
            }
            $this->cache['uuid'] = $uuid;
            $this->updateAccessToken();
            break;
        }
        Timer::add(60, [$this, 'updateAccessToken']);
        Timer::add(2, [$this, 'msgDeamon']);
        Timer::add(300, [$this, 'dbReconnect']);
    }

    /**
     * reconnect every 300 sec
     */
    public function dbReconnect(){
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    public function updateAccessToken(){
        //access_token
        $item = BaseInfo::findOne(['InfomationID' => 339]);
        if(!empty($item) && isset($item->Description))
            $this->cache['access_token'] = $item->Description;
    }

    private function sendMessage($data){
        if(!is_array($data)) {
            echo "need array!\n";
            return false;
        }
        $req = new HttpRequest($this->cache['server'] . '/api/message/push', [
            'access_token' => $this->cache['access_token'],
            'uuid' => $this->cache['uuid']
        ]);
        $out = $req->post([
            'data' => $data,
            'uuid' => $this->cache['uuid']
        ]);
        do {
            if(empty($out) || !isset($out->status))
                break;
            if($out->status === 0)
                return true;
            if($out->status === 401)
                $this->updateAccessToken();
        }while(false);
        return false;
    }

    public function onUdpMessage($conn, $data){
        echo "receive message from udp client: " . $data . "\n";
        $data = json_decode($data, true);
        if(empty($data) || !is_array($data) || !isset($data['type']) || !isset($data['msg']))
            return;
        if(isset($data['uid'])) {
            $data['level'] = isset($data['level'])? $data['level']: 'info';
        }
        else if(!isset($data['level']) || $data['level'] == 'info') {
            echo 'message structure error!' . print_r($data, true) . "\n";
        }
        $this->waitSendMsg[] = [
            'body' => $data,
            'sendTime' => time(),
            'tryCnt' => 0
        ];
    }

    public function msgDeamon(){
        foreach ($this->waitSendMsg as $key => $msg) {
            if($msg['sendTime'] > time())
                continue;
            $ret =$this->sendMessage($msg['body']);
            if($ret == false) {
                $tryCnt = $this->waitSendMsg[$key]['tryCnt'];
                $tryCnt++;
                if($tryCnt >= Common::MSG_MAX_TRYCNT) {
                    unset($this->waitSendMsg[$key]);
                    continue;
                }
                $this->waitSendMsg[$key]['tryCnt'] = $tryCnt;
                $this->waitSendMsg[$key]['sendTime'] = time() + $tryCnt * 5;
            }
            else {
                unset($this->waitSendMsg[$key]);
            }
        }
    }
}
