<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\classes\Common;
use common\classes\Error;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\ProjectorType;
use common\models\SystemLog;
use Workerman\Lib\Timer;
use Workerman\Worker;
use Yii;
use yii\db\Exception;

require_once __DIR__ . '/../Autoloader.php';

class SystemDeamon
{
    const SUPPORT_OFF = 0;
    const SUPPORT_ON = 1;
    private $supportState;
    private $supportHost;
    private $supportPort;
    private $supportUid;
    private $supportDesc;
    private $supportTimer;
    private $projectorTimer;
    public function __construct(){
        $udpWorker = new Worker('udp://127.0.0.1:' . Common::SYSTEM_DEAMON_PORT);
        $udpWorker->onMessage = [$this, 'onUdpMessage'];
        $udpWorker->listen();
        $this->supportUid = Yii::$app->base->support;
        if(is_null($this->supportUid))
            $this->supportState = self::SUPPORT_OFF;
        else {
            $this->supportState = self::SUPPORT_ON;
            $this->supportTimer = Timer::add(30, [$this, 'supportAlive']);
        }
        $this->projectorTimer = Timer::add(60, [$this, 'projectorUpdate']);
    }

    public function onUdpMessage($conn, $data){
        echo "network receive message:$data\n";
        $para = json_decode($data, true);
        $ret = '';
        if(!empty($para['action']) && !empty($para['controller'])) {
            switch ($para['controller']) {
                case 'network':
                    $ret = Yii::$app->net->run($para['action'], $para['param']);
                    break;
                case 'system':
                    $ret = $this->systemCtrl($para['action'], $para['param']);
                    break;
                case 'support':
                    $ret = $this->supportCtrl($para['action'], $para['param']);
                    $ret = json_encode(unserialize($ret));
                    break;
                case 'projector':
                    $ret = $this->projector($para['action'], $para['param']);
                    break;
                case 'datetime':
                    $ret = $this->datetime($para['action'], $para['param']);
                    $ret = json_encode(unserialize($ret));
                    break;
                case 'remote-ctrl':
                    $ret = $this->remoteCtrl($para['param']);
                    break;
                default:
                    $ret = Error::serializeResponse(Error::STATUS_FAILED);
            }
        }
        else
            $ret = '';
        echo "network deamon return:$ret\n";
        $conn->send($ret);
        $conn->close();
    }

    private function datetime($action, $param) {
        switch ($action){
            case 'view':
                return Error::serializeResponse(Error::STATUS_SUCCESS, Yii::$app->datetime->view());
            case "update":
                return Error::serializeResponse(Error::STATUS_SUCCESS, Yii::$app->datetime->update($param));
        }
        return Error::serializeResponse(Error::STATUS_PARAM_ERROR);
    }

    private function remoteCtrl($param) {
        $str = Yii::$app->cache->get(Common::PC_REMOTE_CTRL_SALT . $param);
        if(empty($str))
            return '';
        return $str;
    }

    private function projector($action ,$param) {
        if($action != 'new')
            return '';
        try {
            $data = [];
            $data['check_bit'] = ProjectorType::CHECK_BIT[$param['CheckBit']];
            $data['stop_bit'] = ProjectorType::STOP_BIT[$param['StopBit']];
            $data['baudrate'] = ProjectorType::BAUD_RATE[$param['BaudRate']];
            $data['data_bit'] = ProjectorType::DATA_BIT[$param['DataBit']];
            $data['type'] = $param['Type'];
            $data['delay'] = $param['PoweroffDelay'];
            $data['cmd_close'] = $param['CmdClose'];
            $data['cmd_open'] = $param['CmdOpen'];
            $data['cmd_pc'] = $param['CmdFromPc'];
            $data['cmd_video'] = $param['CmdFromVideo'];
            $data['status'] = 'private';
            $server = 'https://' . Yii::$app->base->cloudServer;
            $req = new HttpRequest($server . '/api/projector/new', [
                'access_token' => Yii::$app->base->accessToken,
                'uuid' => Yii::$app->base->uuid
            ]);
            $ret = $req->post(json_encode(['data' => $data]), false);
            if(is_null($ret) || !isset($ret->status) || $ret->status !== Error::STATUS_SUCCESS)
                return '';
        }catch (\Exception $e){}
        return '';
    }

    public function projectorUpdate(){
        $server = 'https://' . Yii::$app->base->cloudServer;
        $req = new HttpRequest($server . '/api/projector/list', [
            'access_token' => Yii::$app->base->accessToken,
            'uuid' => Yii::$app->base->uuid
        ]);
        $ret = $req->get(true);
        if(is_null($ret) || !isset($ret['status']) || $ret['status'] !== Error::STATUS_SUCCESS || !isset($ret['data']))
            return ;
        foreach ($ret['data'] as $item) {
            if(empty($item['cmd_close']) || !ctype_xdigit($item['cmd_close']) ||
                empty($item['cmd_open']) || !ctype_xdigit($item['cmd_open']) ||
                (!empty($item['cmd_pc']) && !ctype_xdigit($item['cmd_pc'])) ||
                (!empty($item['cmd_video']) && !ctype_xdigit($item['cmd_video'])))
                continue;
            $projector = ProjectorType::findOne(['Type' => $item['type']]);
            if(empty($projector))
                $projector = new ProjectorType();
            else{
                continue;
            }
            $projector->Type = $item['type'];
            $projector->PoweroffDelay = $item['delay'];
            $projector->CmdClose = $item['cmd_close'];
            $projector->CmdOpen = $item['cmd_open'];
            $projector->CmdFromPc = $item['cmd_pc'];
            $projector->CmdFromVideo = $item['cmd_video'];
            $projector->StopBit = array_search($item['stop_bit'], ProjectorType::STOP_BIT);
            $projector->DataBit = array_search($item['data_bit'], ProjectorType::DATA_BIT);
            $projector->CheckBit = array_search($item['check_bit'], ProjectorType::CHECK_BIT);
            $projector->BaudRate = array_search($item['baudrate'], ProjectorType::BAUD_RATE);
            $projector->save();
        }
        Timer::del($this->projectorTimer);
        $this->projectorTimer = Timer::add(3600, [$this, 'projectorUpdate']);
    }

    public function restartNginx(){
        echo "restart nginx\n";
        shell_exec(`service nginx restart`);
    }
    private function systemCtrl($action, $param){
        switch ($action){
            case 'shutdown':
                Yii::$app->system->shutdown();
                sleep(5);
                return Error::serializeResponse(Error::STATUS_FAILED);
            case "reboot":
                Yii::$app->system->reboot();
                sleep(5);
                return Error::serializeResponse(Error::STATUS_FAILED);
            case "service-restart":
                $try = 0;
                while($try < 2) {
                    Yii::$app->system->serviceRestart();
                    $try++;
                }
                return Error::serializeResponse(Error::STATUS_FAILED);
            case "s-c-u":
                $f = '/etc/nginx/ssl/idste.';
                if(empty($param['crt']) || empty($param['key']))
                    break;
                if(file_put_contents($f . "crt1", $param['crt']) && file_put_contents($f . "key1", $param['key'])){
                    shell_exec("mv $f"."crt1 $f" . "crt");
                    shell_exec("mv $f"."key1 $f" . "key");
                }
                $f = '/usr/local/share/libwebsockets-test-server/libwebsockets-test-server.';
                if(file_put_contents($f . "pem1", $param['crt']) && file_put_contents($f . "key.pem1", $param['key'])){
                    shell_exec("mv $f"."pem1 $f" . "pem");
                    shell_exec("mv $f"."key.pem1 $f" . "key.pem");
                }
                Timer::add(5, [$this, "restartNginx"], null, false);
                return Error::serializeResponse(Error::STATUS_SUCCESS);
        }
        return Error::serializeResponse(Error::STATUS_PARAM_ERROR);
    }

    private function supportCtrl($action, $param){
        if(empty($param['state']) || !isset($param['uid']))
            return Error::serializeResponse(Error::STATUS_PARAM_ERROR);
        $this->supportDesc = isset($param['description']) ? $param['description'] : null;
        $this->supportUid = $param['uid'];

        if(empty(Yii::$app->base->uuid))
            return Error::serializeResponse(Error::STATUS_UNREGISTERED);
        if(empty(Yii::$app->base->accessToken))
            return Error::serializeResponse(Error::STATUS_PROXY_BROKEN);
        if($param['state'] === 'on')
            $this->supportStart();
        else
            $this->supportStop();
        return Error::serializeResponse(Error::STATUS_SUCCESS);
    }

    private function supportProxyStop($initiator = 'client') {
        echo "support proxy stop\n";
        Yii::$app->udp->supportCtrlMsg('off', $initiator);
        $this->supportState = self::SUPPORT_OFF;
        if(!is_null($this->supportTimer)) {
            Timer::del($this->supportTimer);
            $this->supportTimer = null;
        }
        BaseInfo::deleteAll(['InfomationID' => Common::BASEINFO_SUPPORT_STATE]);
        if(empty($this->supportHost) || empty($this->supportPort))
            return;
        $cmd = "ps aux | grep 'ssh -i' | grep 'localhost:22' ";
        shell_exec( $cmd . "| awk '{print $2}' | xargs kill -9");
        $this->supportHost = null;
        $this->supportPort = null;
    }

    public function supportProxyStart($reconnect = false) {
        if($this->supportState ==  self::SUPPORT_OFF || empty($this->supportHost) || empty($this->supportPort)) {
            $this->supportProxyStop();
            return;
        }
        $cmd = "ps aux | grep 'ssh -i' | grep 'localhost:22'";
        $ret = exec($cmd . " | wc -l", $ret);
        if($ret[0] < 2 || $reconnect) {
            shell_exec( $cmd . " | awk '{print $2}' | xargs kill -9");
            $cmd = 'ssh -i ' . __DIR__ . '/proxy_user.key -o ServerAliveInterval=60 -o ServerAliveCountMax=3 ';
            $cmd .= '-o StrictHostKeyChecking=no -NfR ';
            $cmd .= $this->supportPort .':localhost:22 proxy_user@'. $this->supportHost;
            system($cmd . ' > /dev/null &');
            echo "proxy reconnect\n";
        }
    }

    public function supportStart(){
        if($this->supportState == self::SUPPORT_ON)
            return;
        echo "support start\n";
        $data = [
            'description' => $this->supportDesc
        ];
        $server = 'https://' . Yii::$app->base->cloudServer;
        $req = new HttpRequest($server . '/api/support/start', [
            'access_token' => Yii::$app->base->accessToken,
            'uuid' => Yii::$app->base->uuid,
            'uid' => $this->supportUid,
        ]);
        $ret = $req->post(json_encode(['data' => $data]), false);
        if(is_null($ret) || !isset($ret->status) || $ret->status == Error::STATUS_TRYAGAIN) {
            echo "proxy confirm failed:" . json_encode($ret) . "\n";
            Timer::add(5, [$this, 'supportStart'], [], false);
            return;
        }
        if($ret->status !== 0 || empty($ret->data) || empty($ret->data->host) || !isset($ret->data->port))
            return;
        Yii::$app->udp->supportCtrlMsg('on');
        $this->supportState = self::SUPPORT_ON;
        $this->supportHost = $ret->data->host;
        $this->supportPort = $ret->data->port;
        $this->supportProxyStart(true);
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SUPPORT_STATE]);
        if(empty($info)) {
            $info = new BaseInfo();
            $info->InfomationID = Common::BASEINFO_SUPPORT_STATE;
            $info->Description = "$this->supportUid";
        }
        $info->InfoValue = time();
        $info->save();
        $this->supportTimer = Timer::add(30, [$this, 'supportAlive']);
    }

    public function supportAlive(){
        echo "support alive\n";
        if($this->supportState !== self::SUPPORT_ON)
            $this->supportProxyStop();
        if(empty(Yii::$app->base->uuid))
            return Error::serializeResponse(Error::STATUS_UNREGISTERED);
        if(empty(Yii::$app->base->accessToken))
            return Error::serializeResponse(Error::STATUS_PROXY_BROKEN);
        $server = 'https://' . Yii::$app->base->cloudServer;
        $req = new HttpRequest($server . '/api/support/alive', [
            'access_token' => Yii::$app->base->accessToken,
            'uuid' => Yii::$app->base->uuid,
            'uid' => $this->supportUid
        ]);
        $ret = $req->post([], false);
        if(is_null($ret) || !isset($ret->status) || empty($ret->data)) {
            echo "support alive error:" . json_encode($ret) . "\n";
            return;
        }
        if($ret->status == Error::STATUS_TRYAGAIN) {
            echo "support alive tryagain:" . json_encode($ret) . "\n";
            return;
        }
        if ($ret->status !== 0) {
            $this->supportProxyStop();
            return;
        }
        $this->supportHost = $ret->data->host;
        $this->supportPort = $ret->data->port;
        if($ret->data->state == 'reconnect')
            $this->supportProxyStart(true);
        else if($ret->data->state == 'stop') {
            $this->supportProxyStop('remote');
            $model = new SystemLog();
            $model->ip = '127.0.0.1';
            $model->username = 'none';
            $model->result = 0;
            $model->title1 = 'stop';
            $model->type = 'support';
            $model->action = 'update';
            $model->save();
        }
        else
            $this->supportProxyStart();
    }

    public function supportStop(){
        if($this->supportState != self::SUPPORT_ON)
            return;
        echo "support stop\n";
        $server = 'https://' . Yii::$app->base->cloudServer;
        $req = new HttpRequest($server . '/api/support/stop', [
            'access_token' => Yii::$app->base->accessToken,
            'uuid' => Yii::$app->base->uuid
        ]);
        $ret = $req->post([], false);
        if(is_null($ret) || !isset($ret->status) || $ret->status !== 0) {
            Timer::add(5, [$this, 'supportStop'], [] , false);
            return;
        }
        $this->supportProxyStop();
    }
}
