<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/08/18 11:54
 */
namespace Workerman\iDste;
use common\models\SystemLog;
use Workerman\Worker;
use Yii;

use common\classes\Error;
use common\classes\HttpRequest;
use common\classes\System;
use common\models\BaseInfo;
use Workerman\Lib\Timer;
use yii\base\ErrorException;
use yii\db\Exception;
use yii\db\Query;

require_once __DIR__ . '/../Autoloader.php';
require_once  __DIR__ . '/../../common/classes/System.php';
require_once  __DIR__ . '/../../common/classes/Error.php';


class TimetableSync
{
//    SQL TEMPLATE,{{start}}\{{end}}\{{size}}will be replaced when query
//    example:
//    const sql = "SELECT * FROM ( SELECT '本校' AS xq, JSZGH AS user_sn, SKSJ AS daylist , SKDD AS classroom, rownum AS id FROM ZFXFZB.JXRWB
//				WHERE XN='2018-2019' AND SKSJ IS NOT NULL AND SKDD IS NOT NULL AND rownum <= {{end}}
//				ORDER BY JSZGH ASC, DAYLIST ASC ) A WHERE id > {{start}}";
    const sql = "";
    //db config: ip, database name, user name, password, database type, database port
    const BIID_IP = 0x11e;
    const BIID_DATABASE = 0x11f;
    const BIID_USERNAME = 0x120;
    const BIID_PASSWORD = 0x121;
    const BIID_DBTYPE = 0x122;
    const BIID_DBPORT = 0x123;
    //fetch rows every time
    const fetchPage = 50;
    //weekday map
    const weekdayMap = [
        '一' => 0,
        '二' => 1,
        '三' => 2,
        '四' => 3,
        '五' => 4,
        '六' => 5,
        '七' => 6
    ];
    //classroom preg
    const classroomPreg = [
        '/*/'
    ];
    public $syncFlag = false;
    public $tryCount = 0;
    public $dbinfo = [];
    public $weekInfo = [
        'openTerm' => null,
        'weekNow' => -1,
        'dayNow' => -1
    ];
    public function __construct($deamon){
        $deamon->onWorkerStart = [$this, 'workerStart'];
    }

    public function workerStart($worker){
        $this->dbReconnect();
        echo "time now:" . time() . "\n";
        echo "date now:" . date("Y-m-d H:i:s", time()) . "\n";
        if(!$this->fetchDbInfo()) {
            echo "fetch db backup infomation failed, stop!\n";
            return;
        }
        $this->sync();
        Timer::add(60, [$this, 'sync']);
    }

    public function log($msg){
        $fp = fopen("../idste_timetable_adapter.log", "a");
        $msg = date('Y-m-d H:i:s') . "$msg\n";
        fwrite($fp, $msg);
        fclose($fp);
    }

    public function dbReconnect()
    {
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    public function fetchDbInfo(){
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_IP]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['ip'] = $bi->Description;
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_DATABASE]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['database'] = $bi->Description;
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_USERNAME]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['username'] = $bi->Description;
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_PASSWORD]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['password'] = $bi->Description;
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_DBTYPE]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['dbtype'] = $bi->Description;
        $bi = BaseInfo::findOne(['InfomationID' => self::BIID_DBPORT]);
        if(empty($bi) || is_null($bi->Description))
            return false;
        $this->dbinfo['port'] = $bi->Description;
        return true;
    }

    public function insert($zone, $weekday, $usersn, $classroom, $sectionStart, $sectionEnd){
        if(!is_numeric($weekday)) {
            if (!array_key_exists($weekday, self::weekdayMap))
                $this->log("weekday($weekday) error");
            $weekday = self::weekdayMap[$weekday];
        }
        else if($weekday < 0 || $weekday > 6 || !is_integer($weekday))
            $this->log("weekday($weekday) error");

        try{
            $sql = "insert into Timetable_Adapter (`zone`, `classroom`, `usersn`, `weekday`, `section`, `sectionStart`, `sectionEnd`, `Sync`)
                  values (:zone, :classroom, :usersn, :weekday, :sectionStart-:sectionEnd, :sectionStart, :sectionEnd, 1) on duplicate key update `section`=values(`section`);";
            Yii::$app->db->createCommand($sql, [
                ':zone' => $zone,
                ':classroom' => $classroom,
                ':usersn' => $usersn,
                ':weekday' => $weekday,
                ':sectionStart' => $sectionStart,
                ':sectionEnd' => $sectionEnd
            ])->execute();
        }catch (Exception $e){
            echo "exception\n";
        }
    }

    public function clean(){
        echo "clean table\n";
        Yii::$app->db->createCommand("truncate table Timetable_Adapter;")->execute();
        Yii::$app->db->createCommand("delete * from Timetable where Sync = 1;")->execute();
    }

//    process data and insert into timetable
    public function rowProc($row) {
//        $this->insert($row['xq'], $weekday, $row['user_sn'], $classroom, $sectionStart, $sectionEnd);
    }

    public function weekCalc() {
        $bi = BaseInfo::findOne(['InfomationID' => 0x11c]);
        if(empty($bi) || empty($bi->Description)) {
            $this->weekInfo = null;
            $this->log("term open date error");
            return false;
        }
        else
            $this->weekInfo['openTerm'] = $bi->Description;

        $timeOpen =strtotime($this->weekInfo['openTerm']);
        $weekday = date('w', $timeOpen);
        $timeOpen -= (($weekday - 1) * 3600 * 24);
        $timeNow =strtotime(date('Y-m-d'));
        if($timeNow < $timeOpen) {
            $this->log("date error, term open:$timeOpen, now:$timeNow");
            return false;
        }
        $this->weekInfo['dayNow'] = floor(($timeNow - $timeOpen)/3600/24);
        $this->weekInfo['weekNow'] = floor($this->weekInfo['dayNow'] / 7);
        var_dump($this->weekInfo);
        return true;
    }

    public function sync()
    {
        $bi = BaseInfo::findOne(['InfomationID' => 0x118]);
        if(empty($bi) || empty($bi->InfoValue) || $bi->InfoValue == 0)
            return;
        if ($this->syncFlag == date('Y-m-d'))
            return;
        if (empty($this->sql))
            return;
        $this->dbReconnect();
        $this->tryCount++;
        $needClean = true;
        $rowCount = 0;
        if(!$this->weekCalc()) {
            $this->log("week calc failed");
            return;
        }
        try {
            if ($this->dbinfo['dbtype'] == 'mssql') {
            }
            else if ($this->dbinfo['dbtype'] == 'oracle/service') {
                $db = "(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=" . $this->dbinfo['ip'] . ")(PORT=" . $this->dbinfo['port'] . ")))(CONNECT_DATA=(SERVER_DEDICATED)(SERVICE_NAME=" . $this->dbinfo['database'] . ")(SID=" . $this->dbinfo['database'] . ")))";
                var_dump($this->dbinfo);
                $conn = oci_connect($this->dbinfo['username'], $this->dbinfo['password'], $db, 'zhs16gbk');
                if (!$conn) {
                    $e = oci_error();
                    trigger_error(htmlentities($e['message'], ENT_QUOTES), E_USER_ERROR);
                }
                $start = 0;
                $size = self::fetchPage;

                while (1) {
                    $end = $start + self::fetchPage;
                    $sql = str_replace(['{{start}}', '{{end}}', '{{size}}'], [$start, $end, $size], self::sql);
                    $stid = oci_parse($conn, $sql);
                    oci_execute($stid);
                    while ($row = oci_fetch_array($stid, OCI_ASSOC + OCI_RETURN_NULLS)) {
                        if ($needClean) {
                            $this->clean();
                            $needClean = false;
                        }
                        $this->rowProc($row);
                        $rowCount++;
                    }
                    if ($rowCount < $end)
                        break;
                    $start += self::fetchPage;
                }
            }
        }catch (ErrorException $e){
            $this->log("open oracle/service connection failed");
        }
        if($rowCount > 0) {
            $this->tryCount = 0;
            $this->syncFlag = date('Y-m-d');
            $this->log("sync success, fetch $rowCount rows");
        }
        if($this->tryCount == 480){
            Yii::$app->udp->timetableSyncError();
        }
    }
}
