<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\classes\Common;
use common\components\UdpService;
use common\models\DeviceList;
use common\models\SystemLog;
use common\models\Users;
use common\models\VDevice;
use Faker\Provider\Base;
use Workerman\Worker;
use Yii;

use common\classes\Error;
use common\classes\HttpRequest;
use common\classes\System;
use common\models\BaseInfo;
use Workerman\Lib\Timer;
use yii\db\Exception;
use yii\db\Query;

require_once __DIR__ . '/../Autoloader.php';
require_once  __DIR__ . '/../../common/classes/System.php';
require_once  __DIR__ . '/../../common/classes/HttpRequest.php';
require_once  __DIR__ . '/../../common/classes/Error.php';
require_once  __DIR__ . '/SystemDeamon.php';


class iDsteDeamon
{
    private $userMagic;
    private $baseInfo;
    private $deamonWorker;
    private $updateUserLastTime;
    private $proxyConnected = false;
    private $systemDeamon;
    private $dbBackupFlag = null;
    public function __construct($deamon){
        $this->userMagic = -1;
        $this->updateUserLastTime = 0;
        $this->baseInfo = null;
        $this->deamonWorker = $deamon;
        $deamon->onWorkerStart = [$this, 'workerStart'];
        shell_exec('chown www-data:www-data /dev/shm/yii2cache/ -R');
    }

    public function workerStart($worker){
        echo "idste worker start\n";
        if(file_exists(Common::FIRMWARE_DIR) && is_file(Common::FIRMWARE_DIR))
            rmdir(Common::FIRMWARE_DIR);
        if(!file_exists(Common::FIRMWARE_DIR))
            mkdir(Common::FIRMWARE_DIR, 0744, true);
        chown(Common::FIRMWARE_DIR, "www-data");
        chgrp(Common::FIRMWARE_DIR, "www-data");
        $this->dbReconnect();
        if($worker->id == 0)
            $this->idsteDeamonInit();
        else if($worker->id == 1)
            $this->systemDeamon = new SystemDeamon();
    }

    private function idsteDeamonInit(){
        $this->baseInfo['mac'] = System::getMac();
        while(true) {
            $this->baseInfo['uuid'] = Yii::$app->base->uuid;
            //uuid
            if (empty($this->baseInfo['uuid'])) {
                echo "unregisted server\n";
                sleep(30);
                continue;
            }
            $version = Yii::$app->base->version;
            $this->baseInfo['version'] = str_replace('.', '', $version);
            $this->baseInfo['server'] = 'https://' . Yii::$app->base->cloudServer;
            //secret_token
            $this->baseInfo['secret_token'] = Yii::$app->base->secretToken;
            //access_token
            $this->baseInfo['access_token'] = Yii::$app->base->accessToken;
            if (empty($this->baseInfo['access_token']))
                $this->baseInfo['access_token'] = 'empty';
            break;
        }
        $this->proxyAlive();
        Timer::add(30, [$this, 'proxyAlive'], []);
        Timer::add(300, [$this, 'dbReconnect']);
        Timer::add(2, [$this, 'userSync']);
        Timer::add(3600, [$this, 'checkProj']);
        Timer::add(600, [$this, 'dbBackup']);
//        Timer::add(60, [$this, 'deviceOfflineMonitorStart'], [], false);
//        Timer::add(60, [$this, 'deviceStatisticMonitorStart'], [], false);
        Timer::add(15, [$this, 'firmwareUpdate']);
        Timer::add(120, [$this, 'firmwareUpdate'], [], false);
        Timer::add(86400, [$this, 'firmwareListCheck']);
    }

    /**
     * reconnect every 300 sec
     */
    public function dbReconnect(){
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    /*
     * send confirm request
     */
    private function proxyConfirm($data){
        if(!isset($data->secret_token) || !isset($data->access_token) || !isset($data->origin))
            return;

        $arg = null;
        $arg['data']['secret_token'] = $data->secret_token;
        $arg['data']['access_token'] = $data->access_token;
        $req = new HttpRequest($this->baseInfo['server'] . '/api/school/token-confirm',
            ['uuid' => $this->baseInfo['uuid']]);
        $ret = $req->post(json_encode($arg), false);
        if(is_null($ret) || !isset($ret->status) || $ret->status !== 0) {
            echo "proxy confirm failed:" . json_encode($ret) . "\n";
            return;
        }

        $this->baseInfo = array_merge($this->baseInfo, $arg['data']);
        $db = \Yii::$app->db;
        $db->createCommand("insert into BaseInfo set `InfomationID` = 338, `Description`='" .$data->secret_token. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
        $db->createCommand("insert into BaseInfo set `InfomationID` = 339, `Description`='" .$data->access_token. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
        $db->createCommand("insert into BaseInfo set `InfomationID` = 340, `Description`='" .$data->origin. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
    }

    private function reloadToken(){
        $this->baseInfo['access_token'] = Yii::$app->base->accessToken;
        $this->baseInfo['secret_token'] = Yii::$app->base->secretToken;
    }
    /*
     * sync user data
     */
    public function userSync(){
        if(!isset($this->baseInfo['access_token']) || !$this->proxyConnected)
            return;
        $db = \Yii::$app->db;
        $magic = 0;
        $info = BaseInfo::findOne(['InfomationID' => 341]);
        if(empty($info)) {
            $db->createCommand("insert into `BaseInfo` (`InfomationID`, `InfoValue`) values (341, 0) on duplicate key update `InfoValue` = `InfoValue` + 1;")
                ->query();
        }
        else if($this->userMagic == $info->InfoValue && (time() - $this->updateUserLastTime) < 1800)
            return;
        else //userMagic unmatch or need force update
            $magic = $info->InfoValue;
        echo "user sync.\n";

        $users = (new Query())->select(['pid' => 'User_ID', 'username' => 'User_Name', 'fullname' => 'User_Desc', 'phone' => 'Phone', 'privilege'])
            ->from('Users')
            ->where(['not', ['Phone' => null]])
            ->all();
        $req = new HttpRequest($this->baseInfo['server'] . '/api/school/user-sync-v2', [
            'access_token' => $this->baseInfo['access_token'],
            'uuid' => $this->baseInfo['uuid']]);
        $out = $req->post(['data' => $users, 'magic' => $magic]);
        if(empty($out)) {
            echo "user sync error\n";
            return;
        }
        if(isset($out->status) && $out->status == 0){
            $this->updateUserLastTime = time();
            $this->userMagic = $magic;
        }
        else if($out->status && $out->status = 401)
            $this->reloadToken();
        else
            echo "user sync error:" . print_r($out, true) . "\n";
    }

    /*
     * kill ssh proxy progress
     */
    private function breakProxy(){
        $cmd = "ps aux | grep 'ssh -i " . __DIR__ . "/proxy_user.key' | awk '{print $2}' | xargs kill -9";
        system($cmd . ' > /dev/null');
    }
    /*
     * rebuild ssh proxy
     */
    private function proxyReconnect($data){
        //kill exist connect
        $this->breakProxy();
        //reconnect
        $cmd = 'ssh -i ' . __DIR__ . '/proxy_user.key -o ServerAliveInterval=60 -o ServerAliveCountMax=3 ';
        $cmd .= '-o StrictHostKeyChecking=no -NfR ';
        $cmd .= $data->port .':127.0.0.1:18080 proxy_user@'. $data->host;
        system($cmd . ' > /dev/null &');
        echo "proxy reconnect\n";
        \Workerman\Lib\Timer::add(10, [$this, 'proxyAlive'], [], false);
    }

    /*
     * proxy keepalive every 30sec
     */
    public function proxyAlive(){
        try{
            $db = \Yii::$app->db;
            $req = new HttpRequest($this->baseInfo['server'] . '/api/school/sync',
                ['uuid' => $this->baseInfo['uuid']]);
            $this->baseInfo['net'] = array_values(Yii::$app->net->ipView()['state']);
            $arg['data'] = $this->baseInfo;
            unset($arg['data']['secret_token']);
            $out = $req->post(json_encode($arg), false);
            $proxyConnected = false;
            if(is_null($out) || !isset($out->status)) {
                echo "proxy alive error\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='error', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
            }
            else if($out->status == 0) {
                $proxyConnected = true;
                if(isset($out->data))
                    $this->proxyConfirm($out->data, $arg['data']['uuid']);
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='success', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
                $this->userSync();
            }
            else if($out->status == Error::STATUS_PROXY_BROKEN) {
                echo "proxy broken\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='reconnect', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
                $this->proxyReconnect($out->data);
            }
            else if($out->status == Error::STATUS_PROXY_OFFLINE){
                echo "proxy offline\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
            }
            if($this->proxyConnected != $proxyConnected) {
                $this->proxyConnected = $proxyConnected;
            }
        }catch (Exception $e){
            $this->dbReconnect();
            Yii::error("proxy alive exception:" . print_r($e, true));
        }
    }

    /**
     * @return null
     */
    public function checkProj()
    {
        $devices = DeviceList::find()->select(['DevID', 'Dev_Name', 'ProjWatermark', 'ProjUsedTime'])
            ->where('ProjUsedTime > ProjWatermark')->all();
        $recordFilename = './projErrRecord.log';
        if (empty($devices)) {
            file_exists($recordFilename) && unlink($recordFilename);
            return;
        }
        $record = null;
        if (file_exists($recordFilename))
            $record = file_get_contents($recordFilename);
        if (empty($record))
            $record = [];
        else
            $record = unserialize($record);

        $newRecord = [];
        foreach ($devices as $device) {
            $newRecord[$device->DevID] = 1;
            if (isset($record[$device->DevID]))
                continue;
            Yii::$app->udp->projectorMsg($device->Dev_Name, $device->ProjWatermark, $device->ProjUsedTime);
        }
        file_put_contents($recordFilename, serialize($newRecord));
    }

    private $dbBackupTrycnt = 0;
    public function dbBackup() {
        if($this->dbBackupFlag == date('Y-m-d'))
            return;
        $this->dbReconnect();
        if(!file_exists(Common::MYSQL_BACKUP_DIR))
            mkdir(Common::MYSQL_BACKUP_DIR);
        $filename = date("Y-m-d-H-i-s_") . Yii::$app->security->generateRandomString(8) . ".sql";
        $filepath = Common::MYSQL_BACKUP_DIR . "/$filename";
        system("mysqldump -u root -piDste1057Hello -hlocalhost -R nccs > $filepath");

        $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_MYSQL_BACKUP_INFO]);
        if(empty($bi)) {
            $bi = new BaseInfo();
            $bi->InfomationID = Common::BASEINFO_MYSQL_BACKUP_INFO;
            $bi->InfoValue = 0;
        }
        $uuid = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SCHOOL_UUID]);
        if(empty($uuid) || empty($uuid->Description))
            $uuid = "idste";
        else
            $uuid = $uuid->Description;
        $tarFilepath = $filepath . ".tar.bz2";
        system("tar -zcf - $filepath | openssl des3 -salt -k $uuid | dd of=$tarFilepath");
        if(file_exists($tarFilepath) && filesize($tarFilepath) > 1024) {
            $this->dbBackupFlag = date('Y-m-d');
            $bi->Description = $filename . ".tar.bz2";
            $this->dbBackupTrycnt = 0;
        }
        else {
            $this->dbBackupTrycnt++;
        }
        unlink($filepath);
        //08:00 send notice mysql
        if($this->dbBackupTrycnt > 48)
            Yii::$app->udp->mysqldumpErrorMsg($bi->InfoValue);
        $bi->save();
    }

    private $defOfflineMsgSended = [];
    private $devOfflineMsgConfig = [];
    private $devOfflineLastGetInfo = 0;
    public function deviceOfflineMonitorStart() {
        Timer::add(1, [$this, 'deviceOfflineMonitor']);
        for($i = Common::BASEINFO_OFFLINE_WARNING_START; $i < Common::BASEINFO_OFFLINE_WARNING_END; $i++)
            $this->defOfflineMsgSended[$i] = 0x02;
    }

    //bit0 offline_warning send flag; bit1 offline recovery msg send flag
    public function deviceOfflineMonitor() {
        if(time() - $this->devOfflineLastGetInfo > 60) {
            $this->devOfflineLastGetInfo = time();
            $this->devOfflineMsgConfig = BaseInfo::find()
                ->select(['id' => 'InfomationID', 'watermark' => 'Description', 'uid' => 'InfoValue'])
                ->where('InfomationID >= ' . Common::BASEINFO_OFFLINE_WARNING_START . ' and InfomationID < ' . Common::BASEINFO_OFFLINE_WARNING_END)
                ->indexBy('id')
                ->asArray()
                ->all();
            foreach ($this->defOfflineMsgSended as $id => $value) {
                if(!array_key_exists($id, $this->devOfflineMsgConfig))
                    $this->defOfflineMsgSended[$id] = 0x00;
            }
        }
        $allCnt = DeviceList::find()->count();
        // TODO:以后需要重新处理这个在线判断
        $onlineCnt = 0; // DeviceStatus::find()->where(['not', ['IpAddress' => null]])->count();
        $offlineCnt = $allCnt - $onlineCnt;
        $receiver = [];
        foreach ($this->devOfflineMsgConfig as $id => $cfg) {
            if($offlineCnt < $cfg['watermark']) {
                if(($this->defOfflineMsgSended[$id] & 0x02) == 0x02)
                    continue;
                $this->defOfflineMsgSended[$id] = 0x02;
                if(!isset($receiver[$cfg['uid'] . 'r'])) {
                    $receiver[$cfg['uid'] . 'r'] = 1;
                    Yii::$app->udp->devOfflineWarningMsg('offline_recovery_warning', $cfg['uid'], $offlineCnt, $onlineCnt, $cfg['watermark']);
                }
                continue;
            }
            if(($this->defOfflineMsgSended[$id] & 0x01) == 0x01)
                continue;
            $this->defOfflineMsgSended[$id] = 0x01;
            if(!isset($receiver[$cfg['uid'] . 'w'])) {
                $receiver[$cfg['uid'] . 'w'] = 1;
                Yii::$app->udp->devOfflineWarningMsg('offline_warning', $cfg['uid'], $offlineCnt, $onlineCnt, $cfg['watermark']);
            }
        }
    }

    public $deviceStatisticConfig = [];
    public $deviceStatisticLastGetConfig = 0;
    public function deviceStatisticMonitorStart(){
        Timer::add(1, [$this, 'deviceStatisticMonitor']);
    }
    public function deviceStatisticMonitor() {
        $msgReceiver = [];
        if(time() - $this->deviceStatisticLastGetConfig > 60) {
            $this->deviceStatisticLastGetConfig = time();
            $this->deviceStatisticConfig = BaseInfo::find()
                ->where('`InfomationID` >= '. Common::BASEINFO_DEVICE_STATISTICS_START . ' && `InfomationID` < ' . Common::BASEINFO_DEVICE_STATISTICS_END)
                ->all();
        }
        $timenow = exec('date +%H:%M:%S');
        foreach ($this->deviceStatisticConfig as $bi) {
            if($timenow == $bi->Description && !is_null($bi->InfoValue))
                $msgReceiver[$bi->InfoValue] = 0;
        }
        if(count($msgReceiver) == 0)
            return;
        $allCnt = DeviceList::find()->count();
        $poweronCnt = VDevice::find()->where(['power' => 'on'])->count();
        $poweroffCnt = VDevice::find()->where(['power' => 'off'])->count();
        $disableCnt = VDevice::find()->where(['power' => 'disable'])->count();
        $offlineCnt = $allCnt - $poweronCnt - $poweroffCnt - $disableCnt;
        $uids = [];
        foreach ($msgReceiver as $uid => $value)
            $uids[] = $uid;
        $uidStr = implode(',', $uids);
        Yii::$app->udp->devStatisticMsg($uidStr, $poweronCnt, $poweroffCnt, $disableCnt, $offlineCnt);
    }

    private $firmware = [];
    private $downloadCmd = [];
    private $firmwareNameList = [];
    public function firmwareUpdate(){
        $path = Common::FIRMWARE_LIST_FILE;
        if (file_exists($path)) {
            $list = file_get_contents(Common::FIRMWARE_LIST_FILE);
            if (!empty($list)) {
                $list = unserialize($list);
                if (empty($list) || !is_array($list)) {
                    $list = [];
                }
            } else {
                $list = [];
            }
            if (count($list) > 0) {
                $upgradeList = [];
                if (file_exists(Common::FIRMWARE_UPGRADE_LIST_FILE)) {
                    $upgradeList = file_get_contents(Common::FIRMWARE_UPGRADE_LIST_FILE);
                    if (!empty($upgradeList)) {
                        $upgradeList = unserialize($upgradeList);
                        if (empty($upgradeList) || !is_array($upgradeList)) {
                            $upgradeList = [];
                        }
                    } else {
                        $upgradeList = [];
                    }
                }
                foreach ($list as $key1 => $item) {
                    if (count($upgradeList) > 0) {
                        foreach ($upgradeList as $key => $value) {
                            if (file_exists($value['file']) && md5_file($value['file']) == $item['md5'] && $value['needUpgrade']) {
                                $device = VDevice::findOne(['id' => $key]);
                                if(!empty($device) && !empty($device->ipaddress)) {
                                    Yii::$app->sendcmd2cmpn->sendCmd('devupgrade', $device->id, ['url' => '/firmware/' . $item['filename']]);
                                }
                                $value['needUpgrade'] = false;
                                $upgradeList[$key] = $value;
                            }
                        }
                        file_put_contents(Common::FIRMWARE_UPGRADE_LIST_FILE, serialize($upgradeList));
                    }
                }
            }
        }

    }
    public function firmwareListCheck(){
        $req = new HttpRequest($this->baseInfo['server'] . '/api/site/version-list-v2');
        $content = $req->get(true);
        if(empty($content) || !array_key_exists('status', $content) || $content['status'] != 0 || empty($content['data']) || empty($content['data']['list']))
            return;
        foreach ($content['data']['list'] as $item) {
            if(array_key_exists($item['id'], $this->firmware) && array_key_exists($item['filename'], $this->downloadCmd) && array_key_exists($item['filename'], $this->firmwareNameList) && $item['md5'] == $this->firmware[$item['id']]['md5'])
                continue;
            $file = Common::FIRMWARE_DIR . '/' . urlencode($item['filename']);
            if(!empty($content['baseurl']))
                $path = $content['baseurl'] . urlencode($item['filename']);
            else
                $path = $this->baseInfo['server'].'/firmware/' . urlencode($item['filename']);
            $cmd = "wget -b $path -O $file &";
            echo "cmd:$cmd";
            $this->downloadCmd[$item['filename']] = $cmd;
            $firmware[$item['id']] = $item;
            $this->firmwareNameList[$item['filename']] = $item;
        }
        file_put_contents(Common::FIRMWARE_WGET_LIST_FILE, serialize($this->downloadCmd));
        file_put_contents(Common::FIRMWARE_NAME_LIST_FILE, serialize($this->firmwareNameList));
        file_put_contents(Common::FIRMWARE_LIST_FILE, serialize($firmware));
    }
}
