﻿-- MySQL Workbench Forward Engineering

SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

-- -----------------------------------------------------
-- Schema mydb
-- -----------------------------------------------------
-- -----------------------------------------------------
-- Schema nccs
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Schema nccs
-- -----------------------------------------------------
CREATE SCHEMA IF NOT EXISTS `nccs` DEFAULT CHARACTER SET utf8 ;
USE `nccs` ;

-- -----------------------------------------------------
-- Table `nccs`.`AlternateDate`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`AlternateDate` ;

CREATE TABLE IF NOT EXISTS `nccs`.`AlternateDate` (
  `SpecialDate` DATE NOT NULL,
  `AlteredScheduleDay` TINYINT(4) NULL DEFAULT NULL,
  PRIMARY KEY (`SpecialDate`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`BaseInfo`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`BaseInfo` ;

CREATE TABLE IF NOT EXISTS `nccs`.`BaseInfo` (
  `InfomationID` INT(11) NOT NULL DEFAULT '0' COMMENT '0~0x04 : air condition command description;\n0x80: User name;\n0x100: MediaFile auto_increasement value;\n0x101 Active task template;\n0x200: Version;',
  `Description` VARCHAR(1024) NULL DEFAULT NULL,
  `InfoValue` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`InfomationID`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`BroadcastMode`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`BroadcastMode` ;

CREATE TABLE IF NOT EXISTS `nccs`.`BroadcastMode` (
  `idBroadcastMode` INT(11) NOT NULL AUTO_INCREMENT,
  `ModeName` VARCHAR(50) NULL DEFAULT NULL,
  `Priority` INT(11) NOT NULL DEFAULT '10',
  `Enabled` TINYINT(1) NULL DEFAULT '0',
  `StartTime` DATETIME NULL DEFAULT NULL,
  `StopTime` DATETIME NULL DEFAULT NULL,
  `TimeStyle` TINYINT(4) NULL DEFAULT '0' COMMENT '0: means for one day\n   the start date and end date is indicated by starttime and stoptime.\n\nother: means the mode should be repeated weekly.\nbit 0: means sunday\nbit1:  monday\nbit2: tuesday\n...',
  `Active` TINYINT(1) NOT NULL DEFAULT '0',
  `StartEntryID` INT(11) NOT NULL DEFAULT '0',
  `StopEntryID` INT(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`idBroadcastMode`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DevClasses`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DevClasses` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DevClasses` (
  `ClassID` INT(11) NOT NULL AUTO_INCREMENT,
  `Name` VARCHAR(50) NULL DEFAULT NULL,
  `Parent` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`ClassID`),
  INDEX `ClassParent` (`Parent` ASC),
  CONSTRAINT `ClassParent`
    FOREIGN KEY (`Parent`)
    REFERENCES `nccs`.`DevClasses` (`ClassID`)
    ON DELETE SET NULL
    ON UPDATE SET NULL)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceList`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceList` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceList` (
  `DevID` VARCHAR(50) NOT NULL,
  `Dev_Name` VARCHAR(32) NULL DEFAULT NULL,
  `Parent` INT(11) NULL DEFAULT NULL,
  `DisableParentSchedule` TINYINT(4) NULL DEFAULT '0' COMMENT 'Indicate whether disable inheritting schedule setting from parents groups.',
  `Dev_Description` VARBINARY(96) NULL DEFAULT NULL,
  `Dev_Configuration` VARBINARY(2048) NULL DEFAULT NULL,
  `RtspUrl` VARCHAR(512) NULL DEFAULT NULL,
  `RtspSub` VARCHAR(512) NULL DEFAULT NULL,
  `RtspUrl1` VARCHAR(512) NULL DEFAULT NULL,
  `RtspSub1` VARCHAR(512) NULL DEFAULT NULL,
  `ProjWatermark` INT(10) UNSIGNED NOT NULL DEFAULT '360000000',
  `ProjUsedTime` INT(10) UNSIGNED NOT NULL DEFAULT '0',
  `Dev_ConfigurationV2` VARCHAR(16384) NULL DEFAULT NULL,
  `PcIp` VARCHAR(41) NULL DEFAULT NULL,
  `PcPort` INT(10) UNSIGNED NULL DEFAULT NULL,
  `PcMainPasswd` VARCHAR(32) NULL DEFAULT NULL,
  `PcViewPasswd` VARCHAR(32) NULL DEFAULT NULL,
  `RfInvalid` INT(10) UNSIGNED NULL DEFAULT '0',
  `Ip` VARCHAR(50) NULL DEFAULT NULL,
  `Proto` INT(10) NULL DEFAULT '1',
  `DevType` int(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`DevID`),
  UNIQUE INDEX `DevID_UNIQUE` (`DevID` ASC),
  INDEX `DevClass` (`Parent` ASC),
  CONSTRAINT `DevClass`
    FOREIGN KEY (`Parent`)
    REFERENCES `nccs`.`DevClasses` (`ClassID`)
    ON DELETE SET NULL
    ON UPDATE SET NULL)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`BrdModeMember`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`BrdModeMember` ;

CREATE TABLE IF NOT EXISTS `nccs`.`BrdModeMember` (
  `DevIDMode` INT(11) NOT NULL AUTO_INCREMENT,
  `BrdModeID` INT(11) NOT NULL,
  `DeviceID` VARCHAR(50) NOT NULL,
  PRIMARY KEY (`DevIDMode`),
  UNIQUE INDEX `DevIDMode_UNIQUE` (`BrdModeID` ASC, `DeviceID` ASC),
  INDEX `BrdModeID` (`BrdModeID` ASC),
  INDEX `ModeDeviceID` (`DeviceID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`CallWaiter`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`CallWaiter` ;

CREATE TABLE IF NOT EXISTS `nccs`.`CallWaiter` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `user` INT(11) NOT NULL,
  `session` VARCHAR(45) NOT NULL,
  `updateAt` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `session_unique` (`session` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DevCtrlSchedule`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DevCtrlSchedule` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DevCtrlSchedule` (
  `ScheduleID` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `DevID` VARCHAR(50) NOT NULL,
  `ActionTime` TIME NOT NULL,
  `Action` INT(11) NULL DEFAULT NULL,
  `WeekDay` TINYINT(4) NOT NULL,
  `State` INT(11) NULL DEFAULT '0',
  `ExtArgs` VARCHAR(256) NULL DEFAULT NULL,
  `Disable` TINYINT(1) NULL DEFAULT '0',
  PRIMARY KEY (`DevID`, `ActionTime`, `WeekDay`),
  UNIQUE INDEX `ScheduleID_UNIQUE` (`ScheduleID` ASC),
  INDEX `DeviD` (`DevID` ASC),
  INDEX `DeviceID` (`DevID` ASC),
  INDEX `DevCtrlDevId` (`DevID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceDesiredStaus`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceDesiredStaus` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceDesiredStaus` (
  `DeviceID` VARCHAR(50) NOT NULL,
  `SessionID` SMALLINT(5) UNSIGNED NULL DEFAULT NULL,
  `BrdPort` SMALLINT(6) NULL DEFAULT NULL,
  `SessionValid` TINYINT(1) NULL DEFAULT '1',
  `SessionType` TINYINT(4) NULL DEFAULT NULL,
  `StreamType` TINYINT(4) UNSIGNED NULL DEFAULT NULL,
  `SysEnabled` TINYINT(4) NULL DEFAULT NULL,
  `PowerAction` TINYINT(4) NULL DEFAULT NULL,
  `RfDevCtrl` BINARY(16) NULL DEFAULT NULL,
  `RfCtrlAction` BINARY(16) NULL DEFAULT NULL,
  `IpCall` TINYINT(1) NOT NULL DEFAULT '0' COMMENT '0: idle\n1:ringing\n2:talking',
  `IpCallPort` INT(11) NULL DEFAULT NULL,
  `IpCallEvent` DATETIME NULL DEFAULT NULL,
  PRIMARY KEY (`DeviceID`),
  INDEX `DesiredDevicesID` (`DeviceID` ASC),
  INDEX `DesiredSessionID` (`SessionID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceExt`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `nccs`.`DeviceExt` (
  `DevID` VARCHAR(50) NOT NULL,
  `Name` VARCHAR(45) NOT NULL,
  `Config` VARCHAR(2048) NOT NULL DEFAULT '{}',
  UNIQUE INDEX `DeviceExt_uni` (`DevID` ASC, `Name` ASC),
  INDEX `DeviceExt_idx` (`DevID` ASC, `Name` ASC),
  CONSTRAINT `DevID`
    FOREIGN KEY (`DevID`)
    REFERENCES `nccs`.`DeviceList` (`DevID`)
    ON DELETE CASCADE
    ON UPDATE CASCADE)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = '设备扩展信息表';


-- -----------------------------------------------------
-- Table `nccs`.`DeviceLog`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceLog` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceLog` (
  `DevLogID` BIGINT(20) NOT NULL AUTO_INCREMENT,
  `DeviceID` VARCHAR(50) NULL DEFAULT NULL,
  `DateTime` DATETIME NULL DEFAULT NULL,
  `Event` TINYINT(4) NULL DEFAULT NULL COMMENT '1  offline/online\n2  power on/power off\n3  enabled/  disabled\n4  Normal Text Message',
  `CtrlUser` INT(11) NULL DEFAULT NULL,
  `MsgContext` VARCHAR(256) NULL DEFAULT NULL,
  `EndTime` DATETIME NULL DEFAULT NULL,
  PRIMARY KEY (`DevLogID`),
  INDEX `device_idx` (`DeviceID` ASC, `Event` ASC),
  INDEX `endTime_idx` USING BTREE (`EndTime` ASC),
  INDEX `dateTime_idx` USING BTREE (`DateTime` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = 'Devices log';


-- -----------------------------------------------------
-- Table `nccs`.`DeviceCtrlList`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceCtrlList` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceCtrlList` (
  `id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `DevID` VARCHAR(50) NOT NULL,
  `SubID` INT(11) NOT NULL,
  `State` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `index` USING BTREE (`DevID` ASC, `SubID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceStatistic`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceStatistic` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceStatistic` (
  `DeviceID` VARCHAR(50) NOT NULL,
  `StatisticDay` DATE NOT NULL,
  `PowerConsumed` BINARY(128) NULL DEFAULT NULL,
  `UsedTime` BINARY(128) NULL DEFAULT NULL,
  PRIMARY KEY (`DeviceID`, `StatisticDay`),
  INDEX `StatisticDay` USING BTREE (`StatisticDay` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceStatus`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceStatus` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceStatus` (
  `DeviceID` VARCHAR(50) NOT NULL,
  `DeviceType` TINYINT(4) NULL DEFAULT '0',
  `DeviceStatus` INT(11) NULL DEFAULT '0',
  `IpAddress` VARCHAR(50) NULL DEFAULT NULL,
  `ShouldSessionID` SMALLINT(6) UNSIGNED NULL DEFAULT '0',
  `ActualSessionID` SMALLINT(6) UNSIGNED NULL DEFAULT '0',
  `State_SysPower` TINYINT(4) NULL DEFAULT '0',
  `State_ProjectorStatus` TINYINT(4) NULL DEFAULT '0',
  `State_ProjectorLampStatus` TINYINT(4) NULL DEFAULT '0',
  `State_ExDevice1Status` TINYINT(4) NULL DEFAULT '0',
  `State_ExDevice2Status` TINYINT(4) NULL DEFAULT '0',
  `State_CurrProjectorSrc` TINYINT(4) NULL DEFAULT '0',
  `State_CurrAudio` TINYINT(4) NULL DEFAULT '0',
  `State_CurrAudioVolume` TINYINT(4) NULL DEFAULT '0',
  `State_CurrMicVolume` TINYINT(4) NULL DEFAULT '0',
  `State_FaultStatus` TINYINT(4) NULL DEFAULT '0',
  `State_MicState` TINYINT(4) NULL DEFAULT '0',
  `State_Mute` TINYINT(4) NULL DEFAULT '0',
  `State_CentralUsedTime` INT(11) NULL DEFAULT '0',
  `State_ProjectorUsedTime` INT(11) NULL DEFAULT '0',
  `State_LightUsedTime` INT(11) NULL DEFAULT '0',
  `Cfg_System` TINYINT(4) NULL DEFAULT NULL,
  `Cfg_Remark` VARBINARY(96) NULL DEFAULT NULL,
  `netid` TINYINT(4) NULL DEFAULT '-1',
  `NetworkID` INT(11) NULL DEFAULT '0',
  `BrdPort` INT(11) NULL DEFAULT NULL,
  `RfDevStatus` BINARY(16) NULL DEFAULT NULL,
  `State_Pc` TINYINT(4) NULL DEFAULT '0',
  `User_ID` INT(11) NULL DEFAULT NULL,
  `SysPowerCmd` TINYINT(4) NULL DEFAULT NULL COMMENT '0:power off; 1:power on',
  `IpCallStatus` TINYINT(4) UNSIGNED NULL DEFAULT '0',
  `Version` VARCHAR(32) NULL DEFAULT NULL,
  `Temperature` INT(10) UNSIGNED NULL DEFAULT '0',
  `Humidity` INT(10) UNSIGNED NULL DEFAULT '0',
  `Trouble` INT(10) UNSIGNED NULL DEFAULT NULL,
  `Recorder` INT(10) UNSIGNED NULL DEFAULT NULL,
  `Scene` INT(10) UNSIGNED NULL DEFAULT NULL,
  `Protocol` INT(10) NULL DEFAULT '1',
  PRIMARY KEY (`DeviceID`),
  UNIQUE INDEX `DeviceID_UNIQUE` (`DeviceID` ASC),
  INDEX `DevID` (`DeviceID` ASC),
  INDEX `DevicestatusNetworkID` (`NetworkID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`DeviceStatusExt`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DeviceStatusExt` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DeviceStatusExt` (
  `ID` VARCHAR(50) NOT NULL COMMENT 'extended device id',
  `Name` VARCHAR(50) NOT NULL COMMENT 'status name',
  `MainID` VARCHAR(50) NOT NULL COMMENT 'main device id',
  `Status` VARCHAR(1024) NOT NULL DEFAULT '{}' COMMENT 'extened device\'s status, json format',
  `UpdateAt` DATETIME NOT NULL,
  UNIQUE INDEX `DeviceStatusExtUni` (`MainID` ASC, `Name` ASC),
  INDEX `DeviceStatusExtIdx` (`MainID` ASC, `ID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Players`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Players` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Players` (
  `PlayerID` VARCHAR(50) NOT NULL,
  `PlayerName` VARBINARY(32) NULL DEFAULT NULL,
  `PlayerType` TINYINT(4) NULL DEFAULT NULL COMMENT '0: celt audio player\n8: vlc player',
  `Priority` TINYINT(4) NULL DEFAULT '10',
  `Remark` VARBINARY(96) NULL DEFAULT NULL,
  PRIMARY KEY (`PlayerID`),
  UNIQUE INDEX `ID_UNIQUE` (`PlayerID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Users`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Users` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Users` (
  `User_ID` INT(11) NOT NULL AUTO_INCREMENT,
  `User_Name` VARCHAR(255) NULL DEFAULT NULL,
  `User_Desc` VARCHAR(255) NULL DEFAULT NULL,
  `Pass_Word` VARCHAR(256) NULL DEFAULT 'password, encode with PASSWORD()',
  `UsbKey` VARCHAR(50) NULL DEFAULT NULL COMMENT 'when the user\'s authorization is expired.',
  `Priority` INT(11) NULL DEFAULT '100',
  `privilege` INT(11) NOT NULL DEFAULT '0' COMMENT 'Bit 0: privilege of device control.\nBit1~Bit7: Reservered\n\nBit 8: privilege of creating audio and vedio broadcast tasks.\nBit9~Bit15: Reservered\n',
  `IcCard` VARCHAR(50) NULL DEFAULT NULL,
  `Phone` VARCHAR(50) NULL DEFAULT NULL,
  `ServerSync` TINYINT(4) NULL DEFAULT '1' COMMENT 'Flag for sync data to cloud server, set when data is inserted or updated, clear after the changed data sysnc to cloud server.',
  `User_Type` TINYINT(4) NULL DEFAULT '3' COMMENT '用户类型：\n1：学生\n2：教师\n3：临时人员',
  `User_Sn` VARCHAR(20) NULL DEFAULT NULL COMMENT '学号或工号',
  `User_Expiry` DATETIME NULL DEFAULT NULL COMMENT '有效期\nUser_Type=3时需要检查用户过期时间\n时间为0表示不过期',
  `Mgr_Privilege` TINYINT(1) NULL DEFAULT '0' COMMENT '设备管理权限标志\n0：具有部分权限，需要查询UserDevices表\n1：具有全部权限，无需查询UserDevices表',
  `Dev_Privilege` TINYINT(1) NULL DEFAULT '0' COMMENT '设备使用权限标志\n0：具有部分权限，需要查询UserDevices表\n1：具有全部权限，无需查询UserDevices表',
  `Create_Type` TINYINT(1) NULL DEFAULT '0' COMMENT '用户创建类型\n0：手动创建\n1：自动(IC卡同步)创建\n',
  `IC_Status` TINYINT(1) NULL DEFAULT '0' COMMENT 'IC卡状态\n0：IC卡正常\n1：IC卡挂失',
  `Dpm_Sn` VARCHAR(20) NULL DEFAULT NULL COMMENT '部门编号',
  `Dpm_Name` VARCHAR(100) NULL DEFAULT NULL COMMENT '部门名称',
  `Del_Flag` TINYINT(1) NULL DEFAULT '0' COMMENT 'delete flag:\n0: normal\n1: deleted',
  `UnionId` VARCHAR(64) NULL DEFAULT NULL,
  PRIMARY KEY (`User_ID`),
  UNIQUE INDEX `UsbKey_UNIQUE` (`UsbKey` ASC),
  UNIQUE INDEX `User_Name_UNIQUE` (`User_Name` ASC),
  UNIQUE INDEX `IcCard_UNIQUE` (`IcCard` ASC),
  UNIQUE INDEX `User_Sn_UNIQUE` (`User_Sn` ASC),
  UNIQUE INDEX `UnionId_uni` (`UnionId` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`TaskTemplate`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`TaskTemplate` ;

CREATE TABLE IF NOT EXISTS `nccs`.`TaskTemplate` (
  `idTaskTemplate` INT(11) NOT NULL AUTO_INCREMENT,
  `TemplateName` VARCHAR(45) NULL DEFAULT NULL,
  `Enabled` TINYINT(4) NULL DEFAULT '0',
  PRIMARY KEY (`idTaskTemplate`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlaySessions`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlaySessions` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlaySessions` (
  `SessionID` SMALLINT(6) UNSIGNED NOT NULL DEFAULT '0',
  `SessionName` VARCHAR(50) NOT NULL,
  `SessionType` TINYINT(4) NULL DEFAULT NULL COMMENT '0: automatic broadcast task\n1: player created task\n2: manually played task\n10:BW803 player created task',
  `Priority` TINYINT(4) NULL DEFAULT NULL,
  `Status` TINYINT(4) NOT NULL DEFAULT '0',
  `CircleLength` INT(11) NOT NULL DEFAULT '0',
  `ClassID` INT(11) NULL DEFAULT NULL,
  `PlayerID` VARCHAR(50) NULL DEFAULT NULL,
  `Parent` SMALLINT(5) UNSIGNED NULL DEFAULT NULL,
  `BrdMode` INT(11) NULL DEFAULT NULL,
  `MemberMode` TINYINT(1) NOT NULL DEFAULT '0',
  `StreamType` TINYINT(4) UNSIGNED NOT NULL DEFAULT '1' COMMENT 'Stream Type: \n1: valid 0: invalid\nBit 0: audio\nBit1: Video',
  `SysEnabled` TINYINT(4) NULL DEFAULT NULL,
  `RfDevCtrl` BINARY(16) NULL DEFAULT NULL,
  `Owner` INT(11) NULL DEFAULT NULL,
  `PlayMode` TINYINT(4) NULL DEFAULT '0' COMMENT '0: Sequential Playing',
  `AudioGain` TINYINT(4) NULL DEFAULT '0',
  `Template` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`SessionID`),
  UNIQUE INDEX `SessionID_UNIQUE` (`SessionID` ASC),
  INDEX `Players` (`PlayerID` ASC),
  INDEX `SessionParent` (`Parent` ASC),
  INDEX `SessionMode` (`BrdMode` ASC),
  INDEX `SessionUser` (`Owner` ASC),
  INDEX `TaskTemplate_idx` (`Template` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlaySessionTime`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlaySessionTime` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlaySessionTime` (
  `ScheduleID` INT(11) NOT NULL AUTO_INCREMENT,
  `SessionID` SMALLINT(6) UNSIGNED NOT NULL,
  `WeekDay` TINYINT(4) NOT NULL,
  `PlayTime` DATETIME NOT NULL,
  `TempFlag` TINYINT(4) NULL DEFAULT NULL,
  `Length` INT(11) NULL DEFAULT NULL,
  `EntryID` INT(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`ScheduleID`),
  UNIQUE INDEX `SesTime_UNIQUE` (`SessionID` ASC, `WeekDay` ASC, `PlayTime` ASC),
  INDEX `Schsessionid` (`SessionID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = 'Auto broadcast playing schedules';


-- -----------------------------------------------------
-- Table `nccs`.`DisableDevSession`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`DisableDevSession` ;

CREATE TABLE IF NOT EXISTS `nccs`.`DisableDevSession` (
  `ScheduleID` INT(11) NOT NULL,
  `DeviceID` VARCHAR(50) NOT NULL,
  `Disabled` TINYINT(4) NULL DEFAULT NULL,
  PRIMARY KEY (`ScheduleID`, `DeviceID`),
  INDEX `DisDevShedule` (`ScheduleID` ASC),
  INDEX `DisabledScheDevice` (`DeviceID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`EventModel`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`EventModel` ;

CREATE TABLE IF NOT EXISTS `nccs`.`EventModel` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `Manufacturer` VARCHAR(31) NOT NULL,
  `Model` VARCHAR(31) NOT NULL,
  `Magic` INT(11) UNSIGNED NOT NULL,
  PRIMARY KEY (`id`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Event`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Event` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Event` (
  `Id` INT(11) NOT NULL AUTO_INCREMENT,
  `Name` VARCHAR(31) NOT NULL,
  `StartTask` SMALLINT(6) UNSIGNED NULL DEFAULT NULL,
  `EndTask` SMALLINT(6) UNSIGNED NULL DEFAULT NULL,
  `Config` VARBINARY(64) NOT NULL COMMENT '事件配置',
  `Model` INT(11) NOT NULL COMMENT '型号',
  `Version` INT(11) NOT NULL DEFAULT '0' COMMENT '版本号:自动生成，前端禁止操作',
  PRIMARY KEY (`Id`),
  INDEX `model_idx` (`Model` ASC),
  INDEX `event_end_fkn_idx` (`EndTask` ASC),
  INDEX `event_start_fkn_idx` (`StartTask` ASC),
  CONSTRAINT `EventModel`
    FOREIGN KEY (`Model`)
    REFERENCES `nccs`.`EventModel` (`id`),
  CONSTRAINT `event_end_fkn`
    FOREIGN KEY (`EndTask`)
    REFERENCES `nccs`.`PlaySessions` (`SessionID`)
    ON DELETE SET NULL
    ON UPDATE NO ACTION,
  CONSTRAINT `event_start_fkn`
    FOREIGN KEY (`StartTask`)
    REFERENCES `nccs`.`PlaySessions` (`SessionID`)
    ON DELETE SET NULL
    ON UPDATE NO ACTION)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`GroupCtrlSchedule`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`GroupCtrlSchedule` ;

CREATE TABLE IF NOT EXISTS `nccs`.`GroupCtrlSchedule` (
  `ScheduleID` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `GroupID` INT(11) NOT NULL,
  `ActionTime` TIME NOT NULL,
  `Action` INT(11) NULL DEFAULT NULL,
  `WeekDay` TINYINT(4) NOT NULL,
  `State` INT(11) NULL DEFAULT '0',
  `ExtArgs` VARCHAR(256) NULL DEFAULT NULL,
  `Disable` TINYINT(1) NULL DEFAULT '0',
  PRIMARY KEY (`GroupID`, `ActionTime`, `WeekDay`),
  UNIQUE INDEX `ScheduleID_UNIQUE` (`ScheduleID` ASC),
  INDEX `DeviD` (`GroupID` ASC),
  INDEX `DeviceID` (`GroupID` ASC),
  INDEX `DevCtrlGroupID` (`GroupID` ASC),
  CONSTRAINT `DevCtrlGroupID`
    FOREIGN KEY (`GroupID`)
    REFERENCES `nccs`.`DevClasses` (`ClassID`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`IpCall`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`IpCall` ;

CREATE TABLE IF NOT EXISTS `nccs`.`IpCall` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `device` VARCHAR(45) NULL DEFAULT NULL COMMENT 'device id',
  `waiter` INT(11) NULL DEFAULT NULL,
  `audioPort` INT(11) NULL DEFAULT NULL,
  `ringing` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
  `status` ENUM('ringing', 'talking', 'idle') NULL DEFAULT 'ringing',
  `direction` ENUM('b2s', 'd2s') NULL DEFAULT 'b2s',
  `webrtcUpdate` TIMESTAMP NULL DEFAULT NULL,
  `webrtcStatus` ENUM('running', 'error', 'ready', 'idle') NULL DEFAULT 'idle',
  `wsPort` INT(11) NULL DEFAULT NULL COMMENT 'websocket listen port',
  `listen` TINYINT(4) NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`),
  UNIQUE INDEX `port_unique` (`audioPort` ASC),
  UNIQUE INDEX `webrtcPort_UNIQUE` (`wsPort` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`IpCameraRtmp`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`IpCameraRtmp` ;

CREATE TABLE IF NOT EXISTS `nccs`.`IpCameraRtmp` (
  `name` VARCHAR(128) NULL DEFAULT NULL COMMENT 'key经md5生成',
  `key` VARCHAR(45) NOT NULL COMMENT 'rtmp节目key值，由摄像头Ip/端口/通道/主副码(main或sub)',
  `pushId` INT(11) NOT NULL DEFAULT '-1' COMMENT 'ffmpeg推流ID',
  `pullCnt` INT(11) NOT NULL DEFAULT '0' COMMENT 'rtmp拉流计数',
  `updateAt` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`key`),
  UNIQUE INDEX `key_UNIQUE` (`key` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`LastProcessedTime`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`LastProcessedTime` ;

CREATE TABLE IF NOT EXISTS `nccs`.`LastProcessedTime` (
  `LastTimeID` INT(11) NOT NULL DEFAULT '1' COMMENT 'Only one record in this table',
  `LastTime` DATETIME NULL DEFAULT NULL,
  PRIMARY KEY (`LastTimeID`))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8
COMMENT = 'a globe time which record last timer process time.';


-- -----------------------------------------------------
-- Table `nccs`.`MediaFiles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`MediaFiles` ;

CREATE TABLE IF NOT EXISTS `nccs`.`MediaFiles` (
  `MediaID` INT(11) NOT NULL AUTO_INCREMENT,
  `FileName` VARCHAR(64) NULL DEFAULT NULL,
  `FileSize` BIGINT(20) NULL DEFAULT NULL,
  `TotalTime` INT(11) NOT NULL DEFAULT '0',
  `FileFmt` INT(11) NULL DEFAULT NULL COMMENT '0: celt\n1: mp3\n2: ts\n3: mp4, mov',
  `BitRate` INT(11) NULL DEFAULT NULL,
  `VedioCodec` INT(11) NULL DEFAULT NULL COMMENT '0: h264\n1: mpeg4',
  `Height` INT(11) NULL DEFAULT NULL,
  `Width` INT(11) NULL DEFAULT NULL,
  `FrameRate` INT(11) NULL DEFAULT NULL,
  `AudioCodec` INT(11) NULL DEFAULT NULL COMMENT '0: celt\n1: mp3',
  `Channels` INT(11) NULL DEFAULT NULL,
  `SampleRate` INT(11) NULL DEFAULT NULL,
  `Parent` INT(11) NULL DEFAULT NULL,
  `Gain` TINYINT(4) NULL DEFAULT '0',
  `CreateTime` DATETIME NULL DEFAULT NULL,
  `FilePath` VARCHAR(256) NULL DEFAULT NULL,
  `Owner` INT(11) NULL DEFAULT NULL COMMENT 'sysfolder owner',
  `Sysfolder` TINYINT(4) NULL DEFAULT NULL COMMENT 'system create folder,not update or delete\n1:true',
  `Check` TINYINT(4) NOT NULL DEFAULT '0',
  PRIMARY KEY (`MediaID`),
  UNIQUE INDEX `ID_UNIQUE` (`MediaID` ASC),
  INDEX `MediaParent` (`Parent` ASC),
  INDEX `MediaUser_idx` (`Owner` ASC),
  CONSTRAINT `MediaParent`
    FOREIGN KEY (`Parent`)
    REFERENCES `nccs`.`MediaFiles` (`MediaID`)
    ON DELETE SET NULL
    ON UPDATE SET NULL)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Messages`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Messages` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Messages` (
  `ID` INT(11) NOT NULL AUTO_INCREMENT,
  `MsgFrom` TINYINT(4) NOT NULL,
  `MsgTo` TINYINT(4) NOT NULL,
  `MsgType` TINYINT(4) NULL DEFAULT NULL,
  `MsgTime` DATETIME NULL DEFAULT NULL,
  `CmdType` INT(11) NULL DEFAULT NULL,
  `CmdObjType` INT(11) NULL DEFAULT NULL,
  `CmdObjID` VARCHAR(50) NULL DEFAULT NULL,
  `CmdObjIP` VARCHAR(50) NULL DEFAULT NULL,
  PRIMARY KEY (`ID`),
  UNIQUE INDEX `ID_UNIQUE` (`ID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`MsgTasks`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`MsgTasks` ;

CREATE TABLE IF NOT EXISTS `nccs`.`MsgTasks` (
  `MsgTaskID` INT(11) NOT NULL,
  `PlayedTime` DATETIME NULL DEFAULT NULL,
  `TimeStyle` INT(11) NULL DEFAULT NULL COMMENT '0: Play one time.\n    played on PlayedTime.\nother: repeat weekly',
  `Duration` INT(11) NULL DEFAULT NULL,
  `Content` VARCHAR(256) NULL DEFAULT NULL,
  PRIMARY KEY (`MsgTaskID`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`MsgTaskDev`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`MsgTaskDev` ;

CREATE TABLE IF NOT EXISTS `nccs`.`MsgTaskDev` (
  `MsgTaskID` INT(11) NOT NULL,
  `DevID` VARCHAR(45) NOT NULL,
  PRIMARY KEY (`MsgTaskID`, `DevID`),
  INDEX `msgtask` (`MsgTaskID` ASC),
  INDEX `msgtaskdev` (`DevID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`NotificationMsg`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`NotificationMsg` ;

CREATE TABLE IF NOT EXISTS `nccs`.`NotificationMsg` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `uid` INT(11) NOT NULL COMMENT '用户ID',
  `type` VARCHAR(32) NOT NULL COMMENT '事件类型',
  `time` DATETIME NULL DEFAULT NULL COMMENT '消息推送的时间',
  `ipaddress` VARCHAR(45) NULL DEFAULT NULL COMMENT '消息推送的ip地址',
  `description` VARCHAR(4096) NOT NULL COMMENT '推送的具体消息内容.',
  `update_at` DATETIME NULL DEFAULT NULL,
  `handle_uid` INT(11) NULL DEFAULT NULL,
  `status` ENUM('PENDING', 'RESOLVED', 'REJECTED') NOT NULL DEFAULT 'PENDING',
  PRIMARY KEY (`id`),
  UNIQUE INDEX `id_UNIQUE` (`id` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8mb4;


-- -----------------------------------------------------
-- Table `nccs`.`PlayListFiles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlayListFiles` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlayListFiles` (
  `PlayListFIleId` INT(11) NOT NULL AUTO_INCREMENT,
  `FileID` INT(11) NULL DEFAULT NULL,
  `SequenceID` INT(11) NOT NULL,
  `SessionID` SMALLINT(6) UNSIGNED NOT NULL,
  `Duration` INT(11) NULL DEFAULT '0',
  `KeyIndex` INT(11) NULL DEFAULT NULL,
  `PlayerID` VARCHAR(50) NULL DEFAULT NULL,
  PRIMARY KEY (`PlayListFIleId`),
  UNIQUE INDEX `SessionIDSeq` (`SessionID` ASC, `SequenceID` ASC),
  INDEX `SessionID` (`SessionID` ASC),
  INDEX `FileID` (`FileID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlayerAlive`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlayerAlive` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlayerAlive` (
  `PlayerID` VARCHAR(50) NOT NULL,
  `RefreshTime` DATETIME NULL DEFAULT NULL,
  PRIMARY KEY (`PlayerID`),
  INDEX `PlayerActivePlayerID` (`PlayerID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlayerKeys`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlayerKeys` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlayerKeys` (
  `PlayerKeyId` INT(11) NOT NULL AUTO_INCREMENT,
  `PlayerID` VARCHAR(50) NOT NULL,
  `Index` INT(11) NOT NULL,
  `GroupDisable` TINYINT(4) NULL DEFAULT '0',
  `DevDisable` TINYINT(4) NULL DEFAULT '0',
  `GroupID` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`PlayerKeyId`),
  UNIQUE INDEX `PlayerIndex` (`PlayerID` ASC, `Index` ASC),
  INDEX `PlayerKey_PlayerID` (`PlayerID` ASC),
  INDEX `PlayerGroupID` (`GroupID` ASC),
  CONSTRAINT `PlayerGroupID`
    FOREIGN KEY (`GroupID`)
    REFERENCES `nccs`.`DevClasses` (`ClassID`)
    ON DELETE SET NULL
    ON UPDATE SET NULL,
  CONSTRAINT `PlayerKey_PlayerID`
    FOREIGN KEY (`PlayerID`)
    REFERENCES `nccs`.`Players` (`PlayerID`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlayerStatus`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlayerStatus` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlayerStatus` (
  `PlayerID` VARCHAR(50) NOT NULL,
  `SessionID` SMALLINT(5) UNSIGNED NULL DEFAULT '0',
  `ScheduleID` INT(11) NULL DEFAULT NULL,
  `IpAddress` VARCHAR(50) NULL DEFAULT NULL,
  `NetID` TINYINT(4) NULL DEFAULT NULL,
  `ActualSesID` INT(11) NULL DEFAULT NULL,
  `PlayPort` INT(11) NULL DEFAULT NULL,
  `PlayType` TINYINT(4) NULL DEFAULT NULL,
  PRIMARY KEY (`PlayerID`),
  INDEX `PlayerStatusPlayerID` (`PlayerID` ASC),
  INDEX `PlayerStatusSessionID` (`SessionID` ASC),
  INDEX `PlayerStatusScheduleID` (`ScheduleID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PlayingSessions`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PlayingSessions` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PlayingSessions` (
  `SessionID` SMALLINT(6) UNSIGNED NOT NULL,
  `PlayerID` VARCHAR(50) NULL DEFAULT NULL,
  `PlayingFileID` INT(11) NULL DEFAULT NULL,
  `MultiPort` INT(11) NULL DEFAULT NULL,
  `CircleLength` INT(11) NULL DEFAULT NULL,
  `CurrFileProgress` INT(11) NULL DEFAULT NULL,
  `CurrTotalProgress` INT(11) NULL DEFAULT NULL,
  `Priority` TINYINT(4) NULL DEFAULT NULL,
  `Pause` TINYINT(4) NOT NULL DEFAULT '0',
  `ScheduleID` INT(11) NULL DEFAULT NULL,
  `BrdMode` INT(11) NULL DEFAULT NULL,
  `SesType` TINYINT(4) NULL DEFAULT '0',
  `StreamType` TINYINT(4) UNSIGNED NULL DEFAULT '1',
  `PlayBeginTime` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`SessionID`),
  UNIQUE INDEX `SessionID_UNIQUE` (`SessionID` ASC),
  UNIQUE INDEX `PlayerID_UNIQUE` (`PlayerID` ASC),
  INDEX `Session` (`SessionID` ASC))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`ProjectorType`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`ProjectorType` ;

CREATE TABLE IF NOT EXISTS `nccs`.`ProjectorType` (
  `Type` VARCHAR(50) NOT NULL,
  `PoweroffDelay` INT(11) NOT NULL,
  `CheckBit` INT(11) NOT NULL,
  `StopBit` INT(11) NOT NULL,
  `DataBit` INT(11) NOT NULL,
  `BaudRate` INT(11) NOT NULL,
  `CmdOpen` VARCHAR(255) NOT NULL,
  `CmdClose` VARCHAR(255) NOT NULL,
  `CmdFromPc` VARCHAR(255) NOT NULL,
  `CmdFromVideo` VARCHAR(255) NOT NULL,
  PRIMARY KEY (`Type`),
  UNIQUE INDEX `Type_UNIQUE` (`Type` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`PwrDisplaySet`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`PwrDisplaySet` ;

CREATE TABLE IF NOT EXISTS `nccs`.`PwrDisplaySet` (
  `PwrID` TINYINT(4) NOT NULL,
  `Name` VARCHAR(45) NULL DEFAULT NULL,
  `Type` TINYINT(4) NULL DEFAULT NULL,
  `Icon` VARCHAR(64) NULL DEFAULT NULL,
  PRIMARY KEY (`PwrID`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Scene`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Scene` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Scene` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(45) NOT NULL DEFAULT '' COMMENT '名称',
  `icon` VARCHAR(45) NOT NULL DEFAULT '' COMMENT '图标',
  `control_list` VARCHAR(2048) NOT NULL DEFAULT '{}' COMMENT '操作清单',
  `product_match_preg` VARCHAR(128) NOT NULL DEFAULT '' COMMENT '产品匹配列表',
  PRIMARY KEY (`id`))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = '设备场景表';


-- -----------------------------------------------------
-- Table `nccs`.`SectionTable`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SectionTable` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SectionTable` (
  `zone` VARCHAR(32) NOT NULL,
  `name` VARCHAR(32) NOT NULL DEFAULT '0' COMMENT '仅填写数字，表示第几节课。\n为保证兼容，未做类型变更',
  `alias` VARCHAR(32) NOT NULL COMMENT '节次别名，如第一节',
  `start` TIME NOT NULL DEFAULT '00:00:00',
  `end` TIME NOT NULL DEFAULT '23:59:59',
  UNIQUE INDEX `SectionTableUnique` (`zone` ASC, `name` ASC),
  INDEX `SectionTableIndex` (`name` ASC, `zone` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SessionCtrlList`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SessionCtrlList` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SessionCtrlList` (
  `id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `SessionID` SMALLINT(6) NOT NULL,
  `SubID` INT(11) NOT NULL,
  `State` INT(11) NULL DEFAULT NULL,
  `Action` INT(11) NULL DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `SesSubID_UNIQUE` USING BTREE (`SessionID` ASC, `SubID` ASC),
  INDEX `SessionID` USING BTREE (`SessionID` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SessionDevice`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SessionDevice` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SessionDevice` (
  `SessionDevID` INT(11) NOT NULL AUTO_INCREMENT,
  `DeviceID` VARCHAR(50) NOT NULL,
  `SessionID` SMALLINT(5) UNSIGNED NOT NULL,
  `KeyIndex` INT(11) NOT NULL DEFAULT '0',
  `DevDisabled` TINYINT(4) NULL DEFAULT '0',
  PRIMARY KEY (`SessionDevID`),
  UNIQUE INDEX `SessionDevKey` (`SessionID` ASC, `DeviceID` ASC, `KeyIndex` ASC),
  INDEX `SessionDevDev` (`DeviceID` ASC),
  INDEX `SessionDevSes` (`SessionID` ASC),
  CONSTRAINT `SessionDevDev`
    FOREIGN KEY (`DeviceID`)
    REFERENCES `nccs`.`DeviceList` (`DevID`)
    ON DELETE CASCADE,
  CONSTRAINT `SessionDevSes`
    FOREIGN KEY (`SessionID`)
    REFERENCES `nccs`.`PlaySessions` (`SessionID`)
    ON DELETE CASCADE)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SrsServer`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SrsServer` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SrsServer` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(32) NOT NULL,
  `server` VARCHAR(128) NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `server_uni` (`server` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SrsServerStatus`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SrsServerStatus` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SrsServerStatus` (
  `id` INT(11) NOT NULL,
  `server` VARCHAR(128) NOT NULL,
  `count` INT(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SubNetwork`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SubNetwork` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SubNetwork` (
  `idSubNetwork` INT(11) NOT NULL,
  `NetMask` VARCHAR(50) NULL DEFAULT NULL,
  `IpAddress` VARCHAR(50) NULL DEFAULT NULL,
  PRIMARY KEY (`idSubNetwork`),
  UNIQUE INDEX `idSubNetwork_UNIQUE` (`idSubNetwork` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`SystemLog`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`SystemLog` ;

CREATE TABLE IF NOT EXISTS `nccs`.`SystemLog` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `at` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `ip` VARCHAR(16) NULL DEFAULT NULL,
  `result` INT(11) NOT NULL,
  `username` VARCHAR(255) NULL DEFAULT NULL,
  `title1` VARCHAR(255) NULL DEFAULT NULL,
  `title2` VARCHAR(255) NULL DEFAULT NULL,
  `type` VARCHAR(64) NOT NULL COMMENT 'controller name',
  `action` VARCHAR(64) NOT NULL COMMENT 'action name',
  `detail` VARCHAR(1024) NULL DEFAULT NULL,
  `parameter_detail` VARCHAR(4096) NULL DEFAULT NULL,
  PRIMARY KEY (`id`),
  INDEX `systemlog_idx` (`username` ASC, `type` ASC, `ip` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`Timetable`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`Timetable` ;

CREATE TABLE IF NOT EXISTS `nccs`.`Timetable` (
  `Id` INT(11) NOT NULL AUTO_INCREMENT,
  `DevId` VARCHAR(50) NOT NULL,
  `UserId` INT(11) NOT NULL,
  `WeekDay` TINYINT(4) NOT NULL,
  `Start` DATETIME NOT NULL DEFAULT '9999-12-31 23:59:59',
  `End` DATETIME NOT NULL DEFAULT '0000-01-01 00:00:00',
  `Type` ENUM('normal', 'temporary') NOT NULL DEFAULT 'normal' COMMENT 'normal:正课\ntemporary:临课',
  `OneTime` TINYINT(4) NOT NULL DEFAULT '0',
  `Sync` TINYINT(4) NOT NULL DEFAULT '0',
  PRIMARY KEY (`Id`),
  UNIQUE INDEX `only` (`DevId` ASC, `UserId` ASC, `WeekDay` ASC, `Start` ASC, `End` ASC, `Type` ASC),
  INDEX `deviceId_idx` (`DevId` ASC),
  INDEX `table_idx` (`DevId` ASC, `UserId` ASC),
  INDEX `userIdKey_idx` (`UserId` ASC),
  CONSTRAINT `deviceIdKey`
    FOREIGN KEY (`DevId`)
    REFERENCES `nccs`.`DeviceList` (`DevID`)
    ON DELETE CASCADE
    ON UPDATE NO ACTION,
  CONSTRAINT `userIdKey`
    FOREIGN KEY (`UserId`)
    REFERENCES `nccs`.`Users` (`User_ID`)
    ON DELETE CASCADE
    ON UPDATE NO ACTION)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = '课表';


-- -----------------------------------------------------
-- Table `nccs`.`TimetableV1`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`TimetableV1` ;

CREATE TABLE IF NOT EXISTS `nccs`.`TimetableV1` (
  `ID` INT(11) NOT NULL AUTO_INCREMENT,
  `Dev_Name` VARCHAR(50) NOT NULL,
  `User_Flag` VARCHAR(20) NOT NULL DEFAULT '',
  `User_Name` VARCHAR(32) NOT NULL DEFAULT '' COMMENT '\"tmp_phone\"临时课需要填写用户姓名',
  `Zone` VARCHAR(32) NOT NULL DEFAULT '',
  `CourseName` VARCHAR(64) NOT NULL DEFAULT '',
  `TermName` VARCHAR(32) NOT NULL DEFAULT '',
  `SectionStart` INT(11) NOT NULL DEFAULT '0',
  `SectionEnd` INT(11) NOT NULL DEFAULT '0',
  `WeekStart` INT(11) NOT NULL DEFAULT '1',
  `WeekEnd` INT(11) NOT NULL DEFAULT '9999',
  `WeekDay` INT(11) NOT NULL DEFAULT '6',
  `WeekType` ENUM('single', 'double', 'all') NOT NULL DEFAULT 'all',
  `Active` ENUM('yes', 'no') NOT NULL DEFAULT 'yes' COMMENT '是否活动，活动状态的课表会用于设备控制',
  `Type` ENUM('normal', 'tmp_usersn', 'tmp_phone') NOT NULL DEFAULT 'normal' COMMENT 'normal: 正式课\ntmp_usersn:用户工号临时课\ntmp_phone:手机号临时课',
  PRIMARY KEY (`ID`),
  INDEX `TimetableV1_idx2` (`Dev_Name` ASC),
  INDEX `TimetableV1_idx1` (`User_Flag` ASC))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;

-- -----------------------------------------------------
-- Table `nccs`.`UserDevices`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`UserDevices` ;

CREATE TABLE IF NOT EXISTS `nccs`.`UserDevices` (
  `UserDevID` INT(11) NOT NULL AUTO_INCREMENT,
  `UserID` INT(11) NOT NULL,
  `DevID` VARCHAR(50) NOT NULL,
  `Pri` TINYINT(1) NOT NULL DEFAULT '0' COMMENT 'bit0: 用户可以使用的设备\nbit1: 用户可以管理的设备',
  PRIMARY KEY (`UserDevID`),
  UNIQUE INDEX `UserDev` (`UserID` ASC, `DevID` ASC),
  INDEX `user_dev` (`DevID` ASC),
  INDEX `UserId` (`UserID` ASC),
  CONSTRAINT `UserId`
    FOREIGN KEY (`UserID`)
    REFERENCES `nccs`.`Users` (`User_ID`)
    ON DELETE CASCADE
    ON UPDATE NO ACTION,
  CONSTRAINT `user_dev`
    FOREIGN KEY (`DevID`)
    REFERENCES `nccs`.`DeviceList` (`DevID`)
    ON DELETE CASCADE
    ON UPDATE NO ACTION)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`tmp_DelMediaFiles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`tmp_DelMediaFiles` ;

CREATE TABLE IF NOT EXISTS `nccs`.`tmp_DelMediaFiles` (
  `filepath` VARCHAR(256) NOT NULL,
  PRIMARY KEY (`filepath`))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`tmp_result`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`tmp_result` ;

CREATE TABLE IF NOT EXISTS `nccs`.`tmp_result` (
  `idtmp_result` INT(11) NULL DEFAULT NULL)
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


-- -----------------------------------------------------
-- Table `nccs`.`tmp_storage`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `nccs`.`tmp_storage` ;

CREATE TABLE IF NOT EXISTS `nccs`.`tmp_storage` (
  `key` VARCHAR(32) NOT NULL,
  `strv` VARCHAR(64) NULL DEFAULT NULL,
  `intv` INT(11) NULL DEFAULT NULL,
  `datev` DATETIME NULL DEFAULT NULL,
  `expire` INT(10) NULL DEFAULT NULL,
  PRIMARY KEY (`key`))
ENGINE = MEMORY
DEFAULT CHARACTER SET = utf8;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
